@@ -36,61 +36,106 @@
 #define _D_ABSTRACT_COMMAND_H_
 
 #include "Command.h"
+#include "SharedHandle.h"
 #include "TimeA2.h"
 #include "RequestGroupAware.h"
-#include "Socket.h"
+
+namespace aria2 {
 
 class Request;
-typedef SharedHandle<Request> RequestHandle;
 class DownloadEngine;
+class RequestGroup;
 class Segment;
-typedef SharedHandle<Segment> SegmentHandle;
-typedef deque<SegmentHandle> Segments;
-class NameResolver;
-typedef SharedHandle<NameResolver> NameResolverHandle;
-class BtProgressInfoFile;
-typedef SharedHandle<BtProgressInfoFile> BtProgressInfoFileHandle;
+class Exception;
+class SocketCore;
+#ifdef ENABLE_ASYNC_DNS
+class AsyncNameResolver;
+#endif // ENABLE_ASYNC_DNS
 
 class AbstractCommand : public Command, public RequestGroupAware {
 private:
   Time checkPoint;
-  int32_t timeout;
+  time_t timeout;
 protected:
-  RequestHandle req;
+  SharedHandle<Request> req;
   DownloadEngine* e;
-  SocketHandle socket;
-  Segments _segments;
+  SharedHandle<SocketCore> socket;
+  std::deque<SharedHandle<Segment> > _segments;
+
+#ifdef ENABLE_ASYNC_DNS
+  SharedHandle<AsyncNameResolver> _asyncNameResolver;
+
+  bool isAsyncNameResolverInitialized() const;
+
+  void initAsyncNameResolver(const std::string& hostname);
+
+  bool asyncResolveHostname();
+
+  const std::deque<std::string>& getResolvedAddresses();
+#endif // ENABLE_ASYNC_DNS
 
   void tryReserved();
-  virtual bool prepareForRetry(int32_t wait);
-  virtual void onAbort(Exception* ex);
+  virtual bool prepareForRetry(time_t wait);
+  virtual void onAbort();
   virtual bool executeInternal() = 0;
 
-  void setReadCheckSocket(const SocketHandle& socket);
-  void setWriteCheckSocket(const SocketHandle& socket);
+  void setReadCheckSocket(const SharedHandle<SocketCore>& socket);
+  void setWriteCheckSocket(const SharedHandle<SocketCore>& socket);
   void disableReadCheckSocket();
   void disableWriteCheckSocket();
-  bool resolveHostname(const string& hostname, const NameResolverHandle& nameResolver);
-#ifdef ENABLE_ASYNC_DNS
-  void setNameResolverCheck(const NameResolverHandle& resolver);
-  void disableNameResolverCheck(const NameResolverHandle& resolver);
-  virtual bool nameResolveFinished() const;
-#endif // ENABLE_ASYNC_DNS
-  void setTimeout(int32_t timeout) { this->timeout = timeout; }
+
+  /**
+   * If pred == true, calls setReadCheckSocket(socket). Otherwise, calls
+   * disableReadCheckSocket().
+   */
+  void setReadCheckSocketIf(const SharedHandle<SocketCore>& socket, bool pred);
+  /**
+   * If pred == true, calls setWriteCheckSocket(socket). Otherwise, calls
+   * disableWriteCheckSocket().
+   */
+  void setWriteCheckSocketIf(const SharedHandle<SocketCore>& socket, bool pred);
+
+  void setTimeout(time_t timeout) { this->timeout = timeout; }
 
   void prepareForNextAction(Command* nextCommand = 0);
 
+  void checkIfConnectionEstablished(const SharedHandle<SocketCore>& socket);
+
+  /*
+   * Returns true if proxy for the procol indicated by Request::getProtocol()
+   * is defined. Otherwise, returns false.
+   */
+  bool isProxyDefined() const;
+
+  /*
+   * Creates Request object for proxy URI and returns it.
+   * If no valid proxy is defined, then returns SharedHandle<Request>().
+   */
+  SharedHandle<Request> createProxyRequest() const;
 private:
   bool checkSocketIsReadable;
   bool checkSocketIsWritable;
-  SocketHandle readCheckTarget;
-  SocketHandle writeCheckTarget;
+  SharedHandle<SocketCore> readCheckTarget;
+  SharedHandle<SocketCore> writeCheckTarget;
   bool nameResolverCheck;
 
+#ifdef ENABLE_ASYNC_DNS
+
+  void setNameResolverCheck(const SharedHandle<AsyncNameResolver>& resolver);
+
+  void disableNameResolverCheck(const SharedHandle<AsyncNameResolver>& resolver);
+  bool nameResolveFinished() const;
+
+#endif // ENABLE_ASYNC_DNS
 public:
-  AbstractCommand(int32_t cuid, const RequestHandle& req, RequestGroup* requestGroup, DownloadEngine* e, const SocketHandle& s = SocketHandle());
+  AbstractCommand(int32_t cuid, const SharedHandle<Request>& req,
+		  RequestGroup* requestGroup, DownloadEngine* e,
+		  const SharedHandle<SocketCore>& s = SharedHandle<SocketCore>());
+
   virtual ~AbstractCommand();
   bool execute();
 };
 
+} // namespace aria2
+
 #endif // _D_ABSTRACT_COMMAND_H_
