@@ -37,14 +37,29 @@
 #include "RequestGroup.h"
 #include "Request.h"
 #include "HttpConnection.h"
+#include "HttpRequest.h"
+#include "Segment.h"
+#include "Option.h"
 #include "prefs.h"
+#include "Socket.h"
+#include "CookieStorage.h"
+#include "AuthConfigFactory.h"
 
-AbstractProxyRequestCommand::AbstractProxyRequestCommand(int cuid,
-							 const RequestHandle& req,
-							 RequestGroup* requestGroup,
-							 DownloadEngine* e,
-							 const SocketHandle& s)
-  :AbstractCommand(cuid, req, requestGroup, e, s), httpConnection(0) {
+namespace aria2 {
+
+AbstractProxyRequestCommand::AbstractProxyRequestCommand
+(int cuid,
+ const RequestHandle& req,
+ RequestGroup* requestGroup,
+ DownloadEngine* e,
+ const SharedHandle<Request>& proxyRequest,
+ const SocketHandle& s)
+  :
+  AbstractCommand(cuid, req, requestGroup, e, s),
+  _proxyRequest(proxyRequest),
+  httpConnection(new HttpConnection(cuid, s, e->option))
+{
+  setTimeout(e->option->getAsInt(PREF_CONNECT_TIMEOUT));
   disableReadCheckSocket();
   setWriteCheckSocket(socket);
 }
@@ -52,17 +67,26 @@ AbstractProxyRequestCommand::AbstractProxyRequestCommand(int cuid,
 AbstractProxyRequestCommand::~AbstractProxyRequestCommand() {}
 
 bool AbstractProxyRequestCommand::executeInternal() {
-  socket->setBlockingMode();
+  //socket->setBlockingMode();
+  if(httpConnection->sendBufferIsEmpty()) {
+    HttpRequestHandle httpRequest(new HttpRequest());
+    httpRequest->setUserAgent(e->option->get(PREF_USER_AGENT));
+    httpRequest->setRequest(req);
+    httpRequest->setAuthConfigFactory(e->getAuthConfigFactory());
+    httpRequest->setProxyRequest(_proxyRequest);
 
-  HttpRequestHandle httpRequest = new HttpRequest();
-  httpRequest->setUserAgent(e->option->get(PREF_USER_AGENT));
-  httpRequest->setRequest(req);
-  httpRequest->configure(e->option);
-
-  httpConnection= new HttpConnection(cuid, socket, e->option);
-
-  httpConnection->sendProxyRequest(httpRequest);
-
-  e->commands.push_back(getNextCommand());
-  return true;
+    httpConnection->sendProxyRequest(httpRequest);
+  } else {
+    httpConnection->sendPendingData();
+  }
+  if(httpConnection->sendBufferIsEmpty()) {
+    e->commands.push_back(getNextCommand());
+    return true;
+  } else {
+    setWriteCheckSocket(socket);
+    e->commands.push_back(this);
+    return false;
+  }
 }
+
+} // namespace aria2
