@@ -36,28 +36,34 @@
 #define _D_ANNOUNCE_LIST_H_
 
 #include "common.h"
-#include "MetaEntry.h"
+#include "SharedHandle.h"
 #include "AnnounceTier.h"
 
+namespace aria2 {
+
+namespace bencode {
+class BDE;
+} // namespace bencode
+
 class AnnounceList {
 public:
 private:
-  AnnounceTiers tiers;
-  AnnounceTiers::iterator currentTier;
-  Strings::iterator currentTracker;
+  std::deque<SharedHandle<AnnounceTier> > tiers;
+  std::deque<SharedHandle<AnnounceTier> >::iterator currentTier;
+  std::deque<std::string>::iterator currentTracker;
   bool currentTrackerInitialized;
 
   void resetIterator();
-  void setCurrentTier(const AnnounceTiers::iterator& itr);
+  void setCurrentTier(const std::deque<SharedHandle<AnnounceTier> >::iterator& itr);
 public:
   AnnounceList():currentTrackerInitialized(false) {}
-  AnnounceList(const MetaEntry* announceListEntry);
-  AnnounceList(const AnnounceTiers& tiers);
+  AnnounceList(const bencode::BDE& announceList);
+  AnnounceList(const std::deque<SharedHandle<AnnounceTier> >& tiers);
 
-  void reconfigure(const MetaEntry* announceListEntry);
-  void reconfigure(const string& url);
+  void reconfigure(const bencode::BDE& announceList);
+  void reconfigure(const std::string& url);
 
-  int32_t countTier() const {
+  size_t countTier() const {
     return tiers.size();
   }
 
@@ -69,12 +75,12 @@ public:
   /**
    * Returns announce URL.
    */
-  string getAnnounce() const;
+  std::string getAnnounce() const;
 
   /**
    * Returns announce event, such as started, stopped, completed, etc.
    */
-  string getEventString() const;
+  std::string getEventString() const;
 
   AnnounceTier::AnnounceEvent getEvent() const;
 
@@ -98,12 +104,12 @@ public:
   /**
    * Counts the number of tiers to which the "stopped" event can be sent.
    */
-  int32_t countStoppedAllowedTier() const;
+  size_t countStoppedAllowedTier() const;
 
   /**
    * Counts the number of tiers to which the "completed" event can be sent.
    */
-  int32_t countCompletedAllowedTier() const;
+  size_t countCompletedAllowedTier() const;
 
   /**
    * Moves current tier pointer to the tier to which the "stopped" event can
@@ -123,6 +129,14 @@ public:
   bool currentTierAcceptsStoppedEvent() const;
 
   bool currentTierAcceptsCompletedEvent() const;
+
+  static const std::string STARTED;
+
+  static const std::string STOPPED;
+
+  static const std::string COMPLETED;
 };
 
+} // namespace aria2
+
 #endif // _D_ANNOUNCE_LIST_H_
