@@ -38,13 +38,15 @@
 #include "BtMessageValidator.h"
 #include "BtBitfieldMessage.h"
 
+namespace aria2 {
+
 class BtBitfieldMessageValidator : public BtMessageValidator {
 private:
   const BtBitfieldMessage* message;
-  int32_t numPiece;
+  size_t numPiece;
 public:
   BtBitfieldMessageValidator(const BtBitfieldMessage* message,
-			     int32_t numPiece):
+			     size_t numPiece):
     message(message),
     numPiece(numPiece) {}
 
@@ -58,4 +60,7 @@ public:
 };
 
 typedef SharedHandle<BtBitfieldMessageValidator> BtBitfieldMessageValidatorHandle;
+
+} // namespace aria2
+
 #endif // _D_BT_BITFIELD_MESSAGE_VALIDATOR_H_
