@@ -41,23 +41,40 @@
 #include "BtCancelSendingPieceEvent.h"
 #include "MessageDigestHelper.h"
 #include "DiskAdaptor.h"
+#include "Logger.h"
+#include "Peer.h"
+#include "Piece.h"
+#include "BtContext.h"
+#include "PieceStorage.h"
+#include "BtMessageDispatcher.h"
+#include "BtMessageFactory.h"
+#include "BtRequestFactory.h"
+#include "PeerConnection.h"
+#include "StringFormat.h"
+#include <cstring>
+#include <cstdlib>
+#include <cassert>
 
-void BtPieceMessage::setBlock(const unsigned char* block, int32_t blockLength) {
+namespace aria2 {
+
+void BtPieceMessage::setBlock(const unsigned char* block, size_t blockLength) {
   delete [] this->block;
   this->blockLength = blockLength;
   this->block = new unsigned char[this->blockLength];
   memcpy(this->block, block, this->blockLength);
 }
 
-BtPieceMessageHandle BtPieceMessage::create(const unsigned char* data, int32_t dataLength) {
+BtPieceMessageHandle BtPieceMessage::create(const unsigned char* data, size_t dataLength) {
   if(dataLength <= 9) {
-    throw new DlAbortEx(EX_INVALID_PAYLOAD_SIZE, "piece", dataLength, 9);
+    throw DlAbortEx
+      (StringFormat(EX_INVALID_PAYLOAD_SIZE, "piece", dataLength, 9).str());
   }
-  int8_t id = PeerMessageUtil::getId(data);
+  uint8_t id = PeerMessageUtil::getId(data);
   if(id != ID) {
-    throw new DlAbortEx(EX_INVALID_BT_MESSAGE_ID, id, "piece", ID);
+    throw DlAbortEx
+      (StringFormat(EX_INVALID_BT_MESSAGE_ID, id, "piece", ID).str());
   }
-  BtPieceMessageHandle message = new BtPieceMessage();
+  BtPieceMessageHandle message(new BtPieceMessage());
   message->setIndex(PeerMessageUtil::getIntParam(data, 1));
   message->setBegin(PeerMessageUtil::getIntParam(data, 5));
   message->setBlock(data+9, dataLength-9);
@@ -70,21 +87,18 @@ void BtPieceMessage::doReceivedAction() {
 						       blockLength);
   peer->updateDownloadLength(blockLength);
   if(!RequestSlot::isNull(slot)) {
-    peer->snubbing = false;
+    peer->snubbing(false);
     peer->updateLatency(slot.getLatencyInMillis());
     PieceHandle piece = pieceStorage->getPiece(index);
-    int64_t offset =
-      ((int64_t)index)*btContext->getPieceLength()+begin;
+    off_t offset = (off_t)index*btContext->getPieceLength()+begin;
     logger->debug(MSG_PIECE_RECEIVED,
 		  cuid, index, begin, blockLength, offset, slot.getBlockIndex());
-    pieceStorage->getDiskAdaptor()->writeData(block,
-					      blockLength,
-					      offset);
+    pieceStorage->getDiskAdaptor()->writeData(block, blockLength, offset);
     piece->completeBlock(slot.getBlockIndex());
-    logger->debug(MSG_PIECE_BITFIELD,
-		  cuid,
+    logger->debug(MSG_PIECE_BITFIELD, cuid,
 		  Util::toHex(piece->getBitfield(),
 			      piece->getBitfieldLength()).c_str());
+    piece->updateHash(begin, block, blockLength);
     dispatcher->removeOutstandingRequest(slot);
     if(piece->pieceComplete()) {
       if(checkPieceHash(piece)) {
@@ -93,10 +107,13 @@ void BtPieceMessage::doReceivedAction() {
 	onWrongPiece(piece);
       }
     }
+  } else {
+    logger->debug("CUID#%d - RequestSlot not found, index=%d, begin=%d",
+		  cuid, index, begin);
   }
 }
 
-int32_t BtPieceMessage::MESSAGE_HEADER_LENGTH = 13;
+size_t BtPieceMessage::MESSAGE_HEADER_LENGTH = 13;
 
 const unsigned char* BtPieceMessage::getMessageHeader() {
   if(!msgHeader) {
@@ -116,7 +133,7 @@ const unsigned char* BtPieceMessage::getMessageHeader() {
   return msgHeader;
 }
 
-int32_t BtPieceMessage::getMessageHeaderLength() {
+size_t BtPieceMessage::getMessageHeaderLength() {
   return MESSAGE_HEADER_LENGTH;
 }
 
@@ -124,77 +141,53 @@ void BtPieceMessage::send() {
   if(invalidate) {
     return;
   }
-  if(!headerSent) {
-    if(!sendingInProgress) {
-      logger->info(MSG_SEND_PEER_MESSAGE,
-		   cuid, peer->ipaddr.c_str(), peer->port,
-		   toString().c_str());
-      getMessageHeader();
-      leftDataLength = getMessageHeaderLength();
-      sendingInProgress = true;
-    }
-    int32_t writtenLength
-      = peerConnection->sendMessage(msgHeader+getMessageHeaderLength()-leftDataLength,
-				    leftDataLength);
-    if(writtenLength == leftDataLength) {
-      headerSent = true;
-      leftDataLength = blockLength;
-    } else {
-      leftDataLength -= writtenLength;
-    }
-  }
-  if(headerSent) {
-    sendingInProgress = false;
-    int64_t pieceDataOffset =
-      ((int64_t)index)*btContext->getPieceLength()+begin+blockLength-leftDataLength;
-    int32_t writtenLength =
-      sendPieceData(pieceDataOffset, leftDataLength);
+  if(!sendingInProgress) {
+    logger->info(MSG_SEND_PEER_MESSAGE,
+		 cuid, peer->ipaddr.c_str(), peer->port,
+		 toString().c_str());
+    getMessageHeader();
+    peerConnection->sendMessage(msgHeader, getMessageHeaderLength());
+    off_t pieceDataOffset =
+      (off_t)index*btContext->getPieceLength()+begin;
+    size_t writtenLength = sendPieceData(pieceDataOffset, blockLength);
+    logger->debug("msglength = %lu bytes",
+		  static_cast<unsigned long>(getMessageHeaderLength()+
+					     blockLength));
     peer->updateUploadLength(writtenLength);
-    if(writtenLength < leftDataLength) {
-      sendingInProgress = true;
-    }
-    leftDataLength -= writtenLength;
+  } else {
+    ssize_t writtenLength = peerConnection->sendPendingData();
+    peer->updateUploadLength(writtenLength);
+  }
+  sendingInProgress = !peerConnection->sendBufferIsEmpty();
+}
+
+size_t BtPieceMessage::sendPieceData(off_t offset, size_t length) const {
+  assert(length <= 16*1024);
+  unsigned char buf[16*1024];
+  if(pieceStorage->getDiskAdaptor()->readData(buf, length, offset) ==
+     static_cast<ssize_t>(length)) {
+    return peerConnection->sendMessage(buf, length);
+  } else {
+    throw DlAbortEx(EX_DATA_READ);
   }
 }
 
-int32_t BtPieceMessage::sendPieceData(int64_t offset, int32_t length) const {
-  int32_t BUF_SIZE = 256;
-  unsigned char buf[BUF_SIZE];
-  int32_t iteration = length/BUF_SIZE;
-  int32_t writtenLength = 0;
-  for(int32_t i = 0; i < iteration; i++) {
-    if(pieceStorage->getDiskAdaptor()->readData(buf, BUF_SIZE, offset+i*BUF_SIZE) < BUF_SIZE) {
-      throw new DlAbortEx(EX_DATA_READ);
-    }
-    int32_t ws = peerConnection->sendMessage(buf, BUF_SIZE);
-    writtenLength += ws;
-    if(ws != BUF_SIZE) {
-      return writtenLength;
-    }
-  }
-
-  int32_t rem = length%BUF_SIZE;
-  if(rem > 0) {
-    if(pieceStorage->getDiskAdaptor()->readData(buf, rem, offset+iteration*BUF_SIZE) < rem) {
-      throw new DlAbortEx(EX_DATA_READ);
-    }
-    int32_t ws = peerConnection->sendMessage(buf, rem);
-    writtenLength += ws;
-  }
-  return writtenLength;
-}
-
-string BtPieceMessage::toString() const {
+std::string BtPieceMessage::toString() const {
   return "piece index="+Util::itos(index)+", begin="+Util::itos(begin)+
     ", length="+Util::itos(blockLength);
 }
 
 bool BtPieceMessage::checkPieceHash(const PieceHandle& piece) {
-  int64_t offset =
-    ((int64_t)piece->getIndex())*btContext->getPieceLength();
-  
-  return MessageDigestHelper::digest("sha1", pieceStorage->getDiskAdaptor(), offset, piece->getLength())
-    == btContext->getPieceHash(piece->getIndex());
+  if(piece->isHashCalculated()) {
+    logger->debug("Hash is available!! index=%lu",
+		  static_cast<unsigned long>(piece->getIndex()));
+    return piece->getHashString() == btContext->getPieceHash(piece->getIndex());
+  } else {
+    off_t offset = (off_t)piece->getIndex()*btContext->getPieceLength();
+    
+    return MessageDigestHelper::staticSHA1Digest(pieceStorage->getDiskAdaptor(), offset, piece->getLength())
+      == btContext->getPieceHash(piece->getIndex());
+  }
 }
 
 void BtPieceMessage::onNewPiece(const PieceHandle& piece) {
@@ -207,22 +200,22 @@ void BtPieceMessage::onWrongPiece(const PieceHandle& piece) {
   logger->info(MSG_GOT_WRONG_PIECE, cuid, piece->getIndex());
   erasePieceOnDisk(piece);
   piece->clearAllBlock();
+  piece->destroyHashContext();
   requestFactory->removeTargetPiece(piece);
 }
 
 void BtPieceMessage::erasePieceOnDisk(const PieceHandle& piece) {
-  int32_t BUFSIZE = 4096;
+  size_t BUFSIZE = 4096;
   unsigned char buf[BUFSIZE];
   memset(buf, 0, BUFSIZE);
-  int64_t offset =
-    ((int64_t)piece->getIndex())*btContext->getPieceLength();
-  for(int32_t i = 0; i < piece->getLength()/BUFSIZE; i++) {
+  off_t offset = (off_t)piece->getIndex()*btContext->getPieceLength();
+  div_t res = div(piece->getLength(), BUFSIZE);
+  for(int i = 0; i < res.quot; i++) {
     pieceStorage->getDiskAdaptor()->writeData(buf, BUFSIZE, offset);
     offset += BUFSIZE;
   }
-  int32_t r = piece->getLength()%BUFSIZE;
-  if(r > 0) {
-    pieceStorage->getDiskAdaptor()->writeData(buf, r, offset);
+  if(res.rem > 0) {
+    pieceStorage->getDiskAdaptor()->writeData(buf, res.rem, offset);
   }
 }
 
@@ -282,3 +275,5 @@ void BtPieceMessage::handleCancelSendingPieceEvent(const BtEventHandle& event) {
     invalidate = true;
   } 
 }
+
+} // namespace aria2
