@@ -36,7 +36,12 @@
 #define _D_COMMAND_H_
 
 #include "common.h"
-#include "LogFactory.h"
+#include <stdint.h>
+#include <deque>
+
+namespace aria2 {
+
+class Logger;
 
 typedef int32_t CommandUuid;
 
@@ -46,7 +51,8 @@ public:
     STATUS_ALL,
     STATUS_INACTIVE,
     STATUS_ACTIVE,
-    STATUS_REALTIME
+    STATUS_REALTIME,
+    STATUS_ONESHOT_REALTIME
   };
 private:
   CommandUuid uuid;
@@ -54,15 +60,21 @@ private:
   STATUS status;
 protected:
   int32_t cuid;
-  const Logger* logger;
+  Logger* logger;
+
+  bool _readEvent;
+  bool _writeEvent;
+  bool _errorEvent;
+  bool _hupEvent;
 public:
-  Command(int32_t cuid):uuid(uuidGen++), status(STATUS_INACTIVE), cuid(cuid) {
-    logger = LogFactory::getInstance();
-  }
+  Command(int32_t cuid);
+
   virtual ~Command() {}
+
   virtual bool execute() = 0;
 
   int32_t getCuid() const { return cuid; }
+
   const CommandUuid& getUuid() const { return uuid; }
 
   void setStatusActive() { this->status = STATUS_ACTIVE; }
@@ -71,22 +83,28 @@ public:
 
   void setStatusRealtime() { this->status = STATUS_REALTIME; }
 
+  void setStatus(STATUS status);
+
   bool statusMatch(Command::STATUS statusFilter) const
   {
     return statusFilter <= status;
   }
 
-  void transitStatus()
-  {
-    switch(status) {
-    case STATUS_REALTIME:
-      break;
-    default:
-      status = STATUS_INACTIVE;
-    }
-  }
+  void transitStatus();
+
+  void readEventReceived();
+
+  void writeEventReceived();
+
+  void errorEventReceived();
+
+  void hupEventReceived();
+
+  void clearIOEvents();
 };
 
-typedef deque<Command*> Commands;
+typedef std::deque<Command*> Commands;
+
+} // namespace aria2
 
 #endif // _D_COMMAND_H_
