@@ -37,62 +37,77 @@
 
 #include "common.h"
 
+#include <string>
+#include <deque>
+
+#include "a2time.h"
+
+namespace aria2 {
+
 class Cookie {
+private:
+  std::string _name;
+  std::string _value;
+  time_t _expiry;
+  std::string _path;
+  std::string _domain;
+  bool _secure;
 public:
-  string name;
-  string value;
-  time_t expires;
-  string path;
-  string domain;
-  bool secure;
-  bool onetime; // if true, this cookie will expire when the user's session ends.
-public:
-  Cookie(const string& name,
-	 const string& value,
+  /*
+   * If expires = 0 is given, then the cookie becomes session cookie.
+   */
+  Cookie(const std::string& name,
+	 const std::string& value,
 	 time_t  expires,
-	 const string& path,
-	 const string& domain,
-	 bool secure):
-    name(name),
-    value(value),
-    expires(expires),
-    path(path),
-    domain(domain),
-    secure(secure),
-    onetime(false) {}
+	 const std::string& path,
+	 const std::string& domain,
+	 bool secure);
 
-  Cookie(const string& name,
-	 const string& value,
-	 const string& path,
-	 const string& domain,
-	 bool secure):
-    name(name),
-    value(value),
-    path(path),
-    domain(domain),
-    secure(secure),
-    onetime(true) {}
+  /*
+   * Creates session cookie. This is equivalent to
+   * Cookie(name, value, 0, path, domain, secure);
+   */
+  Cookie(const std::string& name,
+	 const std::string& value,
+	 const std::string& path,
+	 const std::string& domain,
+	 bool secure);
 
-  Cookie():expires(0), secure(false), onetime(true) {}
+  Cookie();
 
-  ~Cookie() {}
-  string toString() const {
-    return name+"="+value;
-  }
-  void clear() {
-    name = value = path = domain = "";
-    expires = 0;
-    secure = false;
-  }
+  ~Cookie();
 
-  bool good() const
-  {
-    return !name.empty();
-  }
+  std::string toString() const;
 
-  bool match(const string& host, const string& dir, time_t date, bool secure) const;
+  bool good() const;
+
+  bool match(const std::string& requestHost, const std::string& requestPath,
+	     time_t date, bool secure) const;
+
+  bool validate(const std::string& requestHost,
+		const std::string& requestPath) const;
+
+  bool operator==(const Cookie& cookie) const;
+
+  bool isExpired() const;
+
+  const std::string& getName() const;
+
+  const std::string& getValue() const;
+
+  const std::string& getPath() const;
+
+  const std::string& getDomain() const;
+
+  time_t getExpiry() const;
+
+  bool isSecureCookie() const;
+
+  bool isSessionCookie() const;
 };
 
-typedef deque<Cookie> Cookies;
+typedef std::deque<Cookie> Cookies;
+
+} // namespace aria2
 
 #endif // _D_COOKIE_H_
