@@ -33,36 +33,45 @@
  */
 /* copyright --> */
 #include "DefaultPeerListProcessor.h"
-#include "List.h"
-#include "Dictionary.h"
-#include "Data.h"
+#include "Peer.h"
+#include "bencode.h"
 
-bool DefaultPeerListProcessor::canHandle(const MetaEntry* peersEntry) const {
-  const List* peersList = dynamic_cast<const List*>(peersEntry);
-  return peersList != 0;
+namespace aria2 {
+
+const std::string DefaultPeerListProcessor::IP("ip");
+
+const std::string DefaultPeerListProcessor::PORT("port");
+
+DefaultPeerListProcessor::DefaultPeerListProcessor() {}
+
+DefaultPeerListProcessor::~DefaultPeerListProcessor() {}
+
+bool DefaultPeerListProcessor::canHandle(const bencode::BDE& peerData) const
+{
+  return peerData.isList();
 }
 
-Peers DefaultPeerListProcessor::extractPeer(const MetaEntry* peersEntry) {
-  Peers peers;
-  const List* peersList = dynamic_cast<const List*>(peersEntry);
-  if(!peersList) {
-    return peers;
+void DefaultPeerListProcessor::extractPeer
+(std::deque<SharedHandle<Peer> >& peers, const bencode::BDE& peerData)
+{
+  if(!canHandle(peerData)) {
+    return;
   }
-  const MetaList& metaList = peersList->getList();
-  for(MetaList::const_iterator itr = metaList.begin();
-      itr != metaList.end(); itr++) {
-    const Dictionary* peerDic = dynamic_cast<const Dictionary*>(*itr);
-    if(!peerDic) {
-      break;
-    }
-    const Data* ip = dynamic_cast<const Data*>(peerDic->get("ip"));
-    const Data* port = dynamic_cast<const Data*>(peerDic->get("port"));
-    if(!ip || !port || !port->isNumber()) {
+  for(bencode::BDE::List::const_iterator itr = peerData.listBegin();
+      itr != peerData.listEnd(); ++itr) {
+    const bencode::BDE& peerDict = *itr;
+    if(!peerDict.isDict()) {
       continue;
     }
-    PeerHandle peer = new Peer(ip->toString(), port->toInt(), pieceLength,
-			       totalLength);
+    const bencode::BDE& ip = peerDict[IP];
+    const bencode::BDE& port = peerDict[PORT];
+    if(!ip.isString() || !port.isInteger() ||
+       !(0 < port.i() && port.i() < 65536)) {
+      continue;
+    }
+    PeerHandle peer(new Peer(ip.s(), port.i()));
     peers.push_back(peer);
   }
-  return peers;
 }
+
+} // namespace aria2
