@@ -37,20 +37,24 @@
 
 #include "PeerListProcessor.h"
 
+namespace aria2 {
+
 class DefaultPeerListProcessor : public PeerListProcessor {
 private:
-  int32_t pieceLength;
-  int64_t totalLength;
+  static const std::string IP;
+
+  static const std::string PORT;
 public:
-  DefaultPeerListProcessor(int32_t pieceLength, int64_t totalLength)
-  :pieceLength(pieceLength),
-   totalLength(totalLength) {}
+  DefaultPeerListProcessor();
 
-  virtual ~DefaultPeerListProcessor() {}
+  virtual ~DefaultPeerListProcessor();
 
-  virtual Peers extractPeer(const MetaEntry* peersEntry);
+  virtual void extractPeer
+  (std::deque<SharedHandle<Peer> >& peers, const bencode::BDE& peerData);
 
-  virtual bool canHandle(const MetaEntry* peersEntry) const;
+  virtual bool canHandle(const bencode::BDE& peerData) const;
 };
 
+} // namespace aria2
+
 #endif // _D_DEFAULT_PEER_LIST_PROCESSOR_H_
