@@ -36,65 +36,78 @@
 #define _D_DEFAULT_PEER_STORAGE_H_
 
 #include "PeerStorage.h"
-#include "BtContext.h"
-#include "Option.h"
-#include "Logger.h"
-#include "BtRuntime.h"
 
-#define MAX_PEER_LIST_SIZE 60
-#define MAX_PEER_ERROR 5
+namespace aria2 {
+
+class BtContext;
+class Option;
+class Logger;
+class BtRuntime;
+class BtSeederStateChoke;
+class BtLeecherStateChoke;
+class PieceStorage;
 
 class DefaultPeerStorage : public PeerStorage {
 private:
-  BtContextHandle btContext;
+  SharedHandle<BtContext> btContext;
+  SharedHandle<BtRuntime> _btRuntime;
+  SharedHandle<PieceStorage> _pieceStorage;
   const Option* option;
-  Peers peers;
-  Peers incomingPeers;
-  int32_t maxPeerListSize;
+  std::deque<SharedHandle<Peer> > peers;
   Logger* logger;
-  BtRuntimeHandle btRuntime;
-  int64_t removedPeerSessionDownloadLength;
-  int64_t removedPeerSessionUploadLength;
+  size_t maxPeerListSize;
+  uint64_t removedPeerSessionDownloadLength;
+  uint64_t removedPeerSessionUploadLength;
+
+  BtSeederStateChoke* _seederStateChoke;
+  BtLeecherStateChoke* _leecherStateChoke;
+
+  bool isPeerAlreadyAdded(const SharedHandle<Peer>& peer);
 public:
-  DefaultPeerStorage(BtContextHandle btContext, const Option* option);
+  DefaultPeerStorage(const SharedHandle<BtContext>& btContext,
+		     const Option* option);
+
   virtual ~DefaultPeerStorage();
 
-  void setBtRuntime(const BtRuntimeHandle& btRuntime) {
-    this->btRuntime = btRuntime;
-  }
-  BtRuntimeHandle getBtRuntime() const { return btRuntime; }
+  virtual bool addPeer(const SharedHandle<Peer>& peer);
 
-  virtual bool addPeer(const PeerHandle& peer);
+  size_t countPeer() const;
 
-  virtual bool addIncomingPeer(const PeerHandle& peer);
+  virtual SharedHandle<Peer> getUnusedPeer();
 
-  int32_t countPeer() const;
+  SharedHandle<Peer> getPeer(const std::string& ipaddr, uint16_t port) const;
 
-  virtual PeerHandle getUnusedPeer();
+  virtual void addPeer(const std::deque<SharedHandle<Peer> >& peers);
 
-  PeerHandle getPeer(const string& ipaddr, int32_t port) const;
-
-  virtual void addPeer(const Peers& peers);
-
-  virtual const Peers& getPeers();
+  virtual const std::deque<SharedHandle<Peer> >& getPeers();
 
   virtual bool isPeerAvailable();
 
-  virtual Peers getActivePeers();
+  virtual void getActivePeers(std::deque<SharedHandle<Peer> >& peers);
 
   virtual TransferStat calculateStat();
 
-  virtual void returnPeer(const PeerHandle& peer);
+  virtual void returnPeer(const SharedHandle<Peer>& peer);
 
-  void setMaxPeerListSize(int32_t size) { this->maxPeerListSize = size; }
+  virtual bool chokeRoundIntervalElapsed();
+
+  virtual void executeChoke();
+
+  void setMaxPeerListSize(size_t size) { this->maxPeerListSize = size; }
  
-  int32_t getMaxPeerListSize() const { return maxPeerListSize; }
+  size_t getMaxPeerListSize() const { return maxPeerListSize; }
 
-  void deleteUnusedPeer(int32_t delSize);
+  void deleteUnusedPeer(size_t delSize);
   
-  void onErasingPeer(const PeerHandle& peer);
+  void onErasingPeer(const SharedHandle<Peer>& peer);
 
-  const Peers& getIncomingPeers() const { return incomingPeers; }
+  void onReturningPeer(const SharedHandle<Peer>& peer);
+
+  void setPieceStorage(const SharedHandle<PieceStorage>& pieceStorage);
+
+  void setBtRuntime(const SharedHandle<BtRuntime>& btRuntime);
 };
 
+} // namespace aria2
+
 #endif // _D_DEFAULT_PEER_STORAGE_H_
