@@ -33,36 +33,46 @@
  */
 /* copyright --> */
 #include "DefaultPieceStorage.h"
+
+#include <numeric>
+#include <algorithm>
+
 #include "DownloadContext.h"
 #include "Piece.h"
 #include "Peer.h"
 #include "LogFactory.h"
+#include "Logger.h"
 #include "prefs.h"
 #include "DirectDiskAdaptor.h"
 #include "MultiDiskAdaptor.h"
 #include "CopyDiskAdaptor.h"
 #include "DiskWriter.h"
 #include "BitfieldManFactory.h"
+#include "BitfieldMan.h"
 #include "message.h"
 #include "DefaultDiskWriterFactory.h"
+#include "FileEntry.h"
 #include "DlAbortEx.h"
 #include "Util.h"
 #include "a2functional.h"
-#include <numeric>
+#include "Option.h"
+#include "StringFormat.h"
+#include "RarestPieceSelector.h"
 
-DefaultPieceStorage::DefaultPieceStorage(const DownloadContextHandle& downloadContext, const Option* option):
+namespace aria2 {
+
+DefaultPieceStorage::DefaultPieceStorage(const DownloadContextHandle& downloadContext, const Option* option, bool randomPieceStatsOrdering):
   downloadContext(downloadContext),
-  diskAdaptor(0),
+  bitfieldMan(BitfieldManFactory::getFactoryInstance()->
+	      createBitfieldMan(downloadContext->getPieceLength(),
+				downloadContext->getTotalLength())),
   _diskWriterFactory(new DefaultDiskWriterFactory()),
   endGamePieceNum(END_GAME_PIECE_NUM),
-  option(option)
-{
-  bitfieldMan =
-    BitfieldManFactory::getFactoryInstance()->
-    createBitfieldMan(downloadContext->getPieceLength(),
-		      downloadContext->getTotalLength());
-  logger = LogFactory::getInstance();
-}
+  logger(LogFactory::getInstance()),
+  option(option),
+  _pieceSelector(new RarestPieceSelector(downloadContext->getNumPieces(),
+					 randomPieceStatsOrdering))
+{}
 
 DefaultPieceStorage::~DefaultPieceStorage() {
   delete bitfieldMan;
@@ -79,29 +89,39 @@ bool DefaultPieceStorage::isEndGame()
   return bitfieldMan->countMissingBlock() <= endGamePieceNum;
 }
 
-int32_t DefaultPieceStorage::getMissingPieceIndex(const PeerHandle& peer)
+bool DefaultPieceStorage::getMissingPieceIndex(size_t& index,
+					       const unsigned char* bitfield,
+					       size_t& length)
 {
-  int32_t index = -1;
+  std::deque<size_t> indexes;
+  bool r;
   if(isEndGame()) {
-    index = bitfieldMan->getMissingIndex(peer->getBitfield(),
-					 peer->getBitfieldLength());
+    r = bitfieldMan->getAllMissingIndexes(indexes, bitfield, length);
   } else {
-    index = bitfieldMan->getMissingUnusedIndex(peer->getBitfield(),
-					       peer->getBitfieldLength());
+    r = bitfieldMan->getAllMissingUnusedIndexes(indexes, bitfield, length);
+  }
+  if(r) {
+    // We assume indexes is sorted using comparator less.
+    return _pieceSelector->select(index, indexes);
+  } else {
+    return false;
   }
-  return index;
 }
 
-PieceHandle DefaultPieceStorage::checkOutPiece(int32_t index)
+PieceHandle DefaultPieceStorage::checkOutPiece(size_t index)
 {
-  if(index == -1) {
-    return 0;
-  }
   bitfieldMan->setUseBit(index);
 
   PieceHandle piece = findUsedPiece(index);
   if(piece.isNull()) {
-    piece = new Piece(index, bitfieldMan->getBlockLength(index));
+    piece.reset(new Piece(index, bitfieldMan->getBlockLength(index)));
+
+#ifdef ENABLE_MESSAGE_DIGEST
+
+    piece->setHashAlgo(downloadContext->getPieceHashAlgo());
+
+#endif // ENABLE_MESSAGE_DIGEST
+
     addUsedPiece(piece);
     return piece;
   } else {
@@ -113,124 +133,139 @@ PieceHandle DefaultPieceStorage::checkOutPiece(int32_t index)
  * Newly instantiated piece is not added to usedPieces.
  * Because it is waste of memory and there is no chance to use them later.
  */
-PieceHandle DefaultPieceStorage::getPiece(int32_t index)
+PieceHandle DefaultPieceStorage::getPiece(size_t index)
 {
+  PieceHandle piece;
   if(0 <= index && index <= bitfieldMan->getMaxIndex()) {
-    PieceHandle piece = findUsedPiece(index);
+    piece = findUsedPiece(index);
     if(piece.isNull()) {
-      piece = new Piece(index, bitfieldMan->getBlockLength(index));
+      piece.reset(new Piece(index, bitfieldMan->getBlockLength(index)));
       if(hasPiece(index)) {
 	piece->setAllBlock();
       }
     }
-    return piece;
-  } else {
-    return 0;
   }
+  return piece;
 }
 
 void DefaultPieceStorage::addUsedPiece(const PieceHandle& piece)
 {
-  usedPieces.push_back(piece);
+  std::deque<SharedHandle<Piece> >::iterator i =
+    std::lower_bound(usedPieces.begin(), usedPieces.end(), piece);
+  usedPieces.insert(i, piece);
+  logger->debug("usedPieces.size()=%lu",
+		static_cast<unsigned long>(usedPieces.size()));
 }
 
-class FindPiece {
-private:
-  int32_t index;
-public:
-  FindPiece(int32_t index):index(index) {}
-
-  bool operator()(const PieceHandle& piece) {
-    return piece->getIndex() == index;
-  }
-};
-
-PieceHandle DefaultPieceStorage::findUsedPiece(int32_t index) const
+PieceHandle DefaultPieceStorage::findUsedPiece(size_t index) const
 {
-  Pieces::const_iterator itr = find_if(usedPieces.begin(),
-				       usedPieces.end(),
-				       FindPiece(index));
-  if(itr == usedPieces.end()) {
-    return 0;
+  SharedHandle<Piece> p(new Piece());
+  p->setIndex(index);
+
+  std::deque<SharedHandle<Piece> >::const_iterator i =
+    std::lower_bound(usedPieces.begin(), usedPieces.end(), p);
+  if(i != usedPieces.end() && (*i) == p) {
+    return *i;
   } else {
-    return *itr;
+    p.reset(0);
+    return p;
   }
 }
 
-PieceHandle DefaultPieceStorage::getMissingPiece(const PeerHandle& peer)
+SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
+(const unsigned char* bitfield, size_t length)
 {
-  int32_t index = getMissingPieceIndex(peer);
-  return checkOutPiece(index);
+  size_t index;
+  if(getMissingPieceIndex(index, bitfield, length)) {
+    return checkOutPiece(index);
+  } else {
+    return SharedHandle<Piece>();
+  }
 }
 
-int32_t DefaultPieceStorage::getMissingFastPieceIndex(const PeerHandle& peer)
+SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
+(const BitfieldMan& bitfield)
+{
+  return getMissingPiece(bitfield.getBitfield(), bitfield.getBitfieldLength());
+}
+
+PieceHandle DefaultPieceStorage::getMissingPiece(const SharedHandle<Peer>& peer)
+{
+  return getMissingPiece(peer->getBitfield(), peer->getBitfieldLength());
+}
+
+void DefaultPieceStorage::createFastIndexBitfield
+(BitfieldMan& bitfield, const SharedHandle<Peer>& peer)
+{
+  for(std::deque<size_t>::const_iterator itr =
+	peer->getPeerAllowedIndexSet().begin();
+      itr != peer->getPeerAllowedIndexSet().end(); ++itr) {
+    if(!bitfieldMan->isBitSet(*itr) && peer->hasPiece(*itr)) {
+      bitfield.setBit(*itr);
+    }
+  }
+}
+
+PieceHandle DefaultPieceStorage::getMissingFastPiece
+(const SharedHandle<Peer>& peer)
 {
-  int32_t index = -1;
   if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
     BitfieldMan tempBitfield(bitfieldMan->getBlockLength(),
 			     bitfieldMan->getTotalLength());
-    for(Integers::const_iterator itr = peer->getPeerAllowedIndexSet().begin();
-	itr != peer->getPeerAllowedIndexSet().end(); itr++) {
-      if(!bitfieldMan->isBitSet(index) && peer->hasPiece(*itr)) {
-	tempBitfield.setBit(*itr);
-      }
-    }
-    if(isEndGame()) {
-      index = bitfieldMan->getMissingIndex(tempBitfield.getBitfield(),
-					   tempBitfield.getBitfieldLength());
-    } else {
-      index = bitfieldMan->getMissingUnusedIndex(tempBitfield.getBitfield(),
-						 tempBitfield.getBitfieldLength());
-    }
+    createFastIndexBitfield(tempBitfield, peer);
+    return getMissingPiece(tempBitfield);
+  } else {
+    return SharedHandle<Piece>();
   }
-  return index;
 }
 
-PieceHandle DefaultPieceStorage::getMissingFastPiece(const PeerHandle& peer)
+static void unsetExcludedIndexes(BitfieldMan& bitfield,
+				 const std::deque<size_t>& excludedIndexes)
 {
-  int32_t index = getMissingFastPieceIndex(peer);
-  return checkOutPiece(index);
+  for(std::deque<size_t>::const_iterator i = excludedIndexes.begin();
+      i != excludedIndexes.end(); ++i) {
+    bitfield.unsetBit(*i);
+  }
+}
+
+SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
+(const SharedHandle<Peer>& peer, const std::deque<size_t>& excludedIndexes)
+{
+  BitfieldMan tempBitfield(bitfieldMan->getBlockLength(),
+			   bitfieldMan->getTotalLength());
+  tempBitfield.setBitfield(peer->getBitfield(), peer->getBitfieldLength());
+  unsetExcludedIndexes(tempBitfield, excludedIndexes);
+  return getMissingPiece(tempBitfield);
+}
+
+SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
+(const SharedHandle<Peer>& peer, const std::deque<size_t>& excludedIndexes)
+{
+  if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
+    BitfieldMan tempBitfield(bitfieldMan->getBlockLength(),
+			     bitfieldMan->getTotalLength());
+    createFastIndexBitfield(tempBitfield, peer);
+    unsetExcludedIndexes(tempBitfield, excludedIndexes);
+    return getMissingPiece(tempBitfield);
+  } else {
+    return SharedHandle<Piece>();
+  }
 }
 
 PieceHandle DefaultPieceStorage::getMissingPiece()
 {
-  return checkOutPiece(bitfieldMan->getSparseMissingUnusedIndex());
+  size_t index;
+  if(bitfieldMan->getSparseMissingUnusedIndex(index)) {
+    return checkOutPiece(index);
+  } else {
+    return SharedHandle<Piece>();
+  }
 }
 
-PieceHandle DefaultPieceStorage::getMissingPiece(const FileEntryHandle& fileEntry)
-{
-  BitfieldMan temp(*bitfieldMan);
-  temp.clearFilter();
-  temp.addFilter(fileEntry->getOffset(), fileEntry->getLength());
-  temp.enableFilter();
-
-  int32_t firstPieceIndex = START_INDEX(fileEntry->getOffset(), downloadContext->getPieceLength());
-  int32_t endPieceIndex = END_INDEX(fileEntry->getOffset(), fileEntry->getLength(), downloadContext->getPieceLength());
-  if(!temp.isBitSet(firstPieceIndex) && !temp.isUseBitSet(firstPieceIndex)) {
-    PieceHandle piece = findUsedPiece(firstPieceIndex);
-    if(!piece.isNull()) {
-      if(piece->isRangeComplete(fileEntry->getOffset()-firstPieceIndex*downloadContext->getPieceLength(),
-				fileEntry->getLength() > downloadContext->getPieceLength() ?
-				downloadContext->getPieceLength():fileEntry->getLength())) {
-	temp.setBit(firstPieceIndex);
-      }
-    }
-  }
-  if(firstPieceIndex != endPieceIndex && !temp.isBitSet(endPieceIndex) && !temp.isUseBitSet(endPieceIndex)) {
-    PieceHandle piece = findUsedPiece(endPieceIndex);
-    if(!piece.isNull()) {
-      if(piece->isRangeComplete(0, fileEntry->getOffset()+fileEntry->getLength()-endPieceIndex*downloadContext->getPieceLength())) {
-	temp.setBit(endPieceIndex);
-      }
-    }    
-  }
-  return checkOutPiece(temp.getSparseMissingUnusedIndex());
-}
-
-PieceHandle DefaultPieceStorage::getMissingPiece(int32_t index)
+PieceHandle DefaultPieceStorage::getMissingPiece(size_t index)
 {
   if(hasPiece(index) || isPieceUsed(index)) {
-    return 0;
+    return SharedHandle<Piece>();
   } else {
     return checkOutPiece(index);
   }
@@ -241,50 +276,48 @@ void DefaultPieceStorage::deleteUsedPiece(const PieceHandle& piece)
   if(piece.isNull()) {
     return;
   }
-  Pieces::iterator itr = find(usedPieces.begin(), usedPieces.end(), piece);
-  if(itr != usedPieces.end()) {
-    usedPieces.erase(itr);
+  std::deque<SharedHandle<Piece> >::iterator i = 
+    std::lower_bound(usedPieces.begin(), usedPieces.end(), piece);
+  if(i != usedPieces.end() && (*i) == piece) {
+    usedPieces.erase(i);
   }
 }
 
-void DefaultPieceStorage::reduceUsedPieces(int32_t delMax)
-{
-  int32_t toDelete = usedPieces.size()-delMax;
-  if(toDelete <= 0) {
-    return;
-  }
-  int32_t fillRate = 10;
-  while(fillRate < 50) {
-    int32_t deleted = deleteUsedPiecesByFillRate(fillRate, toDelete);
-    if(deleted == 0) {
-      break;
-    }
-    toDelete -= deleted;
-    fillRate += 10;
-  }
-}
+// void DefaultPieceStorage::reduceUsedPieces(size_t upperBound)
+// {
+//   size_t usedPiecesSize = usedPieces.size();
+//   if(usedPiecesSize <= upperBound) {
+//     return;
+//   }
+//   size_t delNum = usedPiecesSize-upperBound;
+//   int fillRate = 10;
+//   while(delNum && fillRate <= 15) {
+//     delNum -= deleteUsedPiecesByFillRate(fillRate, delNum);
+//     fillRate += 5;
+//   }
+// }
 
-int32_t DefaultPieceStorage::deleteUsedPiecesByFillRate(int32_t fillRate,
-							int32_t toDelete)
-{
-  int32_t deleted = 0;
-  for(Pieces::iterator itr = usedPieces.begin();
-      itr != usedPieces.end() && deleted < toDelete;) {
-    PieceHandle& piece = *itr;
-    if(!bitfieldMan->isUseBitSet(piece->getIndex()) &&
-       piece->countCompleteBlock() <= piece->countBlock()*(fillRate/100.0)) {
-      logger->debug(MSG_DELETING_USED_PIECE,
-		    piece->getIndex(),
-		    (piece->countCompleteBlock()*100)/piece->countBlock(),
-		    fillRate);
-      itr = usedPieces.erase(itr);
-      deleted++;
-    } else {
-      itr++;
-    }
-  }
-  return deleted;
-}
+// size_t DefaultPieceStorage::deleteUsedPiecesByFillRate(int fillRate,
+// 						       size_t delNum)
+// {
+//   size_t deleted = 0;
+//   for(Pieces::iterator itr = usedPieces.begin();
+//       itr != usedPieces.end() && deleted < delNum;) {
+//     PieceHandle& piece = *itr;
+//     if(!bitfieldMan->isUseBitSet(piece->getIndex()) &&
+//        piece->countCompleteBlock() <= piece->countBlock()*(fillRate/100.0)) {
+//       logger->info(MSG_DELETING_USED_PIECE,
+// 		    piece->getIndex(),
+// 		    (piece->countCompleteBlock()*100)/piece->countBlock(),
+// 		    fillRate);
+//       itr = usedPieces.erase(itr);
+//       ++deleted;
+//     } else {
+//       ++itr;
+//     }
+//   }
+//   return deleted;
+// }
 
 void DefaultPieceStorage::completePiece(const PieceHandle& piece)
 {
@@ -292,15 +325,17 @@ void DefaultPieceStorage::completePiece(const PieceHandle& piece)
     return;
   }
   deleteUsedPiece(piece);
-  if(!isEndGame()) {
-    reduceUsedPieces(100);
-  }
+//   if(!isEndGame()) {
+//     reduceUsedPieces(100);
+//   }
   if(allDownloadFinished()) {
     return;
   }
   bitfieldMan->setBit(piece->getIndex());
   bitfieldMan->unsetUseBit(piece->getIndex());
+  addPieceStats(piece->getIndex());
   if(downloadFinished()) {
+    downloadContext->resetDownloadStopTime();
     diskAdaptor->onDownloadComplete();
     if(isSelectiveDownloadingMode()) {
       logger->notice(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
@@ -338,52 +373,58 @@ void DefaultPieceStorage::cancelPiece(const PieceHandle& piece)
   }
 }
 
-bool DefaultPieceStorage::hasPiece(int32_t index)
+bool DefaultPieceStorage::hasPiece(size_t index)
 {
   return bitfieldMan->isBitSet(index);
 }
 
-bool DefaultPieceStorage::isPieceUsed(int32_t index)
+bool DefaultPieceStorage::isPieceUsed(size_t index)
 {
   return bitfieldMan->isUseBitSet(index);
 }
 
-int64_t DefaultPieceStorage::getTotalLength()
+uint64_t DefaultPieceStorage::getTotalLength()
 {
   return bitfieldMan->getTotalLength();
 }
 
-int64_t DefaultPieceStorage::getFilteredTotalLength()
+uint64_t DefaultPieceStorage::getFilteredTotalLength()
 {
   return bitfieldMan->getFilteredTotalLength();
 }
 
-int64_t DefaultPieceStorage::getCompletedLength()
+uint64_t DefaultPieceStorage::getCompletedLength()
 {
-  return bitfieldMan->getCompletedLength()+getInFlightPieceCompletedLength();
+  uint64_t completedLength =
+    bitfieldMan->getCompletedLength()+getInFlightPieceCompletedLength();
+  uint64_t totalLength = getTotalLength();
+  if(completedLength > totalLength) {
+    completedLength = totalLength;
+  }
+  return completedLength;
 }
 
-int64_t DefaultPieceStorage::getFilteredCompletedLength()
+uint64_t DefaultPieceStorage::getFilteredCompletedLength()
 {
   return bitfieldMan->getFilteredCompletedLength()+getInFlightPieceCompletedLength();
 }
 
-int32_t DefaultPieceStorage::getInFlightPieceCompletedLength() const
+size_t DefaultPieceStorage::getInFlightPieceCompletedLength() const
 {
-  return accumulate(usedPieces.begin(), usedPieces.end(), 0, adopt2nd(plus<int32_t>(), mem_fun_sh(&Piece::getCompletedLength)));
+  return std::accumulate(usedPieces.begin(), usedPieces.end(), 0, adopt2nd(std::plus<size_t>(), mem_fun_sh(&Piece::getCompletedLength)));
 }
 
 // not unittested
-void DefaultPieceStorage::setFileFilter(const Strings& filePaths)
+void DefaultPieceStorage::setFileFilter(const std::deque<std::string>& filePaths)
 {
   if(downloadContext->getFileMode() != DownloadContext::MULTI || filePaths.empty()) {
     return;
   }
   diskAdaptor->removeAllDownloadEntry();
-  for(Strings::const_iterator pitr = filePaths.begin();
+  for(std::deque<std::string>::const_iterator pitr = filePaths.begin();
       pitr != filePaths.end(); pitr++) {
     if(!diskAdaptor->addDownloadEntry(*pitr)) {
-      throw new DlAbortEx(EX_NO_SUCH_FILE_ENTRY, (*pitr).c_str());
+      throw DlAbortEx(StringFormat(EX_NO_SUCH_FILE_ENTRY, (*pitr).c_str()).str());
     }
     FileEntryHandle fileEntry = diskAdaptor->getFileEntryFromPath(*pitr);
     bitfieldMan->addFilter(fileEntry->getOffset(), fileEntry->getLength());
@@ -393,14 +434,14 @@ void DefaultPieceStorage::setFileFilter(const Strings& filePaths)
 
 void DefaultPieceStorage::setFileFilter(IntSequence seq)
 {
-  Integers fileIndexes = seq.flush();
+  std::deque<int32_t> fileIndexes = seq.flush();
   // TODO Is sorting necessary?
-  sort(fileIndexes.begin(), fileIndexes.end());
-  fileIndexes.erase(unique(fileIndexes.begin(), fileIndexes.end()), fileIndexes.end());
-  Strings filePaths;
+  std::sort(fileIndexes.begin(), fileIndexes.end());
+  fileIndexes.erase(std::unique(fileIndexes.begin(), fileIndexes.end()), fileIndexes.end());
+  std::deque<std::string> filePaths;
   const FileEntries& entries = diskAdaptor->getFileEntries();
-  for(int32_t i = 0; i < (int32_t)entries.size(); i++) {
-    if(find(fileIndexes.begin(), fileIndexes.end(), i+1) != fileIndexes.end()) {
+  for(size_t i = 0; i < entries.size(); i++) {
+    if(std::find(fileIndexes.begin(), fileIndexes.end(), i+1) != fileIndexes.end()) {
       logger->debug("index=%d is %s", i+1, entries[i]->getPath().c_str());
       filePaths.push_back(entries[i]->getPath());
     }
@@ -435,30 +476,29 @@ void DefaultPieceStorage::initStorage()
     logger->debug("Instantiating DirectDiskAdaptor");
     DiskWriterHandle writer = _diskWriterFactory->newDiskWriter();
     writer->setDirectIOAllowed(option->getAsBool(PREF_ENABLE_DIRECT_IO));
-    DirectDiskAdaptorHandle directDiskAdaptor = new DirectDiskAdaptor();
+    DirectDiskAdaptorHandle directDiskAdaptor(new DirectDiskAdaptor());
     directDiskAdaptor->setDiskWriter(writer);
     directDiskAdaptor->setTotalLength(downloadContext->getTotalLength());
     this->diskAdaptor = directDiskAdaptor;
   } else {
     // file mode == DownloadContext::MULTI
-    if(option->get(PREF_DIRECT_FILE_MAPPING) == V_TRUE) {
+    if(option->getAsBool(PREF_DIRECT_FILE_MAPPING)) {
       logger->debug("Instantiating MultiDiskAdaptor");
-      MultiDiskAdaptorHandle multiDiskAdaptor = new MultiDiskAdaptor();
+      MultiDiskAdaptorHandle multiDiskAdaptor(new MultiDiskAdaptor());
       multiDiskAdaptor->setDirectIOAllowed(option->getAsBool(PREF_ENABLE_DIRECT_IO));
       multiDiskAdaptor->setPieceLength(downloadContext->getPieceLength());
       multiDiskAdaptor->setTopDir(downloadContext->getName());
+      multiDiskAdaptor->setMaxOpenFiles(option->getAsInt(PREF_BT_MAX_OPEN_FILES));
       this->diskAdaptor = multiDiskAdaptor;
     } else {
       logger->debug("Instantiating CopyDiskAdaptor");
       DiskWriterHandle writer = _diskWriterFactory->newDiskWriter();
       writer->setDirectIOAllowed(option->getAsBool(PREF_ENABLE_DIRECT_IO));
-      CopyDiskAdaptorHandle copyDiskAdaptor = new CopyDiskAdaptor();
+      CopyDiskAdaptorHandle copyDiskAdaptor(new CopyDiskAdaptor());
       copyDiskAdaptor->setDiskWriter(writer);
       copyDiskAdaptor->setTempFilename(downloadContext->getName()+".a2tmp");
       copyDiskAdaptor->setTotalLength(downloadContext->getTotalLength());
-      if(downloadContext->getFileMode() == DownloadContext::MULTI) {
-	copyDiskAdaptor->setTopDir(downloadContext->getName());
-      }
+      copyDiskAdaptor->setTopDir(downloadContext->getName());
       this->diskAdaptor = copyDiskAdaptor;
     }
   }
@@ -467,12 +507,13 @@ void DefaultPieceStorage::initStorage()
 }
 
 void DefaultPieceStorage::setBitfield(const unsigned char* bitfield,
-				      int32_t bitfieldLength)
+				      size_t bitfieldLength)
 {
   bitfieldMan->setBitfield(bitfield, bitfieldLength);
+  addPieceStats(bitfield, bitfieldLength);
 }
-  
-int32_t DefaultPieceStorage::getBitfieldLength()
+
+size_t DefaultPieceStorage::getBitfieldLength()
 {
   return bitfieldMan->getBitfieldLength();
 }
@@ -486,21 +527,22 @@ DiskAdaptorHandle DefaultPieceStorage::getDiskAdaptor() {
   return diskAdaptor;
 }
 
-int32_t DefaultPieceStorage::getPieceLength(int32_t index)
+size_t DefaultPieceStorage::getPieceLength(size_t index)
 {
   return bitfieldMan->getBlockLength(index);
 }
 
-void DefaultPieceStorage::advertisePiece(int32_t cuid, int32_t index)
+void DefaultPieceStorage::advertisePiece(int32_t cuid, size_t index)
 {
   HaveEntry entry(cuid, index);
   haves.push_front(entry);
 }
 
-Integers DefaultPieceStorage::getAdvertisedPieceIndexes(int32_t myCuid,
-							const Time& lastCheckTime)
+void
+DefaultPieceStorage::getAdvertisedPieceIndexes(std::deque<size_t>& indexes,
+					       int32_t myCuid,
+					       const Time& lastCheckTime)
 {
-  Integers indexes;
   for(Haves::const_iterator itr = haves.begin(); itr != haves.end(); itr++) {
     const Haves::value_type& have = *itr;
     if(have.getCuid() == myCuid) {
@@ -511,15 +553,14 @@ Integers DefaultPieceStorage::getAdvertisedPieceIndexes(int32_t myCuid,
     }
     indexes.push_back(have.getIndex());
   }
-  return indexes;
 }
 
 class FindElapsedHave
 {
 private:
-  int32_t elapsed;
+  time_t elapsed;
 public:
-  FindElapsedHave(int32_t elapsed):elapsed(elapsed) {}
+  FindElapsedHave(time_t elapsed):elapsed(elapsed) {}
 
   bool operator()(const HaveEntry& have) {
     if(have.getRegisteredTime().elapsed(elapsed)) {
@@ -530,10 +571,10 @@ public:
   }
 };
   
-void DefaultPieceStorage::removeAdvertisedPiece(int32_t elapsed)
+void DefaultPieceStorage::removeAdvertisedPiece(time_t elapsed)
 {
   Haves::iterator itr =
-    find_if(haves.begin(), haves.end(), FindElapsedHave(elapsed));
+    std::find_if(haves.begin(), haves.end(), FindElapsedHave(elapsed));
   if(itr != haves.end()) {
     logger->debug(MSG_REMOVED_HAVE_ENTRY, haves.end()-itr);
     haves.erase(itr, haves.end());
@@ -545,47 +586,83 @@ void DefaultPieceStorage::markAllPiecesDone()
   bitfieldMan->setAllBit();
 }
 
-void DefaultPieceStorage::markPiecesDone(int64_t length)
+void DefaultPieceStorage::markPiecesDone(uint64_t length)
 {
   if(length == bitfieldMan->getTotalLength()) {
     bitfieldMan->setAllBit();
   } else {
-    int32_t numPiece = length/bitfieldMan->getBlockLength();
+    size_t numPiece = length/bitfieldMan->getBlockLength();
     if(numPiece > 0) {
       bitfieldMan->setBitRange(0, numPiece-1);
     }
-    int32_t r = (length%bitfieldMan->getBlockLength())/Piece::BLOCK_LENGTH;
+    size_t r = (length%bitfieldMan->getBlockLength())/Piece::BLOCK_LENGTH;
     if(r > 0) {
-      PieceHandle p = new Piece(numPiece, bitfieldMan->getBlockLength(numPiece));
-      for(int32_t i = 0; i < r; ++i) {
+      PieceHandle p(new Piece(numPiece, bitfieldMan->getBlockLength(numPiece)));
+      
+      for(size_t i = 0; i < r; ++i) {
 	p->completeBlock(i);
       }
+
+#ifdef ENABLE_MESSAGE_DIGEST
+
+      p->setHashAlgo(downloadContext->getPieceHashAlgo());
+
+#endif // ENABLE_MESSAGE_DIGEST
+
       addUsedPiece(p);
     }
   }
 }
 
-void DefaultPieceStorage::markPieceMissing(int32_t index)
+void DefaultPieceStorage::markPieceMissing(size_t index)
 {
   bitfieldMan->unsetBit(index);
 }
 
 void DefaultPieceStorage::addInFlightPiece(const Pieces& pieces)
 {
-  copy(pieces.begin(), pieces.end(), back_inserter(usedPieces));
+  usedPieces.insert(usedPieces.end(), pieces.begin(), pieces.end());
+  std::sort(usedPieces.begin(), usedPieces.end());
 }
 
-int32_t DefaultPieceStorage::countInFlightPiece()
+size_t DefaultPieceStorage::countInFlightPiece()
 {
   return usedPieces.size();
 }
 
-Pieces DefaultPieceStorage::getInFlightPieces()
+void DefaultPieceStorage::getInFlightPieces(std::deque<SharedHandle<Piece> >& pieces)
 {
-  return usedPieces;
+  pieces.insert(pieces.end(), usedPieces.begin(), usedPieces.end());
 }
 
 void DefaultPieceStorage::setDiskWriterFactory(const DiskWriterFactoryHandle& diskWriterFactory)
 {
   _diskWriterFactory = diskWriterFactory;
 }
+
+void DefaultPieceStorage::addPieceStats(const unsigned char* bitfield,
+					size_t bitfieldLength)
+{
+  _pieceSelector->addPieceStats(bitfield, bitfieldLength);
+}
+
+void DefaultPieceStorage::subtractPieceStats(const unsigned char* bitfield,
+					     size_t bitfieldLength)
+{
+  _pieceSelector->subtractPieceStats(bitfield, bitfieldLength);
+}
+
+void DefaultPieceStorage::updatePieceStats(const unsigned char* newBitfield,
+					   size_t newBitfieldLength,
+					   const unsigned char* oldBitfield)
+{
+  _pieceSelector->updatePieceStats(newBitfield, newBitfieldLength,
+				   oldBitfield);
+}
+
+void DefaultPieceStorage::addPieceStats(size_t index)
+{
+  _pieceSelector->addPieceStats(index);
+}
+
+} // namespace aria2
