@@ -36,41 +36,58 @@
 #define _D_DOWNLOAD_CONTEXT_H_
 
 #include "common.h"
-#include "FileEntry.h"
+
+#include <string>
+#include <deque>
+
+#include "SharedHandle.h"
+#include "Signature.h"
+#include "TimeA2.h"
+
+namespace aria2 {
+
+class FileEntry;
 
 class DownloadContext
 {
 protected:
-  string _dir;
+  std::string _dir;
 
+private:
+  Time _downloadStartTime;
+
+  Time _downloadStopTime;
+
+  SharedHandle<Signature> _signature;
 public:
-  DownloadContext():_dir(".") {}
-
-  virtual ~DownloadContext() {}
+  DownloadContext();
 
+  virtual ~DownloadContext();
 
   enum FILE_MODE {
     SINGLE,
     MULTI
   };
 
-  virtual string getPieceHash(int32_t index) const = 0;
+  virtual const std::string& getPieceHash(size_t index) const = 0;
   
-  virtual const Strings& getPieceHashes() const = 0;
+  virtual const std::deque<std::string>& getPieceHashes() const = 0;
 
-  virtual int64_t getTotalLength() const = 0;
+  virtual uint64_t getTotalLength() const = 0;
+
+  virtual bool knowsTotalLength() const = 0;
 
   virtual FILE_MODE getFileMode() const = 0;
 
-  virtual FileEntries getFileEntries() const = 0;
+  virtual std::deque<SharedHandle<FileEntry> > getFileEntries() const = 0;
 
-  virtual string getName() const = 0;
+  virtual const std::string& getName() const = 0;
   
-  virtual int32_t getPieceLength() const = 0;
+  virtual size_t getPieceLength() const = 0;
 
-  virtual int32_t getNumPieces() const = 0;
+  virtual size_t getNumPieces() const = 0;
 
-  virtual string getPieceHashAlgo() const = 0;
+  virtual const std::string& getPieceHashAlgo() const = 0;
 
   /**
    * Returns an actual file path.
@@ -80,20 +97,25 @@ public:
    * /tmp/downloads/aria2.bin), then returns its base dir path,
    * for example, "/tmp/downloads"
    */
-  virtual string getActualBasePath() const = 0;
+  virtual std::string getActualBasePath() const = 0;
 
-  string getDir() const
-  {
-    return _dir;
-  }
+  const std::string& getDir() const;
 
-  void setDir(const string& dir)
-  {
-    _dir = dir;
-  }
+  void setDir(const std::string& dir);
 
+  SharedHandle<Signature> getSignature() const;
+
+  void setSignature(const SharedHandle<Signature>& signature);
+
+  void resetDownloadStartTime();
+
+  void resetDownloadStopTime();
+
+  int64_t calculateSessionTime() const;
 };
 
 typedef SharedHandle<DownloadContext> DownloadContextHandle;
 
+} // namespace aria2
+
 #endif // _D_DOWNLOAD_CONTEXT_H_
