@@ -39,7 +39,9 @@
 #include "FileAllocationEntry.h"
 #include "FileAllocationCommand.h"
 #include "message.h"
-#include "CUIDCounter.h"
+#include "Logger.h"
+
+namespace aria2 {
 
 FileAllocationDispatcherCommand::FileAllocationDispatcherCommand(int32_t cuid, DownloadEngine* e):
   Command(cuid), _e(e)
@@ -54,10 +56,10 @@ bool FileAllocationDispatcherCommand::execute()
   if(_e->_requestGroupMan->downloadFinished() || _e->isHaltRequested()) {
     return true;
   }
-  if(!_e->_fileAllocationMan->isFileAllocationBeingExecuted() &&
-     _e->_fileAllocationMan->nextFileAllocationEntryExists()) {
+  if(_e->_fileAllocationMan->nextFileAllocationEntryExists() &&
+     !_e->_fileAllocationMan->isFileAllocationBeingExecuted()) {
     FileAllocationEntryHandle entry = _e->_fileAllocationMan->popNextFileAllocationEntry();
-    int32_t newCUID = CUIDCounterSingletonHolder::instance()->newID();
+    int32_t newCUID = _e->newCUID();
     logger->info(MSG_FILE_ALLOCATION_DISPATCH, newCUID);
     FileAllocationCommand* command =
       new FileAllocationCommand(newCUID,
@@ -65,10 +67,12 @@ bool FileAllocationDispatcherCommand::execute()
 				_e,
 				entry);
     _e->commands.push_back(command);
-    _e->noWait = true;
+    _e->setNoWait(true);
   }
 
-  _e->commands.push_back(this);
+  _e->addRoutineCommand(this);
 
   return false;
 }
+
+} // namespace aria2
