@@ -36,27 +36,29 @@
 #define _D_HTTP_RESPONSE_H_
 
 #include "common.h"
-#include "HttpRequest.h"
-#include "HttpHeader.h"
-#include "TransferEncoding.h"
-#include "LogFactory.h"
+
+#include <stdint.h>
+
+#include "SharedHandle.h"
+#include "TimeA2.h"
+
+namespace aria2 {
+
+class HttpRequest;
+class HttpHeader;
+class Logger;
+class Decoder;
 
 class HttpResponse {
 private:
   int32_t cuid;
-  int32_t status;
-  HttpRequestHandle httpRequest;
-  HttpHeaderHandle httpHeader;
-  const Logger* logger;
+  SharedHandle<HttpRequest> httpRequest;
+  SharedHandle<HttpHeader> httpHeader;
+  Logger* logger;
 public:
-  HttpResponse():cuid(0),
-		 status(0),
-		 httpRequest(0),
-		 httpHeader(0),
-		 logger(LogFactory::getInstance())
-  {}
+  HttpResponse();
   
-  ~HttpResponse() {}
+  ~HttpResponse();
 
   void validateResponse() const;
 
@@ -66,7 +68,7 @@ public:
    * this function returns the filename from it.
    * If it is not there, returns the part of filename from the request URL.
    */
-  string determinFilename() const;
+  std::string determinFilename() const;
 
   void retrieveCookie();
 
@@ -77,56 +79,53 @@ public:
 
   void processRedirect();
 
-  string getRedirectURI() const;
+  std::string getRedirectURI() const;
 
   bool isTransferEncodingSpecified() const;
 
-  string getTransferEncoding() const;
+  std::string getTransferEncoding() const;
 
-  TransferEncodingHandle getTransferDecoder() const;
+  SharedHandle<Decoder> getTransferEncodingDecoder() const;
 
-  int64_t getContentLength() const;
+  bool isContentEncodingSpecified() const;
 
-  int64_t getEntityLength() const;
+  const std::string& getContentEncoding() const;
 
-  string getContentType() const;
+  SharedHandle<Decoder> getContentEncodingDecoder() const;
 
-  void setHttpHeader(const HttpHeaderHandle& httpHeader)
-  {
-    this->httpHeader = httpHeader;
-  }
+  uint64_t getContentLength() const;
 
-  HttpHeaderHandle getHttpHeader() const
-  {
-    return httpHeader;
-  }
+  uint64_t getEntityLength() const;
 
-  void setStatus(int32_t status)
-  {
-    this->status = status;
-  }
+  // Returns type "/" subtype. The parameter is removed.
+  std::string getContentType() const;
 
-  int32_t getStatus() const
-  {
-    return status;
-  }
+  void setHttpHeader(const SharedHandle<HttpHeader>& httpHeader);
 
-  void setHttpRequest(const HttpRequestHandle& httpRequest)
-  {
-    this->httpRequest = httpRequest;
-  }
+  SharedHandle<HttpHeader> getHttpHeader() const;
 
-  HttpRequestHandle getHttpRequest() const
-  {
-    return httpRequest;
-  }
+  const std::string& getResponseStatus() const;
+
+  void setHttpRequest(const SharedHandle<HttpRequest>& httpRequest);
+
+  SharedHandle<HttpRequest> getHttpRequest() const;
 
   void setCuid(int32_t cuid)
   {
     this->cuid = cuid;
   }
+
+  bool hasRetryAfter() const;
+
+  time_t getRetryAfter() const;
+
+  Time getLastModifiedTime() const;
+
+  bool supportsPersistentConnection() const;
 };
 
 typedef SharedHandle<HttpResponse> HttpResponseHandle;
 
+} // namespace aria2
+
 #endif // _D_HTTP_RESPONSE_H_
