@@ -36,104 +36,99 @@
 #define _D_MULTI_DISK_ADAPTOR_H_
 
 #include "DiskAdaptor.h"
-#include "DiskWriter.h"
-#include "File.h"
+
+namespace aria2 {
 
 class MultiFileAllocationIterator;
+class FileEntry;
+class DiskWriter;
 
 class DiskWriterEntry {
 private:
-  FileEntryHandle fileEntry;
-  DiskWriterHandle diskWriter;
+  SharedHandle<FileEntry> fileEntry;
+  SharedHandle<DiskWriter> diskWriter;
+  bool _open;
+  bool _directIO;
+  bool _needsFileAllocation;
 public:
-  DiskWriterEntry(const FileEntryHandle& fileEntry):
-    fileEntry(fileEntry),
-    diskWriter(0) {}
+  DiskWriterEntry(const SharedHandle<FileEntry>& fileEntry);
 
-  ~DiskWriterEntry() {}
+  ~DiskWriterEntry();
 
-  string getFilePath(const string& topDir) const
-  {
-    return topDir+"/"+fileEntry->getPath();
-  }
+  std::string getFilePath(const std::string& topDir) const;
 
-  void initAndOpenFile(const string& topDir)
-  {
-    diskWriter->initAndOpenFile(getFilePath(topDir), fileEntry->getLength());
-  }
+  void initAndOpenFile(const std::string& topDir);
 
-  void openFile(const string& topDir)
-  {
-    diskWriter->openFile(getFilePath(topDir), fileEntry->getLength());
-  }
+  void openFile(const std::string& topDir);
 
-  void openExistingFile(const string& topDir)
-  {
-    diskWriter->openExistingFile(getFilePath(topDir), fileEntry->getLength());
-  }
+  void openExistingFile(const std::string& topDir);
 
-  void closeFile()
-  {
-    diskWriter->closeFile();
-  }
+  void closeFile();
 
-  bool fileExists(const string& topDir)
-  {
-    return File(getFilePath(topDir)).exists();
-  }
+  bool isOpen() const;
 
-  int64_t size() const
-  {
-    return diskWriter->size();
-  }
+  bool fileExists(const std::string& topDir);
 
-  FileEntryHandle getFileEntry() const {
-    return fileEntry;
-  }
+  uint64_t size() const;
 
-  void setDiskWriter(const DiskWriterHandle& diskWriter) {
-    this->diskWriter = diskWriter;
-  }
+  SharedHandle<FileEntry> getFileEntry() const;
 
-  DiskWriterHandle getDiskWriter() const {
-    return diskWriter;
-  }
+  void setDiskWriter(const SharedHandle<DiskWriter>& diskWriter);
 
-  bool operator<(const DiskWriterEntry& entry) const
-  {
-    return fileEntry < entry.fileEntry;
-  }
+  SharedHandle<DiskWriter> getDiskWriter() const;
+
+  bool operator<(const DiskWriterEntry& entry) const;
+
+  // Set _directIO to true.
+  // Additionally, if diskWriter is opened, diskWriter->enableDirectIO() is
+  // called.
+  void enableDirectIO();
+
+  // Set _directIO to false.
+  // Additionally, if diskWriter is opened, diskWriter->disableDirectIO() is
+  // called.
+  void disableDirectIO();
+
+  bool needsFileAllocation() const;
+
+  void needsFileAllocation(bool f);
 };
 
 typedef SharedHandle<DiskWriterEntry> DiskWriterEntryHandle;
 
-typedef deque<DiskWriterEntryHandle> DiskWriterEntries;
+typedef std::deque<DiskWriterEntryHandle> DiskWriterEntries;
 
 class MultiDiskAdaptor : public DiskAdaptor {
   friend class MultiFileAllocationIterator;
 private:
-  string topDir;
-  int32_t pieceLength;
+  std::string topDir;
+  size_t pieceLength;
   DiskWriterEntries diskWriterEntries;
 
+  std::string _cachedTopDirPath;
+
+  std::deque<SharedHandle<DiskWriterEntry> > _openedDiskWriterEntries;
+
+  size_t _maxOpenFiles;
+
   bool _directIOAllowed;
 
   void resetDiskWriterEntries();
 
-  void mkdir() const;
+  void mkdir(const std::string& topDirPath) const;
 
-  bool isInRange(const DiskWriterEntryHandle entry, int64_t offset) const;
+  std::string getTopDirPath() const;
 
-  int32_t calculateLength(const DiskWriterEntryHandle entry,
-			  int64_t fileOffset,
-			  int32_t rem) const;
+  void openIfNot(const SharedHandle<DiskWriterEntry>& entry,
+		 void (DiskWriterEntry::*f)(const std::string&),
+		 const std::string& topDirPath);
+ 
+  static const size_t DEFAULT_MAX_OPEN_FILES = 100;
 
-  string getTopDirPath() const;
 public:
-  MultiDiskAdaptor():pieceLength(0)
-  {}
+  MultiDiskAdaptor();
 
-  virtual ~MultiDiskAdaptor() {}
+  virtual ~MultiDiskAdaptor();
 
   virtual void initAndOpenFile();
 
@@ -145,39 +140,39 @@ public:
 
   virtual void onDownloadComplete();
 
-  virtual void writeData(const unsigned char* data, int32_t len,
-			 int64_t offset);
+  virtual void writeData(const unsigned char* data, size_t len,
+			 off_t offset);
 
-  virtual int32_t readData(unsigned char* data, int32_t len, int64_t offset);
+  virtual ssize_t readData(unsigned char* data, size_t len, off_t offset);
 
   virtual bool fileExists();
 
-  virtual string getFilePath()
+  virtual std::string getFilePath()
   {
     return getTopDirPath();
   }
 
-  virtual int64_t size() const;
+  virtual uint64_t size();
 
-  virtual FileAllocationIteratorHandle fileAllocationIterator();
+  virtual SharedHandle<FileAllocationIterator> fileAllocationIterator();
 
   virtual void enableDirectIO();
 
   virtual void disableDirectIO();
 
-  void setTopDir(const string& topDir) {
+  void setTopDir(const std::string& topDir) {
     this->topDir = topDir;
   }
 
-  const string& getTopDir() const {
+  const std::string& getTopDir() const {
     return topDir;
   }
 
-  void setPieceLength(int32_t pieceLength) {
+  void setPieceLength(size_t pieceLength) {
     this->pieceLength = pieceLength;
   }
 
-  int32_t getPieceLength() const {
+  size_t getPieceLength() const {
     return pieceLength;
   }
 
@@ -190,8 +185,19 @@ public:
   {
     _directIOAllowed = b;
   }
+
+  virtual void cutTrailingGarbage();
+
+  void setMaxOpenFiles(size_t maxOpenFiles);
+
+  virtual size_t utime(const Time& actime, const Time& modtime);
+
+  const std::deque<SharedHandle<DiskWriterEntry> >&
+  getDiskWriterEntries() const;
 };
 
 typedef SharedHandle<MultiDiskAdaptor> MultiDiskAdaptorHandle;
 
+} // namespace aria2
+
 #endif // _D_MULTI_DISK_ADAPTOR_H_
