@@ -34,58 +34,65 @@
 /* copyright --> */
 #include "Option.h"
 #include "prefs.h"
+#include "A2STR.h"
+#include <cstdlib>
+#include <cstring>
+
+namespace aria2 {
 
 Option::Option() {}
 
 Option::~Option() {}
 
-void Option::put(const string& name, const string& value) {
+void Option::put(const std::string& name, const std::string& value) {
   table[name] = value;
 }
 
-bool Option::defined(const string& name) const {
+bool Option::defined(const std::string& name) const
+{
   return table.count(name) == 1;
 }
 
-string Option::get(const string& name) const {
-  map<string, string>::const_iterator itr = table.find(name);
+bool Option::blank(const std::string& name) const
+{
+  std::map<std::string, std::string>::const_iterator i = table.find(name);
+  return i == table.end() || (*i).second.empty();
+}
+
+const std::string& Option::get(const std::string& name) const {
+  std::map<std::string, std::string>::const_iterator itr = table.find(name);
   if(itr == table.end()) {
-    return "";
+    return A2STR::NIL;
   } else {
     return (*itr).second;
   }
 }
 
-int32_t Option::getAsInt(const string& name) const {
-  string value = get(name);
-  if(value == "") {
+int32_t Option::getAsInt(const std::string& name) const {
+  const std::string& value = get(name);
+  if(value.empty()) {
     return 0;
   } else {
-    return strtol(value.c_str(), NULL, 10);
+    return strtol(value.c_str(), 0, 10);
   }
 }
 
-int64_t Option::getAsLLInt(const string& name) const {
-  string value = get(name);
-  if(value == "") {
+int64_t Option::getAsLLInt(const std::string& name) const {
+  const std::string& value = get(name);
+  if(value.empty()) {
     return 0;
   } else {
-    return strtoll(value.c_str(), NULL, 10);
+    return strtoll(value.c_str(), 0, 10);
   }
 }
 
-bool Option::getAsBool(const string& name) const {
-  string value = get(name);
-  if(value == V_TRUE) {
-    return true;
-  } else {
-    return false;
-  }
+bool Option::getAsBool(const std::string& name) const {
+  return get(name) == V_TRUE;
 }
 
-double Option::getAsDouble(const string& name) const {
-  string value = get(name);
-  if(value == "") {
+double Option::getAsDouble(const std::string& name) const {
+  const std::string& value = get(name);
+  if(value.empty()) {
     return 0.0;
   } else {
     return strtod(value.c_str(), 0);
@@ -96,3 +103,5 @@ void Option::clear()
 {
   table.clear();
 }
+
+} // namespace aria2
