@@ -33,81 +33,103 @@
  */
 /* copyright --> */
 #include "PeerInitiateConnectionCommand.h"
-#include "DownloadEngine.h"
+#include "InitiatorMSEHandshakeCommand.h"
 #include "PeerInteractionCommand.h"
-#include "Util.h"
+#include "DownloadEngine.h"
 #include "DlAbortEx.h"
 #include "message.h"
 #include "prefs.h"
-#include "CUIDCounter.h"
+#include "Socket.h"
+#include "Logger.h"
+#include "Peer.h"
+#include "BtContext.h"
+#include "BtRuntime.h"
+#include "PeerStorage.h"
+#include "PieceStorage.h"
+#include "PeerConnection.h"
 
-PeerInitiateConnectionCommand::PeerInitiateConnectionCommand(int cuid,
-							     RequestGroup* requestGroup,
-							     const PeerHandle& peer,
-							     DownloadEngine* e,
-							     const BtContextHandle& btContext)
-  :PeerAbstractCommand(cuid, peer, e),
-   BtContextAwareCommand(btContext),
-   RequestGroupAware(requestGroup)
+namespace aria2 {
+
+PeerInitiateConnectionCommand::PeerInitiateConnectionCommand
+(int cuid,
+ RequestGroup* requestGroup,
+ const PeerHandle& peer,
+ DownloadEngine* e,
+ const SharedHandle<BtContext>& btContext,
+ const SharedHandle<BtRuntime>& btRuntime,
+ bool mseHandshakeEnabled)
+  :
+  PeerAbstractCommand(cuid, peer, e),
+  RequestGroupAware(requestGroup),
+  _btContext(btContext),
+  _btRuntime(btRuntime),
+  _mseHandshakeEnabled(mseHandshakeEnabled)
 {
-  btRuntime->increaseConnections();
+  _btRuntime->increaseConnections();
 }
 
 PeerInitiateConnectionCommand::~PeerInitiateConnectionCommand()
 {
-  btRuntime->decreaseConnections();
+  _btRuntime->decreaseConnections();
 }
 
 bool PeerInitiateConnectionCommand::executeInternal() {
-  Command* command;
   logger->info(MSG_CONNECTING_TO_SERVER, cuid, peer->ipaddr.c_str(),
 	       peer->port);
+  socket.reset(new SocketCore());
   socket->establishConnection(peer->ipaddr, peer->port);
-  command =
-    new PeerInteractionCommand(cuid,
-			       _requestGroup,
-			       peer,
-			       e,
-			       btContext,
-			       socket,
-			       PeerInteractionCommand::INITIATOR_SEND_HANDSHAKE);
-
-  e->commands.push_back(command);
-  return true;
-}
-
-// TODO this method removed when PeerBalancerCommand is implemented
-bool PeerInitiateConnectionCommand::prepareForNextPeer(int wait) {
-  if(peerStorage->isPeerAvailable() && btRuntime->lessThanEqMinPeer()) {
-    PeerHandle peer = peerStorage->getUnusedPeer();
-    peer->cuid = CUIDCounterSingletonHolder::instance()->newID();
-    PeerInitiateConnectionCommand* command =
-      new PeerInitiateConnectionCommand(peer->cuid,
-					_requestGroup,
-					peer,
-					e,
-					btContext);
+  if(_mseHandshakeEnabled) {
+    InitiatorMSEHandshakeCommand* c =
+      new InitiatorMSEHandshakeCommand(cuid, _requestGroup, peer, e, _btContext,
+				       _btRuntime, socket);
+    c->setPeerStorage(_peerStorage);
+    c->setPieceStorage(_pieceStorage);
+    e->commands.push_back(c);
+  } else {
+    PeerInteractionCommand* command =
+      new PeerInteractionCommand
+      (cuid, _requestGroup, peer, e, _btContext, _btRuntime, _pieceStorage,
+       socket, PeerInteractionCommand::INITIATOR_SEND_HANDSHAKE);
+    command->setPeerStorage(_peerStorage);
     e->commands.push_back(command);
   }
   return true;
 }
 
-bool PeerInitiateConnectionCommand::prepareForRetry(int wait) {
-  PeerInitiateConnectionCommand* command =
-    new PeerInitiateConnectionCommand(cuid,
-				      _requestGroup,
-				      peer,
-				      e,
-				      btContext);
-  e->commands.push_back(command);
+// TODO this method removed when PeerBalancerCommand is implemented
+bool PeerInitiateConnectionCommand::prepareForNextPeer(time_t wait) {
+  if(_peerStorage->isPeerAvailable() && _btRuntime->lessThanEqMinPeers()) {
+    PeerHandle peer = _peerStorage->getUnusedPeer();
+    peer->usedBy(e->newCUID());
+    PeerInitiateConnectionCommand* command =
+      new PeerInitiateConnectionCommand(peer->usedBy(), _requestGroup, peer, e,
+					_btContext, _btRuntime);
+    command->setPeerStorage(_peerStorage);
+    command->setPieceStorage(_pieceStorage);
+    e->commands.push_back(command);
+  }
   return true;
 }
 
-void PeerInitiateConnectionCommand::onAbort(Exception* ex) {
-  peerStorage->returnPeer(peer);
+void PeerInitiateConnectionCommand::onAbort() {
+  _peerStorage->returnPeer(peer);
 }
 
 bool PeerInitiateConnectionCommand::exitBeforeExecute()
 {
-  return btRuntime->isHalt();
+  return _btRuntime->isHalt();
 }
+
+void PeerInitiateConnectionCommand::setPeerStorage
+(const SharedHandle<PeerStorage>& peerStorage)
+{
+  _peerStorage = peerStorage;
+}
+
+void PeerInitiateConnectionCommand::setPieceStorage
+(const SharedHandle<PieceStorage>& pieceStorage)
+{
+  _pieceStorage = pieceStorage;
+}
+
+} // namespace aria2
