@@ -37,43 +37,61 @@
 
 #include "PeerAbstractCommand.h"
 #include "RequestGroupAware.h"
-#include "BtContextAwareCommand.h"
-#include "BtInteractive.h"
-#include "PeerConnection.h"
+
+namespace aria2 {
+
+class BtContext;
+class BtInteractive;
+class PeerConnection;
+class BtRuntime;
+class PeerStorage;
+class PieceStorage;
 
 class PeerInteractionCommand : public PeerAbstractCommand,
-			       public BtContextAwareCommand,
 			       public RequestGroupAware
 {
 public:
   enum Seq {
     INITIATOR_SEND_HANDSHAKE,
     INITIATOR_WAIT_HANDSHAKE,
-    //RECEIVER_SEND_HANDSHAKE,
     RECEIVER_WAIT_HANDSHAKE,
     WIRED};
 private:
+  SharedHandle<BtContext> _btContext;
+
+  SharedHandle<BtRuntime> _btRuntime;
+
+  SharedHandle<PieceStorage> _pieceStorage;
+
+  SharedHandle<PeerStorage> _peerStorage;
+
   Seq sequence;
-  BtInteractiveHandle btInteractive;
-  int32_t maxDownloadSpeedLimit;
+  SharedHandle<BtInteractive> btInteractive;
+  unsigned int maxDownloadSpeedLimit;
 protected:
   virtual bool executeInternal();
-  virtual bool prepareForRetry(int32_t wait);
-  virtual bool prepareForNextPeer(int32_t wait);
-  virtual void onAbort(Exception* ex);
+  virtual bool prepareForNextPeer(time_t wait);
+  virtual void onAbort();
+  virtual void onFailure();
   virtual bool exitBeforeExecute();
 public:
   PeerInteractionCommand(int32_t cuid,
 			 RequestGroup* requestGroup,
-			 const PeerHandle& peer,
+			 const SharedHandle<Peer>& peer,
 			 DownloadEngine* e,
-			 const BtContextHandle& btContext,
-			 const SocketHandle& s,
+			 const SharedHandle<BtContext>& btContext,
+			 const SharedHandle<BtRuntime>& btRuntime,
+			 const SharedHandle<PieceStorage>& pieceStorage,
+			 const SharedHandle<SocketCore>& s,
 			 Seq sequence,
-			 const PeerConnectionHandle& peerConnection = 0);
+			 const SharedHandle<PeerConnection>& peerConnection =
+			 SharedHandle<PeerConnection>());
 
   virtual ~PeerInteractionCommand();
 
+  void setPeerStorage(const SharedHandle<PeerStorage>& peerStorage);
 };
 
+} // namespace aria2
+
 #endif // _D_PEER_INTERACTION_COMMAND_H_
