@@ -40,58 +40,100 @@
 #include "BtContext.h"
 #include "DlAbortEx.h"
 #include "PeerInteractionCommand.h"
+#include "Peer.h"
+#include "BtRegistry.h"
+#include "PeerStorage.h"
+#include "PieceStorage.h"
+#include "BtRuntime.h"
+#include "BtAnnounce.h"
+#include "BtProgressInfoFile.h"
+#include "BtConstants.h"
 #include "message.h"
+#include "Socket.h"
+#include "Logger.h"
+#include "prefs.h"
+#include "Option.h"
+#include "RequestGroupMan.h"
+#include "StringFormat.h"
+#include "RequestGroup.h"
 
-PeerReceiveHandshakeCommand::PeerReceiveHandshakeCommand(int32_t cuid,
-							 const PeerHandle& peer,
-							 DownloadEngine* e,
-							 const SocketHandle& s):
+namespace aria2 {
+
+PeerReceiveHandshakeCommand::PeerReceiveHandshakeCommand
+(int32_t cuid,
+ const PeerHandle& peer,
+ DownloadEngine* e,
+ const SocketHandle& s,
+ const SharedHandle<PeerConnection>& peerConnection)
+  :
   PeerAbstractCommand(cuid, peer, e, s),
-  _peerConnection(new PeerConnection(cuid, s, e->option)),
-  _lowestSpeedLimit(20*1024)
-{}
+  _peerConnection(peerConnection),
+  _thresholdSpeed(e->option->getAsInt(PREF_BT_REQUEST_PEER_SPEED_LIMIT))
+{
+  if(_peerConnection.isNull()) {
+    _peerConnection.reset(new PeerConnection(cuid, socket, e->option));
+  }
+  unsigned int maxDownloadSpeed = e->option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT);
+  if(maxDownloadSpeed > 0) {
+    _thresholdSpeed = std::min(maxDownloadSpeed, _thresholdSpeed);
+  }
+}
 
 PeerReceiveHandshakeCommand::~PeerReceiveHandshakeCommand() {}
 
 bool PeerReceiveHandshakeCommand::exitBeforeExecute()
 {
-  return e->isHaltRequested();
+  return e->isHaltRequested() || e->_requestGroupMan->downloadFinished();
 }
 
 bool PeerReceiveHandshakeCommand::executeInternal()
 {
   unsigned char data[BtHandshakeMessage::MESSAGE_LENGTH];
-  int32_t dataLength = BtHandshakeMessage::MESSAGE_LENGTH;
+  size_t dataLength = BtHandshakeMessage::MESSAGE_LENGTH;
   // ignore return value. The received data is kept in PeerConnection object
   // because of peek = true.
   _peerConnection->receiveHandshake(data, dataLength, true);
   // To handle tracker's NAT-checking feature
   if(dataLength >= 48) {
     // check info_hash
-    string infoHash = Util::toHex(&data[28], INFO_HASH_LENGTH);
-    BtContextHandle btContext = BtRegistry::getBtContext(infoHash);
-    if(btContext.isNull() || !BT_RUNTIME(btContext)->ready()) {
-      throw new DlAbortEx("Unknown info hash %s", infoHash.c_str());
+    std::string infoHash = Util::toHex(&data[28], INFO_HASH_LENGTH);
+
+    SharedHandle<BtRegistry> btRegistry = e->getBtRegistry();
+    SharedHandle<BtContext> btContext = btRegistry->getBtContext(infoHash);
+    SharedHandle<BtRuntime> btRuntime = btRegistry->getBtRuntime(infoHash);
+    SharedHandle<PieceStorage> pieceStorage =
+      btRegistry->getPieceStorage(infoHash);
+    SharedHandle<PeerStorage> peerStorage =
+      btRegistry->getPeerStorage(infoHash);
+
+    if(btContext.isNull() || !btRuntime->ready()) {
+      throw DlAbortEx
+	(StringFormat("Unknown info hash %s", infoHash.c_str()).str());
     }
-    TransferStat tstat = PEER_STORAGE(btContext)->calculateStat();
-    if(!PIECE_STORAGE(btContext)->downloadFinished() && tstat.getDownloadSpeed() < _lowestSpeedLimit ||
-       BT_RUNTIME(btContext)->getConnections() < MAX_PEERS) {
-      peer->reconfigure(btContext->getPieceLength(), btContext->getTotalLength());
-      PEER_STORAGE(btContext)->addIncomingPeer(peer);
+    TransferStat tstat = btContext->getOwnerRequestGroup()->calculateStat();
+    if((!pieceStorage->downloadFinished() &&
+       tstat.getDownloadSpeed() < _thresholdSpeed) ||
+       btRuntime->lessThanMaxPeers()) {
+      if(peerStorage->addPeer(peer)) {
 
-      peer->cuid = cuid;
-
-      PeerInteractionCommand* command =
-	new PeerInteractionCommand(cuid,
-				   btContext->getOwnerRequestGroup(),
-				   peer,
-				   e,
-				   btContext,
-				   socket,
-				   PeerInteractionCommand::RECEIVER_WAIT_HANDSHAKE,
-				   _peerConnection);
-      e->commands.push_back(command);
-      logger->debug(MSG_INCOMING_PEER_CONNECTION, cuid, peer->cuid);
+	peer->usedBy(cuid);
+	
+	PeerInteractionCommand* command =
+	  new PeerInteractionCommand
+	  (cuid,
+	   btContext->getOwnerRequestGroup(),
+	   peer,
+	   e,
+	   btContext,
+	   btRuntime,
+	   pieceStorage,
+	   socket,
+	   PeerInteractionCommand::RECEIVER_WAIT_HANDSHAKE,
+	   _peerConnection);
+	command->setPeerStorage(peerStorage);
+	e->commands.push_back(command);
+	logger->debug(MSG_INCOMING_PEER_CONNECTION, cuid, peer->usedBy());
+      }
     }
     return true;
   } else {
@@ -99,3 +141,5 @@ bool PeerReceiveHandshakeCommand::executeInternal()
     return false;
   }
 }
+
+} // namespace aria2
