@@ -36,16 +36,17 @@
 #define _D_PIECE_STORAGE_H_
 
 #include "common.h"
+#include "SharedHandle.h"
 #include "TimeA2.h"
 #include "IntSequence.h"
+#include <string>
+#include <deque>
+
+namespace aria2 {
 
 class Piece;
-typedef SharedHandle<Piece> PieceHandle;
-typedef deque<PieceHandle> Pieces;
 class Peer;
-typedef SharedHandle<Peer> PeerHandle;
 class DiskAdaptor;
-typedef SharedHandle<DiskAdaptor> DiskAdaptorHandle;
 
 class PieceStorage {
 public:
@@ -55,7 +56,7 @@ public:
    * Returns true if the peer has a piece that localhost doesn't have.
    * Otherwise returns false.
    */
-  virtual bool hasMissingPiece(const PeerHandle& peer) = 0;
+  virtual bool hasMissingPiece(const SharedHandle<Peer>& peer) = 0;
 
   /**
    * Returns a piece that the peer has but localhost doesn't.
@@ -63,7 +64,17 @@ public:
    * from get the same piece. But in end game mode, same piece may be returned
    * to several commands.
    */
-  virtual PieceHandle getMissingPiece(const PeerHandle& peer) = 0;
+  virtual SharedHandle<Piece>
+  getMissingPiece(const SharedHandle<Peer>& peer) = 0;
+
+  /**
+   * Same as getMissingPiece(const SharedHandle<Peer>& peer), but the indexes in
+   * excludedIndexes are excluded.
+   */
+  virtual SharedHandle<Piece> getMissingPiece
+  (const SharedHandle<Peer>& peer,
+   const std::deque<size_t>& excludedIndexes) = 0;
+
   /**
    * Returns a piece that the peer has but localhost doesn't.
    * Only pieces that declared as "fast" are returned.
@@ -71,12 +82,21 @@ public:
    * from get the same piece. But in end game mode, same piece may be returned
    * to several commands.
    */
-  virtual PieceHandle getMissingFastPiece(const PeerHandle& peer) = 0;
+  virtual SharedHandle<Piece>
+  getMissingFastPiece(const SharedHandle<Peer>& peer) = 0;
+  
+  /**
+   * Same as getMissingFastPiece(const SharedHandle<Peer>& peer), but the
+   * indexes in excludedIndexes are excluded.
+   */
+  virtual SharedHandle<Piece> getMissingFastPiece
+  (const SharedHandle<Peer>& peer,
+   const std::deque<size_t>& excludedIndexes) = 0;
 
   /**
    * Returns a missing piece if available. Otherwise returns 0;
    */
-  virtual PieceHandle getMissingPiece() = 0;
+  virtual SharedHandle<Piece> getMissingPiece() = 0;
 
   /**
    * Returns a missing piece whose index is index.
@@ -84,46 +104,46 @@ public:
    * then returns 0.
    * Also returns 0 if any of missing piece is not available.
    */
-  virtual PieceHandle getMissingPiece(int32_t index) = 0;
+  virtual SharedHandle<Piece> getMissingPiece(size_t index) = 0;
 
   /**
    * Returns the piece denoted by index.
    * No status of the piece is changed in this method.
    */
-  virtual PieceHandle getPiece(int32_t index) = 0;
+  virtual SharedHandle<Piece> getPiece(size_t index) = 0;
 
   /**
    * Marks the piece whose index is index as missing.
    */
-  virtual void markPieceMissing(int32_t index) = 0;
+  virtual void markPieceMissing(size_t index) = 0;
 
   /**
    * Tells that the download of the specfied piece completes.
    */
-  virtual void completePiece(const PieceHandle& piece) = 0;
+  virtual void completePiece(const SharedHandle<Piece>& piece) = 0;
 
   /**
    * Tells that the download of the specified piece is canceled.
    */
-  virtual void cancelPiece(const PieceHandle& piece) = 0;
+  virtual void cancelPiece(const SharedHandle<Piece>& piece) = 0;
 
   /**
    * Returns true if the specified piece is already downloaded.
    * Otherwise returns false.
    */
-  virtual bool hasPiece(int32_t index) = 0;
+  virtual bool hasPiece(size_t index) = 0;
 
-  virtual bool isPieceUsed(int32_t index) = 0;
+  virtual bool isPieceUsed(size_t index) = 0;
 
-  virtual int64_t getTotalLength() = 0;
+  virtual uint64_t getTotalLength() = 0;
 
-  virtual int64_t getFilteredTotalLength() = 0;
+  virtual uint64_t getFilteredTotalLength() = 0;
 
-  virtual int64_t getCompletedLength() = 0;
+  virtual uint64_t getCompletedLength() = 0;
 
-  virtual int64_t getFilteredCompletedLength() = 0;
+  virtual uint64_t getFilteredCompletedLength() = 0;
   
-  virtual void setFileFilter(const Strings& filePaths) = 0;
+  virtual void setFileFilter(const std::deque<std::string>& filePaths) = 0;
 
   virtual void setFileFilter(IntSequence seq) = 0;
 
@@ -151,9 +171,9 @@ public:
   virtual const unsigned char* getBitfield() = 0;
 
   virtual void setBitfield(const unsigned char* bitfield,
-			   int32_t bitfieldLength) = 0;
+			   size_t bitfieldLength) = 0;
   
-  virtual int32_t getBitfieldLength() = 0;
+  virtual size_t getBitfieldLength() = 0;
 
   virtual bool isSelectiveDownloadingMode() = 0;
 
@@ -161,28 +181,29 @@ public:
 
   virtual bool isEndGame() = 0;
 
-  virtual DiskAdaptorHandle getDiskAdaptor() = 0;
+  virtual SharedHandle<DiskAdaptor> getDiskAdaptor() = 0;
   
-  virtual int32_t getPieceLength(int32_t index) = 0;
+  virtual size_t getPieceLength(size_t index) = 0;
 
   /**
    * Adds piece index to advertise to other commands. They send have message
    * based on this information.
    */
-  virtual void advertisePiece(int32_t cuid, int32_t index) = 0;
+  virtual void advertisePiece(int32_t cuid, size_t index) = 0;
 
   /**
-   * Returns piece index which is not advertised by the caller command and
-   * newer than lastCheckTime.
+   * indexes is filled with piece index which is not advertised by the caller
+   * command and newer than lastCheckTime.
    */
-  virtual Integers getAdvertisedPieceIndexes(int32_t myCuid,
-					     const Time& lastCheckTime) = 0;
+  virtual void getAdvertisedPieceIndexes(std::deque<size_t>& indexes,
+					 int32_t myCuid,
+					 const Time& lastCheckTime) = 0;
 
   /**
    * Removes have entry if specified seconds have elapsed since its
    * registration.
    */
-  virtual void removeAdvertisedPiece(int32_t elapsed) = 0;
+  virtual void removeAdvertisedPiece(time_t elapsed) = 0;
 
   /**
    * Sets all bits in bitfield to 1.
@@ -192,15 +213,31 @@ public:
   /**
    * Sets all bits in bitfield(0 to length) to 1.
    */
-  virtual void markPiecesDone(int64_t length) = 0;
+  virtual void markPiecesDone(uint64_t length) = 0;
 
-  virtual void addInFlightPiece(const Pieces& pieces) = 0;
+  virtual void
+  addInFlightPiece(const std::deque<SharedHandle<Piece> >& pieces) = 0;
 
-  virtual int32_t countInFlightPiece() = 0;
+  virtual size_t countInFlightPiece() = 0;
+
+  virtual void getInFlightPieces(std::deque<SharedHandle<Piece> >& pieces) = 0;
+
+  virtual void addPieceStats(size_t index) = 0;
+
+  virtual void addPieceStats(const unsigned char* bitfield,
+			     size_t bitfieldLength) = 0;
+
+  virtual void subtractPieceStats(const unsigned char* bitfield,
+				  size_t bitfieldLength) = 0;
+
+  virtual void updatePieceStats(const unsigned char* newBitfield,
+				size_t newBitfieldLength,
+				const unsigned char* oldBitfield) = 0;
 
-  virtual Pieces getInFlightPieces() = 0;
 };
 
 typedef SharedHandle<PieceStorage> PieceStorageHandle;
 
+} // namespace aria2
+
 #endif // _D_PIECE_STORAGE_H_
