@@ -37,55 +37,71 @@
 
 #include "Segment.h"
 
+namespace aria2 {
+
 class PiecedSegment:public Segment {
 private:
   /**
    * Piece class has length property but it is a actual length of piece.
    * The last piece likely have shorter length than the other length.
    */
-  int32_t _pieceLength;
-  int32_t _writtenLength;
-  int32_t _overflowLength;
-  PieceHandle _piece;
+  size_t _pieceLength;
+  size_t _overflowLength;
+  SharedHandle<Piece> _piece;
+  size_t _writtenLength;
 
 public:
-  PiecedSegment(int32_t pieceLength, const PieceHandle& piece);
+  PiecedSegment(size_t pieceLength, const SharedHandle<Piece>& piece);
 
   virtual ~PiecedSegment();
 
   virtual bool complete() const;
 
-  virtual int32_t getIndex() const;
+  virtual size_t getIndex() const;
 
-  virtual int64_t getPosition() const;
+  virtual off_t getPosition() const;
 
-  virtual int64_t getPositionToWrite() const;
+  virtual off_t getPositionToWrite() const;
 
-  virtual int32_t getLength() const;
+  virtual size_t getLength() const;
 
-  virtual int32_t getSegmentLength() const
+  virtual size_t getSegmentLength() const
   {
     return _pieceLength;
   }
 
-  virtual int32_t getWrittenLength() const
+  virtual size_t getWrittenLength() const
   {
     return _writtenLength;
   }
 
-  virtual int32_t getOverflowLength() const
+  virtual size_t getOverflowLength() const
   {
     return _overflowLength;
   }
 
-  virtual void updateWrittenLength(int32_t bytes);
+  virtual void updateWrittenLength(size_t bytes);
+
+#ifdef ENABLE_MESSAGE_DIGEST
+
+  // `begin' is a offset inside this segment.
+  virtual bool updateHash(uint32_t begin,
+			  const unsigned char* data, size_t dataLength);
+
+  virtual bool isHashCalculated() const;
+
+  virtual std::string getHashString();
+
+#endif // ENABLE_MESSAGE_DIGEST
 
   virtual void clear();
 
-  virtual PieceHandle getPiece() const;
+  virtual SharedHandle<Piece> getPiece() const;
 };
 
 typedef SharedHandle<PiecedSegment> PiecedSegmentHandle;
 
+} // namespace aria2
+
 #endif // _D_PIECED_SEGMENT_H_
 
