@@ -33,16 +33,30 @@
  */
 /* copyright --> */
 #include "RealtimeCommand.h"
+#include "DownloadEngine.h"
+#include "Exception.h"
+
+namespace aria2 {
+
+RealtimeCommand::RealtimeCommand(int cuid, RequestGroup* requestGroup,
+				 DownloadEngine* e):
+  Command(cuid),
+  RequestGroupAware(requestGroup),
+  _e(e)
+{
+  setStatusRealtime();
+}
 
 bool RealtimeCommand::execute()
 {
   setStatusRealtime();
-  _e->noWait = true;
+  _e->setNoWait(true);
   try {
     return executeInternal();
-  } catch(Exception* e) {
+  } catch(Exception& e) {
     bool r =  handleException(e);
-    delete e;
     return r;
   }
 }
+
+} // namespace aria2
