@@ -36,36 +36,31 @@
 #define _D_SEGMENT_MAN_H_
 
 #include "common.h"
+#include "SharedHandle.h"
+#include <deque>
+
+namespace aria2 {
 
 class Segment;
-typedef SharedHandle<Segment> SegmentHandle;
-typedef deque<SegmentHandle> Segments;
 class Logger;
 class Option;
 class PeerStat;
-typedef SharedHandle<PeerStat> PeerStatHandle;
-typedef deque<PeerStatHandle> PeerStats;
 class DownloadContext;
-typedef SharedHandle<DownloadContext> DownloadContextHandle;
 class PieceStorage;
-typedef SharedHandle<PieceStorage> PieceStorageHandle;
 class Piece;
-typedef SharedHandle<Piece> PieceHandle;
-
-#define SEGMENT_FILE_EXTENSION ".aria2"
 
 class SegmentEntry {
 public:
   int32_t cuid;
-  SegmentHandle segment;
+  SharedHandle<Segment> segment;
 public:
-  SegmentEntry(int32_t cuid, const SegmentHandle& segment);
+  SegmentEntry(int32_t cuid, const SharedHandle<Segment>& segment);
 
   ~SegmentEntry();
 };
 
 typedef SharedHandle<SegmentEntry> SegmentEntryHandle;
-typedef deque<SegmentEntryHandle> SegmentEntries;
+typedef std::deque<SegmentEntryHandle> SegmentEntries;
 
 /**
  * This class holds the download progress of the one download entry.
@@ -74,23 +69,25 @@ class SegmentMan {
 private:
   const Option* _option;
 
-  const Logger* logger;
+  Logger* logger;
 
-  DownloadContextHandle _downloadContext;
+  SharedHandle<DownloadContext> _downloadContext;
 
-  PieceStorageHandle _pieceStorage;
+  SharedHandle<PieceStorage> _pieceStorage;
 
   SegmentEntries usedSegmentEntries;
 
-  PeerStats peerStats;
+  std::deque<SharedHandle<PeerStat> > peerStats;
 
-  SegmentHandle checkoutSegment(int32_t cuid, const PieceHandle& piece);
+  SharedHandle<Segment> checkoutSegment(int32_t cuid,
+					const SharedHandle<Piece>& piece);
 
-  SegmentEntryHandle findSlowerSegmentEntry(const PeerStatHandle& peerStat) const;
+  SharedHandle<SegmentEntry> findSlowerSegmentEntry
+  (const SharedHandle<PeerStat>& peerStat);
 public:
   SegmentMan(const Option* option,
-	     const DownloadContextHandle& downloadContext = 0,
-	     const PieceStorageHandle& pieceStorage = 0);
+	     const SharedHandle<DownloadContext>& downloadContext,
+	     const SharedHandle<PieceStorage>& pieceStorage);
 
   ~SegmentMan();
 
@@ -104,7 +101,7 @@ public:
    * If Transfer-Encoding is Chunked or Content-Length header is not provided,
    * then this value is set to be 0.
    */
-  int64_t getTotalLength() const;
+  uint64_t getTotalLength() const;
 
   /**
    * Returs true when the download has finished.
@@ -114,19 +111,21 @@ public:
   bool downloadFinished() const;
 
   /**
-   * Returns a vacant segment.
-   * If there is no vacant segment, then returns a segment instance whose
-   * isNull call is true.
+   * Fill segments which are assigned to the command whose CUID is cuid.
+   * This function doesn't clear passed segments.
    */
-  Segments getInFlightSegment(int32_t cuid);
-  SegmentHandle getSegment(int32_t cuid);
+  void getInFlightSegment(std::deque<SharedHandle<Segment> >& segments,
+			  int32_t cuid);
+
+  SharedHandle<Segment> getSegment(int32_t cuid);
+
   /**
    * Returns a segment whose index is index. 
    * If it has already assigned
    * to another cuid or has been downloaded, then returns a segment instance
    * whose isNull call is true.
    */
-  SegmentHandle getSegment(int32_t cuid, int32_t index);
+  SharedHandle<Segment> getSegment(int32_t cuid, size_t index);
   /**
    * Updates download status.
    */
@@ -139,46 +138,57 @@ public:
   /**
    * Tells SegmentMan that the segment has been downloaded successfully.
    */
-  bool completeSegment(int32_t cuid, const SegmentHandle& segment);
+  bool completeSegment(int32_t cuid, const SharedHandle<Segment>& segment);
 
   /**
    * Injects PieceStorage.
    */
-  void setPieceStorage(const PieceStorageHandle& pieceStorage);
+  void setPieceStorage(const SharedHandle<PieceStorage>& pieceStorage);
 
   /**
    * Injects DownloadContext.
    */
-  void setDownloadContext(const DownloadContextHandle& downloadContext);
+  void setDownloadContext(const SharedHandle<DownloadContext>& downloadContext);
 
   /**
    * Returns true if the segment whose index is index has been downloaded.
    */
-  bool hasSegment(int32_t index) const;
+  bool hasSegment(size_t index) const;
   /**
    * Returns the length of bytes downloaded.
    */
-  int64_t getDownloadLength() const;
+  uint64_t getDownloadLength() const;
 
   /**
-   * Registers given peerStat if it has not been registerd.
-   * Otherwise does nothing.
+   * Registers given peerStat if it has not been registerd and returns true.
+   * Otherwise does nothing and returns false.
    */
-  void registerPeerStat(const PeerStatHandle& peerStat);
+  bool registerPeerStat(const SharedHandle<PeerStat>& peerStat);
 
   /**
    * Returns peerStat whose cuid is given cuid. If it is not found, returns
    * 0.
    */
-  PeerStatHandle getPeerStat(int32_t cuid) const;
+  SharedHandle<PeerStat> getPeerStat(int32_t cuid) const;
+
+
+  const std::deque<SharedHandle<PeerStat> >& getPeerStats() const;
 
   /**
    * Returns current download speed in bytes per sec. 
    */
-  int32_t calculateDownloadSpeed() const;
+  unsigned int calculateDownloadSpeed() const;
 
-  int32_t countFreePieceFrom(int32_t index) const;
+  /**
+   * Returns the downloaded bytes in this session.
+   */
+  uint64_t calculateSessionDownloadLength() const;
+
+  size_t countFreePieceFrom(size_t index) const;
 };
 
 typedef SharedHandle<SegmentMan> SegmentManHandle;
+
+} // namespace aria2
+
 #endif // _D_SEGMENT_MAN_H_
