@@ -37,9 +37,9 @@
 
 #include "AbstractBtMessage.h"
 
+namespace aria2 {
+
 class SimpleBtMessage : public AbstractBtMessage {
-private:
-  int32_t leftDataLength;
 public:
   SimpleBtMessage();
 
@@ -49,7 +49,7 @@ public:
 
   virtual const unsigned char* getMessage() = 0;
 
-  virtual int32_t getMessageLength() = 0;
+  virtual size_t getMessageLength() = 0;
 
   virtual void onSendComplete() {};
 
@@ -57,4 +57,6 @@ public:
 
 };
 
+} // namespace aria2
+
 #endif // _D_SIMPLE_BT_MESSAGE_H_
