@@ -35,23 +35,28 @@
 #ifndef _D_SLEEP_COMMAND_H_
 #define _D_SLEEP_COMMAND_H_
 
-#include "DownloadEngine.h"
 #include "Command.h"
 #include "TimeA2.h"
 
+namespace aria2 {
+
+class DownloadEngine;
+class RequestGroup;
+
 class SleepCommand:public Command {
 private:
   DownloadEngine* engine;
+  RequestGroup* _requestGroup;
   Command* nextCommand;
-  int32_t wait;
+  time_t wait;
   Time checkPoint;
-
-  bool isHaltRequested() const;
-
 public:
-  SleepCommand(int32_t cuid, DownloadEngine* e, Command* nextCommand, int32_t wait);
+  SleepCommand(int32_t cuid, DownloadEngine* e, RequestGroup* requestGroup,
+	       Command* nextCommand, time_t wait);
   virtual ~SleepCommand();
   bool execute();
 };
 
+} // namespace aria2
+
 #endif // _D_SLEEP_COMMAND_H_
