@@ -38,13 +38,15 @@
 #include "SeedCriteria.h"
 #include "TimeA2.h"
 
+namespace aria2 {
+
 class TimeSeedCriteria : public SeedCriteria {
 private:
   // How much time the client does seeding in seconds.
-  int32_t duration;
+  time_t duration;
   Time watch;
 public:
-  TimeSeedCriteria(int32_t duration):duration(duration) {}
+  TimeSeedCriteria(time_t duration):duration(duration) {}
   virtual ~TimeSeedCriteria() {}
 
   virtual void reset() {
@@ -55,13 +57,15 @@ public:
     return watch.elapsed(duration);
   }
 
-  void setDuration(int32_t duration) {
+  void setDuration(time_t duration) {
     this->duration = duration;
   }
 
-  int32_t getDuration() const {
+  time_t getDuration() const {
     return duration;
   }
 };
 
+} // namespace aria2
+
 #endif // _D_TIME_SEED_CRITERIA_H_
