@@ -1,15 +1,22 @@
 #include "BtRequestMessage.h"
+
+#include <cstring>
+
+#include <cppunit/extensions/HelperMacros.h>
+
 #include "PeerMessageUtil.h"
 #include "MockBtContext.h"
 #include "MockBtMessage.h"
 #include "MockPieceStorage.h"
 #include "MockBtMessageFactory.h"
 #include "MockBtMessageDispatcher.h"
-#include "DefaultBtContext.h"
 #include "BtAbortOutstandingRequestEvent.h"
-#include <cppunit/extensions/HelperMacros.h>
+#include "Peer.h"
+#include "FileEntry.h"
+#include "BtHandshakeMessage.h"
+#include "BtRequestMessageValidator.h"
 
-using namespace std;
+namespace aria2 {
 
 class BtRequestMessageTest:public CppUnit::TestFixture {
 
@@ -26,6 +33,8 @@ class BtRequestMessageTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testHandleAbortRequestEvent_alreadyInvalidated);
   CPPUNIT_TEST(testHandleAbortRequestEvent_sendingInProgress);
   CPPUNIT_TEST(testToString);
+  CPPUNIT_TEST(testValidate);
+  CPPUNIT_TEST(testValidate_lengthTooLong);
   CPPUNIT_TEST_SUITE_END();
 private:
 
@@ -42,82 +51,83 @@ public:
   void testHandleAbortRequestEvent_alreadyInvalidated();
   void testHandleAbortRequestEvent_sendingInProgress();
   void testToString();
+  void testValidate();
+  void testValidate_lengthTooLong();
 
   class MockPieceStorage2 : public MockPieceStorage {
   public:
-    virtual bool hasPiece(int index) {
+    virtual bool hasPiece(size_t index) {
       return index == 1;
     }
   };
 
   class MockBtMessage2 : public MockBtMessage {
   public:
-    string type;
-    int32_t index;
-    int32_t begin;
-    int32_t length;
+    std::string type;
+    size_t index;
+    uint32_t begin;
+    size_t length;
   public:
-    MockBtMessage2(string type, int32_t index, int32_t begin, int32_t length):type(type), index(index), begin(begin), length(length) {}
+    MockBtMessage2(std::string type, size_t index, uint32_t begin,
+		   size_t length)
+      :
+      type(type), index(index), begin(begin), length(length) {}
   };
 
   typedef SharedHandle<MockBtMessage2> MockBtMessage2Handle;
 
   class MockBtMessageFactory2 : public MockBtMessageFactory {
   public:
-    virtual BtMessageHandle
-    createPieceMessage(int32_t index, int32_t begin, int32_t length) {
-      MockBtMessage2Handle btMsg = new MockBtMessage2("piece", index, begin, length);
+    virtual SharedHandle<BtMessage>
+    createPieceMessage(size_t index, uint32_t begin, size_t length) {
+      SharedHandle<MockBtMessage2> btMsg
+	(new MockBtMessage2("piece", index, begin, length));
       return btMsg;
     }
 
-    virtual BtMessageHandle
-    createRejectMessage(int32_t index, int32_t begin, int32_t length) {
-      MockBtMessage2Handle btMsg = new MockBtMessage2("reject", index, begin, length);
+    virtual SharedHandle<BtMessage>
+    createRejectMessage(size_t index, uint32_t begin, size_t length) {
+      SharedHandle<MockBtMessage2> btMsg
+	(new MockBtMessage2("reject", index, begin, length));
       return btMsg;
     }
   };
 
   typedef SharedHandle<MockBtMessageFactory2> MockBtMessageFactory2Handle;
 
-  PeerHandle peer;
-  MockBtMessageDispatcherHandle dispatcher;
-  BtRequestMessageHandle msg;
-
-  BtRequestMessageTest():peer(0), dispatcher(0), msg(0) {}
+  SharedHandle<MockBtContext> _btContext;
+  SharedHandle<MockPieceStorage> _pieceStorage;
+  SharedHandle<Peer> _peer;
+  SharedHandle<MockBtMessageDispatcher> _dispatcher;
+  SharedHandle<MockBtMessageFactory> _messageFactory;
+  SharedHandle<BtRequestMessage> msg;
 
   void setUp() {
-    BtRegistry::unregisterAll();
+    _btContext.reset(new MockBtContext());
+    _btContext->setInfoHash((const unsigned char*)"12345678901234567890");
+    _btContext->setPieceLength(16*1024);
+    _btContext->setTotalLength(256*1024);
 
-    MockBtContextHandle btContext = new MockBtContext();
-    btContext->setInfoHash((const unsigned char*)"12345678901234567890");
-    btContext->setPieceLength(16*1024);
-    btContext->setTotalLength(256*1024);
+    _pieceStorage.reset(new MockPieceStorage2());
 
-    MockPieceStorageHandle pieceStorage = new MockPieceStorage2();
+    _peer.reset(new Peer("host", 6969));
+    _peer->allocateSessionResource(_btContext->getPieceLength(),
+				  _btContext->getTotalLength());
 
-    BtRegistry::registerPieceStorage(btContext->getInfoHashAsString(),
-				     pieceStorage);
+    _dispatcher.reset(new MockBtMessageDispatcher());
 
-    peer = new Peer("host", 6969, btContext->getPieceLength(), btContext->getTotalLength());
+    _messageFactory.reset(new MockBtMessageFactory2());
 
-    BtRegistry::registerPeerObjectCluster(btContext->getInfoHashAsString(),
-					  new PeerObjectCluster());
-    PEER_OBJECT_CLUSTER(btContext)->registerHandle(peer->getId(), new PeerObject());
-
-    dispatcher = new MockBtMessageDispatcher();
-
-    PEER_OBJECT(btContext, peer)->btMessageDispatcher = dispatcher;
-    PEER_OBJECT(btContext, peer)->btMessageFactory = new MockBtMessageFactory2();
-
-    msg = new BtRequestMessage();
-    msg->setBtContext(btContext);
-    msg->setPeer(peer);
+    msg.reset(new BtRequestMessage());
+    msg->setBtContext(_btContext);
+    msg->setPeer(_peer);
     msg->setIndex(1);
     msg->setBegin(16);
     msg->setLength(32);
     msg->setBlockIndex(2);
-    msg->setBtMessageDispatcher(dispatcher);
-    msg->setBtMessageFactory(BT_MESSAGE_FACTORY(btContext, peer));
+    msg->setBtMessageDispatcher(_dispatcher);
+    msg->setBtMessageFactory(_messageFactory);
+    msg->setPieceStorage(_pieceStorage);
   }
 };
 
@@ -130,11 +140,11 @@ void BtRequestMessageTest::testCreate() {
   PeerMessageUtil::setIntParam(&msg[5], 12345);
   PeerMessageUtil::setIntParam(&msg[9], 256);
   PeerMessageUtil::setIntParam(&msg[13], 1024);
-  BtRequestMessageHandle pm = BtRequestMessage::create(&msg[4], 13);
-  CPPUNIT_ASSERT_EQUAL((int8_t)6, pm->getId());
-  CPPUNIT_ASSERT_EQUAL((int32_t)12345, pm->getIndex());
-  CPPUNIT_ASSERT_EQUAL((int32_t)256, pm->getBegin());
-  CPPUNIT_ASSERT_EQUAL((int32_t)1024, pm->getLength());
+  SharedHandle<BtRequestMessage> pm = BtRequestMessage::create(&msg[4], 13);
+  CPPUNIT_ASSERT_EQUAL((uint8_t)6, pm->getId());
+  CPPUNIT_ASSERT_EQUAL((size_t)12345, pm->getIndex());
+  CPPUNIT_ASSERT_EQUAL((uint32_t)256, pm->getBegin());
+  CPPUNIT_ASSERT_EQUAL((uint32_t)1024, pm->getLength());
 
   // case: payload size is wrong
   try {
@@ -168,63 +178,66 @@ void BtRequestMessageTest::testGetMessage() {
 }
 
 void BtRequestMessageTest::testDoReceivedAction_hasPieceAndAmNotChoking() {
-  peer->amChoking = false;
+  _peer->amChoking(false);
   msg->doReceivedAction();
   
-  CPPUNIT_ASSERT_EQUAL((size_t)1, dispatcher->messageQueue.size());
-  MockBtMessage2* pieceMsg = (MockBtMessage2*)dispatcher->messageQueue.front().get();
-  CPPUNIT_ASSERT_EQUAL(string("piece"), pieceMsg->type);
-  CPPUNIT_ASSERT_EQUAL((int32_t)1, pieceMsg->index);
-  CPPUNIT_ASSERT_EQUAL((int32_t)16, pieceMsg->begin);
-  CPPUNIT_ASSERT_EQUAL((int32_t)32, pieceMsg->length);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, _dispatcher->messageQueue.size());
+  MockBtMessage2* pieceMsg =
+    (MockBtMessage2*)_dispatcher->messageQueue.front().get();
+  CPPUNIT_ASSERT_EQUAL(std::string("piece"), pieceMsg->type);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, pieceMsg->index);
+  CPPUNIT_ASSERT_EQUAL((uint32_t)16, pieceMsg->begin);
+  CPPUNIT_ASSERT_EQUAL((size_t)32, pieceMsg->length);
 }
 
 void BtRequestMessageTest::testDoReceivedAction_hasPieceAndAmChokingAndFastExtensionEnabled() {
-  peer->amChoking = true;
-  peer->setFastExtensionEnabled(true);
+  _peer->amChoking(true);
+  _peer->setFastExtensionEnabled(true);
   msg->doReceivedAction();
   
-  CPPUNIT_ASSERT_EQUAL((size_t)1, dispatcher->messageQueue.size());
-  MockBtMessage2* pieceMsg = (MockBtMessage2*)dispatcher->messageQueue.front().get();
-  CPPUNIT_ASSERT_EQUAL(string("reject"), pieceMsg->type);
-  CPPUNIT_ASSERT_EQUAL((int32_t)1, pieceMsg->index);
-  CPPUNIT_ASSERT_EQUAL((int32_t)16, pieceMsg->begin);
-  CPPUNIT_ASSERT_EQUAL((int32_t)32, pieceMsg->length);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, _dispatcher->messageQueue.size());
+  MockBtMessage2* pieceMsg =
+    (MockBtMessage2*)_dispatcher->messageQueue.front().get();
+  CPPUNIT_ASSERT_EQUAL(std::string("reject"), pieceMsg->type);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, pieceMsg->index);
+  CPPUNIT_ASSERT_EQUAL((uint32_t)16, pieceMsg->begin);
+  CPPUNIT_ASSERT_EQUAL((size_t)32, pieceMsg->length);
 }
 
 void BtRequestMessageTest::testDoReceivedAction_hasPieceAndAmChokingAndFastExtensionDisabled() {
-  peer->amChoking = true;
+  _peer->amChoking(true);
   msg->doReceivedAction();
   
-  CPPUNIT_ASSERT_EQUAL((size_t)0, dispatcher->messageQueue.size());
+  CPPUNIT_ASSERT_EQUAL((size_t)0, _dispatcher->messageQueue.size());
 }
 
 void BtRequestMessageTest::testDoReceivedAction_doesntHavePieceAndFastExtensionEnabled() {
   msg->setIndex(2);
-  peer->amChoking = false;
-  peer->setFastExtensionEnabled(true);
+  _peer->amChoking(false);
+  _peer->setFastExtensionEnabled(true);
   msg->doReceivedAction();
   
-  CPPUNIT_ASSERT_EQUAL((size_t)1, dispatcher->messageQueue.size());
-  MockBtMessage2* pieceMsg = (MockBtMessage2*)dispatcher->messageQueue.front().get();
-  CPPUNIT_ASSERT_EQUAL(string("reject"), pieceMsg->type);
-  CPPUNIT_ASSERT_EQUAL((int32_t)2, pieceMsg->index);
-  CPPUNIT_ASSERT_EQUAL((int32_t)16, pieceMsg->begin);
-  CPPUNIT_ASSERT_EQUAL((int32_t)32, pieceMsg->length);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, _dispatcher->messageQueue.size());
+  MockBtMessage2* pieceMsg =
+    (MockBtMessage2*)_dispatcher->messageQueue.front().get();
+  CPPUNIT_ASSERT_EQUAL(std::string("reject"), pieceMsg->type);
+  CPPUNIT_ASSERT_EQUAL((size_t)2, pieceMsg->index);
+  CPPUNIT_ASSERT_EQUAL((uint32_t)16, pieceMsg->begin);
+  CPPUNIT_ASSERT_EQUAL((size_t)32, pieceMsg->length);
 }
 
 void BtRequestMessageTest::testDoReceivedAction_doesntHavePieceAndFastExtensionDisabled() {
   msg->setIndex(2);
-  peer->amChoking = false;
+  _peer->amChoking(false);
   msg->doReceivedAction();
   
-  CPPUNIT_ASSERT_EQUAL((size_t)0, dispatcher->messageQueue.size());
+  CPPUNIT_ASSERT_EQUAL((size_t)0, _dispatcher->messageQueue.size());
 }
 
 void BtRequestMessageTest::testHandleAbortRequestEvent() {
-  PieceHandle piece = new Piece(1, 16*1024);
-  BtAbortOutstandingRequestEventHandle event =
-    new BtAbortOutstandingRequestEvent(piece);
+  SharedHandle<Piece> piece(new Piece(1, 16*1024));
+  SharedHandle<BtAbortOutstandingRequestEvent> event
+    (new BtAbortOutstandingRequestEvent(piece));
 
   CPPUNIT_ASSERT(!msg->isInvalidate());
   msg->handleEvent(event);
@@ -233,9 +246,9 @@ void BtRequestMessageTest::testHandleAbortRequestEvent() {
 }
 
 void BtRequestMessageTest::testHandleAbortRequestEvent_indexNoMatch() {
-  PieceHandle piece = new Piece(2, 16*1024);
-  BtAbortOutstandingRequestEventHandle event =
-    new BtAbortOutstandingRequestEvent(piece);
+  SharedHandle<Piece> piece(new Piece(2, 16*1024));
+  SharedHandle<BtAbortOutstandingRequestEvent> event
+    (new BtAbortOutstandingRequestEvent(piece));
 
   CPPUNIT_ASSERT(!msg->isInvalidate());
   CPPUNIT_ASSERT(!msg->isSendingInProgress());
@@ -244,9 +257,9 @@ void BtRequestMessageTest::testHandleAbortRequestEvent_indexNoMatch() {
 }
 
 void BtRequestMessageTest::testHandleAbortRequestEvent_alreadyInvalidated() {
-  PieceHandle piece = new Piece(1, 16*1024);
-  BtAbortOutstandingRequestEventHandle event =
-    new BtAbortOutstandingRequestEvent(piece);
+  SharedHandle<Piece> piece(new Piece(1, 16*1024));
+  SharedHandle<BtAbortOutstandingRequestEvent> event
+    (new BtAbortOutstandingRequestEvent(piece));
   msg->setInvalidate(true);
 
   CPPUNIT_ASSERT(msg->isInvalidate());
@@ -256,9 +269,9 @@ void BtRequestMessageTest::testHandleAbortRequestEvent_alreadyInvalidated() {
 }
 
 void BtRequestMessageTest::testHandleAbortRequestEvent_sendingInProgress() {
-  PieceHandle piece = new Piece(1, 16*1024);
-  BtAbortOutstandingRequestEventHandle event =
-    new BtAbortOutstandingRequestEvent(piece);
+  SharedHandle<Piece> piece(new Piece(1, 16*1024));
+  SharedHandle<BtAbortOutstandingRequestEvent> event
+    (new BtAbortOutstandingRequestEvent(piece));
   msg->setSendingInProgress(true);
 
   CPPUNIT_ASSERT(!msg->isInvalidate());
@@ -268,6 +281,33 @@ void BtRequestMessageTest::testHandleAbortRequestEvent_sendingInProgress() {
 }
 
 void BtRequestMessageTest::testToString() {
-  CPPUNIT_ASSERT_EQUAL(string("request index=1, begin=16, length=32"),
+  CPPUNIT_ASSERT_EQUAL(std::string("request index=1, begin=16, length=32"),
 		       msg->toString());
 }
+
+void BtRequestMessageTest::testValidate() {
+  BtRequestMessage msg(0, 0, 16*1024);
+  msg.setBtMessageValidator
+    (SharedHandle<BtMessageValidator>
+     (new BtRequestMessageValidator(&msg, 1024, 256*1024)));
+  std::deque<std::string> errors;
+
+  msg.validate(errors);
+}
+
+void BtRequestMessageTest::testValidate_lengthTooLong() {
+  BtRequestMessage msg(0, 0, 16*1024+1);
+  msg.setBtMessageValidator
+    (SharedHandle<BtMessageValidator>
+     (new BtRequestMessageValidator(&msg, 1024, 256*1024)));
+  std::deque<std::string> errors;
+  try {
+    msg.validate(errors);
+    CPPUNIT_FAIL("exception must be thrown.");
+  } catch(DlAbortEx& e) {
+    CPPUNIT_ASSERT_EQUAL(std::string("Length too long: 16385 > 16KB"),
+			 std::string(e.what()));
+  }
+}
+
+} // namespace aria2
