@@ -1,17 +1,21 @@
 #include "DefaultBtMessageDispatcher.h"
+
+#include <cassert>
+
+#include <cppunit/extensions/HelperMacros.h>
+
 #include "Util.h"
 #include "Exception.h"
 #include "MockPieceStorage.h"
 #include "MockPeerStorage.h"
-#include "BtRegistry.h"
 #include "DefaultBtContext.h"
 #include "MockBtMessage.h"
 #include "MockBtMessageFactory.h"
 #include "prefs.h"
 #include "BtCancelSendingPieceEvent.h"
-#include <cppunit/extensions/HelperMacros.h>
+#include "BtHandshakeMessage.h"
 
-using namespace std;
+namespace aria2 {
 
 class DefaultBtMessageDispatcherTest:public CppUnit::TestFixture {
 
@@ -32,14 +36,13 @@ class DefaultBtMessageDispatcherTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testRemoveOutstandingRequest);
   CPPUNIT_TEST_SUITE_END();
 private:
-  BtContextHandle btContext;
-  PeerHandle peer;
-  DefaultBtMessageDispatcherHandle btMessageDispatcher;
-  MockPeerStorageHandle peerStorage;
-  MockPieceStorageHandle pieceStorage;
+  SharedHandle<BtContext> btContext;
+  SharedHandle<Peer> peer;
+  SharedHandle<DefaultBtMessageDispatcher> btMessageDispatcher;
+  SharedHandle<MockPeerStorage> peerStorage;
+  SharedHandle<MockPieceStorage> pieceStorage;
+  SharedHandle<MockBtMessageFactory> _messageFactory;
 public:
-  DefaultBtMessageDispatcherTest():btContext(0), peer(0), btMessageDispatcher(0) {}
-
   void tearDown() {}
 
   void testAddMessage();
@@ -63,7 +66,7 @@ public:
     bool sendCalled;
     bool doCancelActionCalled;
   public:
-    string type;
+    std::string type;
   public:
     MockBtMessage2():onQueuedCalled(false),
 		     sendCalled(false),
@@ -88,7 +91,7 @@ public:
       return sendCalled;
     }
 
-    virtual void handleEvent(const BtEventHandle& event) {
+    virtual void handleEvent(const SharedHandle<BtEvent>& event) {
       BtCancelSendingPieceEvent* e =
 	dynamic_cast<BtCancelSendingPieceEvent*>(event.get());
       if(e) {
@@ -101,58 +104,48 @@ public:
     }  
   };
 
-  typedef SharedHandle<MockBtMessage2> MockBtMessage2Handle;
-
   class MockPieceStorage2 : public MockPieceStorage {
   private:
-    PieceHandle piece;
+    SharedHandle<Piece> piece;
   public:
-    virtual PieceHandle getPiece(int index) {
+    virtual SharedHandle<Piece> getPiece(size_t index) {
       return piece;
     }
 
-    void setPiece(const PieceHandle& piece) {
+    void setPiece(const SharedHandle<Piece>& piece) {
       this->piece = piece;
     }
   };
 
   class MockBtMessageFactory2 : public MockBtMessageFactory {
   public:
-    virtual BtMessageHandle
-    createCancelMessage(int32_t index, int32_t begin, int32_t length) {
-      MockBtMessage2Handle btMsg = new MockBtMessage2();
+    virtual SharedHandle<BtMessage>
+    createCancelMessage(size_t index, uint32_t begin, size_t length) {
+      SharedHandle<MockBtMessage2> btMsg(new MockBtMessage2());
       btMsg->type = "cancel";
       return btMsg;
     }
   };
 
   void setUp() {
-    btContext = new DefaultBtContext();
+    btContext.reset(new DefaultBtContext());
     btContext->load("test.torrent");
-    peer = new Peer("192.168.0.1", 6969,
-		    btContext->getPieceLength(),
-		    btContext->getTotalLength());    
-    peerStorage = new MockPeerStorage();
-    pieceStorage = new MockPieceStorage();
-    BtRegistry::unregisterAll();
-    BtRegistry::registerPeerStorage(btContext->getInfoHashAsString(),
-				    peerStorage);
-    BtRegistry::registerPieceStorage(btContext->getInfoHashAsString(),
-				     pieceStorage);
-    BtRegistry::registerPeerObjectCluster(btContext->getInfoHashAsString(),
-					  new PeerObjectCluster());
+    peer.reset(new Peer("192.168.0.1", 6969));
+    peer->allocateSessionResource(btContext->getPieceLength(),
+				  btContext->getTotalLength());
+    peerStorage.reset(new MockPeerStorage());
+    pieceStorage.reset(new MockPieceStorage());
 
-    PeerObjectHandle peerObject = new PeerObject();
-    peerObject->btMessageFactory = new MockBtMessageFactory2();
+    _messageFactory.reset(new MockBtMessageFactory2());
 
-    PEER_OBJECT_CLUSTER(btContext)->registerHandle(peer->getId(), peerObject);
-
-    btMessageDispatcher = new DefaultBtMessageDispatcher();
-    btMessageDispatcher->setCuid(1);
-    btMessageDispatcher->setBtContext(btContext);
+    btMessageDispatcher.reset(new DefaultBtMessageDispatcher());
     btMessageDispatcher->setPeer(peer);
+    btMessageDispatcher->setBtContext(btContext);
+    btMessageDispatcher->setPieceStorage(pieceStorage);
+    btMessageDispatcher->setPeerStorage(peerStorage);
+    btMessageDispatcher->setBtMessageFactory(_messageFactory);
+    btMessageDispatcher->setCuid(1);
     btMessageDispatcher->setMaxUploadSpeedLimit(0);
-    btMessageDispatcher->setBtMessageFactory(peerObject->btMessageFactory);
   }
 };
 
@@ -160,7 +153,7 @@ public:
 CPPUNIT_TEST_SUITE_REGISTRATION(DefaultBtMessageDispatcherTest);
 
 void DefaultBtMessageDispatcherTest::testAddMessage() {
-  MockBtMessage2Handle msg = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg(new MockBtMessage2());
   CPPUNIT_ASSERT_EQUAL(false, msg->isOnQueuedCalled());
   btMessageDispatcher->addMessageToQueue(msg);
   CPPUNIT_ASSERT_EQUAL(true, msg->isOnQueuedCalled());
@@ -173,10 +166,10 @@ void DefaultBtMessageDispatcherTest::testSendMessages() {
   stat.setUploadSpeed(0);
   peerStorage->setStat(stat);
 
-  MockBtMessage2Handle msg1 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg1(new MockBtMessage2());
   msg1->setSendingInProgress(false);
   msg1->setUploading(false);
-  MockBtMessage2Handle msg2 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg2(new MockBtMessage2());
   msg2->setSendingInProgress(false);
   msg2->setUploading(false);
   btMessageDispatcher->addMessageToQueue(msg1);
@@ -192,10 +185,10 @@ void DefaultBtMessageDispatcherTest::testSendMessages_underUploadLimit() {
   stat.setUploadSpeed(0);
   peerStorage->setStat(stat);
 
-  MockBtMessage2Handle msg1 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg1(new MockBtMessage2());
   msg1->setSendingInProgress(false);
   msg1->setUploading(true);
-  MockBtMessage2Handle msg2 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg2(new MockBtMessage2());
   msg2->setSendingInProgress(false);
   msg2->setUploading(true);
   btMessageDispatcher->addMessageToQueue(msg1);
@@ -212,13 +205,13 @@ void DefaultBtMessageDispatcherTest::testSendMessages_overUploadLimit() {
   stat.setUploadSpeed(150);
   peerStorage->setStat(stat);
 
-  MockBtMessage2Handle msg1 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg1(new MockBtMessage2());
   msg1->setSendingInProgress(false);
   msg1->setUploading(true);
-  MockBtMessage2Handle msg2 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg2(new MockBtMessage2());
   msg2->setSendingInProgress(false);
   msg2->setUploading(true);
-  MockBtMessage2Handle msg3 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg3(new MockBtMessage2());
   msg3->setSendingInProgress(false);
   msg3->setUploading(false);
 
@@ -231,17 +224,18 @@ void DefaultBtMessageDispatcherTest::testSendMessages_overUploadLimit() {
   CPPUNIT_ASSERT(!msg2->isSendCalled());
   CPPUNIT_ASSERT(msg3->isSendCalled());
 
-  CPPUNIT_ASSERT_EQUAL((size_t)2, btMessageDispatcher->getMessageQueue().size());
+  CPPUNIT_ASSERT_EQUAL((size_t)2,
+		       btMessageDispatcher->getMessageQueue().size());
 }
 
 void DefaultBtMessageDispatcherTest::testSendMessages_sendingInProgress() {
-  MockBtMessage2Handle msg1 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg1(new MockBtMessage2());
   msg1->setSendingInProgress(false);
   msg1->setUploading(false);
-  MockBtMessage2Handle msg2 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg2(new MockBtMessage2());
   msg2->setSendingInProgress(true);
   msg2->setUploading(false);
-  MockBtMessage2Handle msg3 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg3(new MockBtMessage2());
   msg3->setSendingInProgress(false);
   msg3->setUploading(false);
 
@@ -255,12 +249,13 @@ void DefaultBtMessageDispatcherTest::testSendMessages_sendingInProgress() {
   CPPUNIT_ASSERT(msg2->isSendCalled());
   CPPUNIT_ASSERT(!msg3->isSendCalled());
 
-  CPPUNIT_ASSERT_EQUAL((size_t)2, btMessageDispatcher->getMessageQueue().size());
+  CPPUNIT_ASSERT_EQUAL((size_t)2,
+		       btMessageDispatcher->getMessageQueue().size());
 }
 
 void DefaultBtMessageDispatcherTest::testDoCancelSendingPieceAction() {
-  MockBtMessage2Handle msg1 = new MockBtMessage2();
-  MockBtMessage2Handle msg2 = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg1(new MockBtMessage2());
+  SharedHandle<MockBtMessage2> msg2(new MockBtMessage2());
 
   btMessageDispatcher->addMessageToQueue(msg1);
   btMessageDispatcher->addMessageToQueue(msg2);
@@ -274,92 +269,79 @@ void DefaultBtMessageDispatcherTest::testDoCancelSendingPieceAction() {
 int MY_PIECE_LENGTH = 16*1024;
 
 void DefaultBtMessageDispatcherTest::testCheckRequestSlotAndDoNecessaryThing() {
-  RequestSlot slot(0, 0, MY_PIECE_LENGTH, 0);
+  SharedHandle<Piece> piece(new Piece(0, MY_PIECE_LENGTH));
+  RequestSlot slot(0, 0, MY_PIECE_LENGTH, 0, piece);
   
-  PieceHandle piece = new Piece(0, MY_PIECE_LENGTH);
-  assert(piece->getMissingUnusedBlockIndex() == 0);
+  size_t index;
+  CPPUNIT_ASSERT(piece->getMissingUnusedBlockIndex(index));
+  CPPUNIT_ASSERT_EQUAL((size_t)0, index);
 
-  SharedHandle<MockPieceStorage2> pieceStorage = new MockPieceStorage2();
+  SharedHandle<MockPieceStorage2> pieceStorage(new MockPieceStorage2());
   pieceStorage->setPiece(piece);
-  BtRegistry::registerPieceStorage(btContext->getInfoHashAsString(),
-				   pieceStorage);
-
-  btMessageDispatcher = new DefaultBtMessageDispatcher();
-  btMessageDispatcher->setCuid(1);
-  btMessageDispatcher->setBtContext(btContext);
-  btMessageDispatcher->setPeer(peer);
-  btMessageDispatcher->setRequestTimeout(60);
   
+  btMessageDispatcher->setRequestTimeout(60);
+  btMessageDispatcher->setPieceStorage(pieceStorage);
   btMessageDispatcher->addOutstandingRequest(slot);
 
   btMessageDispatcher->checkRequestSlotAndDoNecessaryThing();
 
-  CPPUNIT_ASSERT_EQUAL((size_t)0, btMessageDispatcher->getMessageQueue().size());
-  CPPUNIT_ASSERT_EQUAL((size_t)1, btMessageDispatcher->getRequestSlots().size());
+  CPPUNIT_ASSERT_EQUAL((size_t)0,
+		       btMessageDispatcher->getMessageQueue().size());
+  CPPUNIT_ASSERT_EQUAL((size_t)1,
+		       btMessageDispatcher->getRequestSlots().size());
 }
 
 void DefaultBtMessageDispatcherTest::testCheckRequestSlotAndDoNecessaryThing_timeout() {
-  RequestSlot slot(0, 0, MY_PIECE_LENGTH, 0);
+  SharedHandle<Piece> piece(new Piece(0, MY_PIECE_LENGTH));
+  RequestSlot slot(0, 0, MY_PIECE_LENGTH, 0, piece);
   // make this slot timeout
   slot.setDispatchedTime(0);
 
-  PieceHandle piece = new Piece(0, MY_PIECE_LENGTH);
-  assert(piece->getMissingUnusedBlockIndex() == 0);
+  size_t index;
+  CPPUNIT_ASSERT(piece->getMissingUnusedBlockIndex(index));
+  CPPUNIT_ASSERT_EQUAL((size_t)0, index);
 
-  SharedHandle<MockPieceStorage2> pieceStorage = new MockPieceStorage2();
+  SharedHandle<MockPieceStorage2> pieceStorage(new MockPieceStorage2());
   pieceStorage->setPiece(piece);
-  BtRegistry::registerPieceStorage(btContext->getInfoHashAsString(),
-				   pieceStorage);
 
-  btMessageDispatcher = new DefaultBtMessageDispatcher();
-  btMessageDispatcher->setCuid(1);
-  btMessageDispatcher->setBtContext(btContext);
-  btMessageDispatcher->setPeer(peer);
   btMessageDispatcher->setRequestTimeout(60);
-  btMessageDispatcher->setBtMessageFactory(BT_MESSAGE_FACTORY(btContext,
-							      peer));
-
+  btMessageDispatcher->setPieceStorage(pieceStorage);
   btMessageDispatcher->addOutstandingRequest(slot);
 
   btMessageDispatcher->checkRequestSlotAndDoNecessaryThing();
 
-  CPPUNIT_ASSERT_EQUAL((size_t)0, btMessageDispatcher->getMessageQueue().size());
-  CPPUNIT_ASSERT_EQUAL((size_t)0, btMessageDispatcher->getRequestSlots().size());
+  CPPUNIT_ASSERT_EQUAL((size_t)0,
+		       btMessageDispatcher->getMessageQueue().size());
+  CPPUNIT_ASSERT_EQUAL((size_t)0,
+		       btMessageDispatcher->getRequestSlots().size());
   CPPUNIT_ASSERT_EQUAL(false, piece->isBlockUsed(0));
-  CPPUNIT_ASSERT_EQUAL(true, peer->snubbing);
+  CPPUNIT_ASSERT_EQUAL(true, peer->snubbing());
 }
 
 void DefaultBtMessageDispatcherTest::testCheckRequestSlotAndDoNecessaryThing_completeBlock() {
-  RequestSlot slot(0, 0, MY_PIECE_LENGTH, 0);
-  
-  PieceHandle piece = new Piece(0, MY_PIECE_LENGTH);
+  SharedHandle<Piece> piece(new Piece(0, MY_PIECE_LENGTH));
   piece->completeBlock(0);
 
-  SharedHandle<MockPieceStorage2> pieceStorage = new MockPieceStorage2();
+  RequestSlot slot(0, 0, MY_PIECE_LENGTH, 0, piece);
+  
+  SharedHandle<MockPieceStorage2> pieceStorage(new MockPieceStorage2());
   pieceStorage->setPiece(piece);
-  BtRegistry::registerPieceStorage(btContext->getInfoHashAsString(),
-				   pieceStorage);
 
-
-  btMessageDispatcher = new DefaultBtMessageDispatcher();
-  btMessageDispatcher->setCuid(1);
-  btMessageDispatcher->setBtContext(btContext);
-  btMessageDispatcher->setPeer(peer);
   btMessageDispatcher->setRequestTimeout(60);
-  btMessageDispatcher->setBtMessageFactory(BT_MESSAGE_FACTORY(btContext,
-							      peer));
-
+  btMessageDispatcher->setPieceStorage(pieceStorage);
   btMessageDispatcher->addOutstandingRequest(slot);
 
   btMessageDispatcher->checkRequestSlotAndDoNecessaryThing();
 
-  CPPUNIT_ASSERT_EQUAL((size_t)1, btMessageDispatcher->getMessageQueue().size());
-  CPPUNIT_ASSERT_EQUAL((size_t)0, btMessageDispatcher->getRequestSlots().size());
+  CPPUNIT_ASSERT_EQUAL((size_t)1,
+		       btMessageDispatcher->getMessageQueue().size());
+  CPPUNIT_ASSERT_EQUAL((size_t)0,
+		       btMessageDispatcher->getRequestSlots().size());
 }
 
 void DefaultBtMessageDispatcherTest::testIsSendingInProgress() {
   CPPUNIT_ASSERT(!btMessageDispatcher->isSendingInProgress());
-  MockBtMessage2Handle msg = new MockBtMessage2();
+  SharedHandle<MockBtMessage2> msg(new MockBtMessage2());
   msg->setSendingInProgress(false);
   btMessageDispatcher->addMessageToQueue(msg);
   CPPUNIT_ASSERT(!btMessageDispatcher->isSendingInProgress());
@@ -370,7 +352,8 @@ void DefaultBtMessageDispatcherTest::testIsSendingInProgress() {
 void DefaultBtMessageDispatcherTest::testCountOutstandingRequest() {
   RequestSlot slot(0, 0, MY_PIECE_LENGTH, 0);
   btMessageDispatcher->addOutstandingRequest(slot);
-  CPPUNIT_ASSERT_EQUAL((int32_t)1, btMessageDispatcher->countOutstandingRequest());
+  CPPUNIT_ASSERT_EQUAL((size_t)1,
+		       btMessageDispatcher->countOutstandingRequest());
 }
 
 void DefaultBtMessageDispatcherTest::testIsOutstandingRequest() {
@@ -393,7 +376,8 @@ void DefaultBtMessageDispatcherTest::testGetOutstandingRequest() {
   RequestSlot s3 = btMessageDispatcher->getOutstandingRequest(1, 1024, 17*1024);
   CPPUNIT_ASSERT(RequestSlot::isNull(s3));
 
-  RequestSlot s4 = btMessageDispatcher->getOutstandingRequest(1, 2*1024, 16*1024);
+  RequestSlot s4 =
+    btMessageDispatcher->getOutstandingRequest(1, 2*1024, 16*1024);
   CPPUNIT_ASSERT(RequestSlot::isNull(s4));
 
   RequestSlot s5 = btMessageDispatcher->getOutstandingRequest(2, 1024, 16*1024);
@@ -401,14 +385,25 @@ void DefaultBtMessageDispatcherTest::testGetOutstandingRequest() {
 }
 
 void DefaultBtMessageDispatcherTest::testRemoveOutstandingRequest() {
-  RequestSlot slot(1, 1024, 16*1024, 10);
+  SharedHandle<Piece> piece(new Piece(1, 1024*1024));
+  size_t blockIndex = 0;
+  CPPUNIT_ASSERT(piece->getMissingUnusedBlockIndex(blockIndex));
+  uint32_t begin = blockIndex*piece->getBlockLength();
+  size_t length = piece->getBlockLength(blockIndex);
+  RequestSlot slot(piece->getIndex(), begin, length, blockIndex, piece);
   btMessageDispatcher->addOutstandingRequest(slot);
 
-  RequestSlot s2 = btMessageDispatcher->getOutstandingRequest(1, 1024, 16*1024);
+  RequestSlot s2 = btMessageDispatcher->getOutstandingRequest
+    (piece->getIndex(), begin, length);
   CPPUNIT_ASSERT(!RequestSlot::isNull(s2));
+  CPPUNIT_ASSERT(piece->isBlockUsed(blockIndex));
 
   btMessageDispatcher->removeOutstandingRequest(s2);
 
-  RequestSlot s3 = btMessageDispatcher->getOutstandingRequest(1, 1024, 16*1024);
+  RequestSlot s3 = btMessageDispatcher->getOutstandingRequest
+    (piece->getIndex(), begin, length);
   CPPUNIT_ASSERT(RequestSlot::isNull(s3));  
+  CPPUNIT_ASSERT(!piece->isBlockUsed(blockIndex));
 }
+
+} // namespace aria2
