@@ -1,7 +1,7 @@
 #include "DefaultDiskWriter.h"
 #include <cppunit/extensions/HelperMacros.h>
 
-using namespace std;
+namespace aria2 {
 
 class DefaultDiskWriterTest:public CppUnit::TestFixture {
 
@@ -24,6 +24,7 @@ void DefaultDiskWriterTest::testSize()
 {
   DefaultDiskWriter dw;
   dw.openExistingFile("4096chunk.txt");
-  CPPUNIT_ASSERT_EQUAL((int64_t)4096, dw.size());
+  CPPUNIT_ASSERT_EQUAL((uint64_t)4096ULL, dw.size());
 }
 
+} // namespace aria2
