@@ -1,11 +1,21 @@
 #include "HttpRequest.h"
+
+#include <sstream>
+
+#include <cppunit/extensions/HelperMacros.h>
+
 #include "prefs.h"
 #include "AuthConfigFactory.h"
 #include "PiecedSegment.h"
 #include "Piece.h"
-#include <cppunit/extensions/HelperMacros.h>
+#include "Range.h"
+#include "Request.h"
+#include "Option.h"
+#include "array_fun.h"
+#include "CookieStorage.h"
+#include "Util.h"
 
-using namespace std;
+namespace aria2 {
 
 class HttpRequestTest : public CppUnit::TestFixture {
 
@@ -15,23 +25,38 @@ class HttpRequestTest : public CppUnit::TestFixture {
   CPPUNIT_TEST(testCreateRequest);
   CPPUNIT_TEST(testCreateRequest_ftp);
   CPPUNIT_TEST(testCreateRequest_with_cookie);
+  CPPUNIT_TEST(testCreateRequest_query);
+  CPPUNIT_TEST(testCreateRequest_head);
   CPPUNIT_TEST(testCreateProxyRequest);
   CPPUNIT_TEST(testIsRangeSatisfied);
   CPPUNIT_TEST(testUserAgent);
+  CPPUNIT_TEST(testAddHeader);
+  CPPUNIT_TEST(testAddAcceptType);
+  CPPUNIT_TEST(testEnableAcceptEncoding);
   CPPUNIT_TEST_SUITE_END();
 private:
-
+  SharedHandle<Option> _option;
+  SharedHandle<AuthConfigFactory> _authConfigFactory;
 public:
-  void setUp() {}
+  void setUp()
+  {
+    _option.reset(new Option());    
+    _authConfigFactory.reset(new AuthConfigFactory(_option.get()));
+  }
 
   void testGetStartByte();
   void testGetEndByte();
   void testCreateRequest();
   void testCreateRequest_ftp();
   void testCreateRequest_with_cookie();
+  void testCreateRequest_query();
+  void testCreateRequest_head();
   void testCreateProxyRequest();
   void testIsRangeSatisfied();
   void testUserAgent();
+  void testAddHeader();
+  void testAddAcceptType();
+  void testEnableAcceptEncoding();
 };
 
 
@@ -40,74 +65,70 @@ CPPUNIT_TEST_SUITE_REGISTRATION( HttpRequestTest );
 void HttpRequestTest::testGetStartByte()
 {
   HttpRequest httpRequest;
-  SegmentHandle segment = new PiecedSegment(1024, new Piece(1, 1024));
+  SharedHandle<Piece> p(new Piece(1, 1024));
+  SharedHandle<Segment> segment(new PiecedSegment(1024, p));
 
-  CPPUNIT_ASSERT_EQUAL((int64_t)0, httpRequest.getStartByte());
+  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getStartByte());
 
   httpRequest.setSegment(segment);
   
-  CPPUNIT_ASSERT_EQUAL((int64_t)1024, httpRequest.getStartByte());
+  CPPUNIT_ASSERT_EQUAL((off_t)1024LL, httpRequest.getStartByte());
 
 }
 
 void HttpRequestTest::testGetEndByte()
 {
-  int32_t index = 1;
-  int32_t length = 1024*1024-1024;
-  int32_t segmentLength = 1024*1024;
+  size_t index = 1;
+  size_t length = 1024*1024-1024;
+  size_t segmentLength = 1024*1024;
 
   HttpRequest httpRequest;
-  SegmentHandle segment = new PiecedSegment(segmentLength,
-					    new Piece(index, length));
-  
+  SharedHandle<Piece> piece(new Piece(index, length));
+  SharedHandle<Segment> segment(new PiecedSegment(segmentLength, piece));
 
-  CPPUNIT_ASSERT_EQUAL((int64_t)0, httpRequest.getEndByte());
+  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getEndByte());
 
   httpRequest.setSegment(segment);
 
-  CPPUNIT_ASSERT_EQUAL((int64_t)0, httpRequest.getEndByte());
+  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getEndByte());
 
-  RequestHandle request = new Request();
-  request->setKeepAlive(true);
+  SharedHandle<Request> request(new Request());
+  request->supportsPersistentConnection(true);
+  request->setPipeliningHint(true);
 
   httpRequest.setRequest(request);
 
-  CPPUNIT_ASSERT_EQUAL((int64_t)segmentLength*index+length-1,
+  CPPUNIT_ASSERT_EQUAL((off_t)(segmentLength*index+length-1),
 		       httpRequest.getEndByte());
 
 
-  request->setKeepAlive(false);
+  request->setPipeliningHint(false);
 
-  CPPUNIT_ASSERT_EQUAL((int64_t)0, httpRequest.getEndByte());
+  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getEndByte());
 }
 
 void HttpRequestTest::testCreateRequest()
 {
-  Option option;
-  option.put(PREF_HTTP_AUTH_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
-  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_USER, "aria2user");
-  option.put(PREF_HTTP_PASSWD, "aria2passwd");
-  option.put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
-  option.put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
+  SharedHandle<Piece> p;
 
-  AuthConfigFactoryHandle authConfigFactory = new AuthConfigFactory(&option);
-  AuthConfigFactorySingleton::instance(authConfigFactory);
+  SharedHandle<Request> request(new Request());
+  request->supportsPersistentConnection(true);
 
-  RequestHandle request = new Request();
   request->setUrl("http://localhost:8080/archives/aria2-1.0.0.tar.bz2");
 
-  SegmentHandle segment = new PiecedSegment(1024, new Piece(0, 1024));
+  p.reset(new Piece(0, 1024));
+  SharedHandle<Segment> segment(new PiecedSegment(1024, p));
 
   HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
 
-  request->setKeepAlive(true);
+  // remove "Connection: close" and add end byte range
+  request->setPipeliningHint(true);  
   
-  string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+  std::string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
     "Host: localhost:8080\r\n"
@@ -118,7 +139,7 @@ void HttpRequestTest::testCreateRequest()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  request->setKeepAlive(false);
+  request->setPipeliningHint(false);
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -131,7 +152,8 @@ void HttpRequestTest::testCreateRequest()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  segment = new PiecedSegment(1024*1024, new Piece(1, 1024*1024));
+  p.reset(new Piece(1, 1024*1024));
+  segment.reset(new PiecedSegment(1024*1024, p));
   httpRequest.setSegment(segment);
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
@@ -146,7 +168,7 @@ void HttpRequestTest::testCreateRequest()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  request->setKeepAlive(true);
+  request->setPipeliningHint(true);
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -159,8 +181,7 @@ void HttpRequestTest::testCreateRequest()
   
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  request->setKeepAlive(false);
-
+  // redirection set persistent connection flag to true
   request->redirectUrl("http://localhost:8080/archives/download/aria2-1.0.0.tar.bz2");
 
   expectedText = "GET /archives/download/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
@@ -169,21 +190,39 @@ void HttpRequestTest::testCreateRequest()
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
-    "Connection: close\r\n"
+    "Range: bytes=1048576-2097151\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
+
+  request->supportsPersistentConnection(true);
+  request->setPipeliningHint(false);
+
+  // this only removes "Connection: close".
+  request->setKeepAliveHint(true);
+
+  expectedText = "GET /archives/download/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*\r\n"
+    "Host: localhost:8080\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
     "Range: bytes=1048576-\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
-  
+
+  request->setKeepAliveHint(false);
+
   request->resetUrl();
 
-  segment = new PiecedSegment(1024*1024, new Piece(0, 1024*1024));
+  p.reset(new Piece(0, 1024*1024));
+  segment.reset(new PiecedSegment(1024*1024, p));
   httpRequest.setSegment(segment);
 
   // enable http auth
-  option.put(PREF_HTTP_AUTH_ENABLED, V_TRUE);
-  
-  httpRequest.configure(&option);
+  _option->put(PREF_HTTP_USER, "aria2user");
+  _option->put(PREF_HTTP_PASSWD, "aria2passwd");
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -198,41 +237,10 @@ void HttpRequestTest::testCreateRequest()
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
   // enable http proxy auth
-  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
-
-  httpRequest.configure(&option);
-
-  expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
-    "User-Agent: aria2\r\n"
-    "Accept: */*\r\n"
-    "Host: localhost:8080\r\n"
-    "Pragma: no-cache\r\n"
-    "Cache-Control: no-cache\r\n"
-    "Connection: close\r\n"
-    "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
-    "\r\n";
-
-  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
-
-  option.put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
-
-  httpRequest.configure(&option);
-
-  expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
-    "User-Agent: aria2\r\n"
-    "Accept: */*\r\n"
-    "Host: localhost:8080\r\n"
-    "Pragma: no-cache\r\n"
-    "Cache-Control: no-cache\r\n"
-    "Connection: close\r\n"
-    "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
-    "\r\n";
-
-  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
-
-  option.put(PREF_HTTP_PROXY_METHOD, V_GET);
-
-  httpRequest.configure(&option);
+  SharedHandle<Request> proxyRequest(new Request());
+  CPPUNIT_ASSERT(proxyRequest->setUrl
+		 ("http://aria2proxyuser:aria2proxypasswd@localhost:9000"));
+  httpRequest.setProxyRequest(proxyRequest);
 
   expectedText = "GET http://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -248,7 +256,7 @@ void HttpRequestTest::testCreateRequest()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  request->setKeepAlive(true);
+  request->setPipeliningHint(true);
 
   expectedText = "GET http://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -264,11 +272,10 @@ void HttpRequestTest::testCreateRequest()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  request->setKeepAlive(false);
+  request->setPipeliningHint(false);
 
-  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-
-  httpRequest.configure(&option);
+  // turn off proxy auth
+  CPPUNIT_ASSERT(proxyRequest->setUrl("http://localhost:9000"));
 
   expectedText = "GET http://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -286,48 +293,48 @@ void HttpRequestTest::testCreateRequest()
 
 void HttpRequestTest::testCreateRequest_ftp()
 {
-  Option option;
-  option.put(PREF_HTTP_AUTH_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
-  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-  option.put(PREF_HTTP_USER, "aria2user");
-  option.put(PREF_HTTP_PASSWD, "aria2passwd");
-  option.put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
-  option.put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
+   _option->put(PREF_FTP_USER, "aria2user");
+   _option->put(PREF_FTP_PASSWD, "aria2passwd");
 
-  AuthConfigFactoryHandle authConfigFactory = new AuthConfigFactory(&option);
-  AuthConfigFactorySingleton::instance(authConfigFactory);
-
-  RequestHandle request = new Request();
+  SharedHandle<Request> request(new Request());
   request->setUrl("ftp://localhost:8080/archives/aria2-1.0.0.tar.bz2");
 
+  SharedHandle<Request> proxyRequest(new Request());
+  CPPUNIT_ASSERT(proxyRequest->setUrl
+		 ("http://localhost:9000"));
+
   HttpRequest httpRequest;
-  SegmentHandle segment = new PiecedSegment(1024*1024, new Piece(0, 1024*1024));
+  SharedHandle<Piece> p(new Piece(0, 1024*1024));
+  SharedHandle<Segment> segment
+    (new PiecedSegment(1024*1024, p));
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
+  httpRequest.setProxyRequest(proxyRequest);
 
-  httpRequest.configure(&option);
-
-  string expectedText = "GET ftp://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+  std::string expectedText =
+    "GET ftp://aria2user@localhost:8080/archives/aria2-1.0.0.tar.bz2"
+    " HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
     "Connection: close\r\n"
+    "Proxy-Connection: close\r\n"
+    "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  // How to enable HTTP proxy authorization in FTP download via HTTP proxy
-  option.put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
-  option.put(PREF_HTTP_PROXY_METHOD, V_GET);
-  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
+  // test proxy authorization
+  CPPUNIT_ASSERT(proxyRequest->setUrl
+		 ("http://aria2proxyuser:aria2proxypasswd@localhost:9000"));
 
-  httpRequest.configure(&option);
-
-  expectedText = "GET ftp://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+  expectedText =
+    "GET ftp://aria2user@localhost:8080/archives/aria2-1.0.0.tar.bz2"
+    " HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
     "Host: localhost:8080\r\n"
@@ -336,34 +343,40 @@ void HttpRequestTest::testCreateRequest_ftp()
     "Connection: close\r\n"
     "Proxy-Connection: close\r\n"
     "Proxy-Authorization: Basic YXJpYTJwcm94eXVzZXI6YXJpYTJwcm94eXBhc3N3ZA==\r\n"
+    "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
-  
 }
 
 void HttpRequestTest::testCreateRequest_with_cookie()
 {
-  RequestHandle request = new Request();
+  SharedHandle<Request> request(new Request());
   request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
-  SegmentHandle segment = new PiecedSegment(1024*1024, new Piece(0, 1024*1024));
+  SharedHandle<Piece> p(new Piece(0, 1024*1024));
+  SharedHandle<Segment> segment
+    (new PiecedSegment(1024*1024, p));
 
   Cookie cookie1("name1", "value1", "/archives", "localhost", false);
   Cookie cookie2("name2", "value2", "/archives/download", "localhost", false);
-  Cookie cookie3("name3", "value3", "/archives/download", "tt.localhost", false);
-  Cookie cookie4("name4", "value4", "/archives/download", "tt.localhost", true);
+  Cookie cookie3("name3", "value3", "/archives/download", ".aria2.org", false);
+  Cookie cookie4("name4", "value4", "/archives/", ".aria2.org", true);
 
-  request->cookieBox->add(cookie1);
-  request->cookieBox->add(cookie2);
-  request->cookieBox->add(cookie3);
-  request->cookieBox->add(cookie4);
+  SharedHandle<CookieStorage> st(new CookieStorage());
+  CPPUNIT_ASSERT(st->store(cookie1));
+  CPPUNIT_ASSERT(st->store(cookie2));
+  CPPUNIT_ASSERT(st->store(cookie3));
+  CPPUNIT_ASSERT(st->store(cookie4));
 
   HttpRequest httpRequest;
 
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
+  httpRequest.setCookieStorage(st);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
 
-  string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+  std::string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
     "Host: localhost\r\n"
@@ -384,96 +397,171 @@ void HttpRequestTest::testCreateRequest_with_cookie()
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
     "Connection: close\r\n"
-    "Cookie: name1=value1;name2=value2;\r\n"
+    "Cookie: name2=value2;name1=value1;\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  request->setUrl("http://tt.localhost/archives/download/aria2-1.0.0.tar.bz2");
+  request->setUrl("http://www.aria2.org/archives/download/aria2-1.0.0.tar.bz2");
 
   expectedText = "GET /archives/download/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
-    "Host: tt.localhost\r\n"
+    "Host: www.aria2.org\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
     "Connection: close\r\n"
-    "Cookie: name1=value1;name2=value2;name3=value3;\r\n"
+    "Cookie: name3=value3;\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  request->setUrl("https://tt.localhost/archives/download/aria2-1.0.0.tar.bz2");
+  request->setUrl("https://www.aria2.org/archives/download/"
+		  "aria2-1.0.0.tar.bz2");
 
   expectedText = "GET /archives/download/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
-    "Host: tt.localhost\r\n"
+    "Host: www.aria2.org\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
     "Connection: close\r\n"
-    "Cookie: name1=value1;name2=value2;name3=value3;name4=value4;\r\n"
+    "Cookie: name3=value3;name4=value4;\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
   
 }
 
+void HttpRequestTest::testCreateRequest_query()
+{
+  SharedHandle<Request> request(new Request());
+  request->setUrl("http://localhost/wiki?id=9ad5109a-b8a5-4edf-9373-56a1c34ae138");
+  HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
+  httpRequest.setRequest(request);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
+
+  std::string expectedText =
+    "GET /wiki?id=9ad5109a-b8a5-4edf-9373-56a1c34ae138 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*\r\n"
+    "Host: localhost\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
+    "\r\n";
+  
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
+}
+
+void HttpRequestTest::testCreateRequest_head()
+{
+  SharedHandle<Request> request(new Request());
+  request->setMethod(Request::METHOD_HEAD);
+  request->setUrl("http://localhost/aria2-1.0.0.tar.bz2");
+
+  HttpRequest httpRequest;
+  httpRequest.setRequest(request);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
+  
+  std::stringstream result(httpRequest.createRequest());
+  std::string line;
+  CPPUNIT_ASSERT(getline(result, line));
+  Util::trimSelf(line);
+  CPPUNIT_ASSERT_EQUAL(std::string("HEAD /aria2-1.0.0.tar.bz2 HTTP/1.1"), line);
+}
+
 void HttpRequestTest::testCreateProxyRequest()
 {
-  RequestHandle request = new Request();
+  SharedHandle<Request> request(new Request());
   request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
-  SegmentHandle segment = new PiecedSegment(1024*1024, new Piece(0, 1024*1024));
+  SharedHandle<Piece> p(new Piece(0, 1024*1024));
+  SharedHandle<Segment> segment(new PiecedSegment(1024*1024, p));
+
+  SharedHandle<Request> proxyRequest(new Request());
+  CPPUNIT_ASSERT(proxyRequest->setUrl("http://localhost:9000"));
 
   HttpRequest httpRequest;
 
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
+  httpRequest.setProxyRequest(proxyRequest);
 
-  string expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
+  request->supportsPersistentConnection(true);
+
+  std::string expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Host: localhost:80\r\n"
-    "Proxy-Connection: close\r\n"
+    //"Proxy-Connection: close\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());
 
-  request->setKeepAlive(true);
+  // adds Keep-Alive header.
+  request->setKeepAliveHint(true);
 
   expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Host: localhost:80\r\n"
-    "Proxy-Connection: Keep-Alive\r\n"
+    //"Proxy-Connection: Keep-Alive\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());
+
+  request->setKeepAliveHint(false);
+  // pipelining also adds Keep-Alive header.
+  request->setPipeliningHint(true);
+
+  expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Host: localhost:80\r\n"
+    //"Proxy-Connection: Keep-Alive\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());
+
+  // test proxy authorization
+  CPPUNIT_ASSERT(proxyRequest->setUrl
+		 ("http://aria2proxyuser:aria2proxypasswd@localhost:9000"));
+
+  expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Host: localhost:80\r\n"
+    //"Proxy-Connection: Keep-Alive\r\n"
+    "Proxy-Authorization: Basic YXJpYTJwcm94eXVzZXI6YXJpYTJwcm94eXBhc3N3ZA==\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());
-  
 }
 
 void HttpRequestTest::testIsRangeSatisfied()
 {
-  RequestHandle request = new Request();
+  SharedHandle<Request> request(new Request());
+  request->supportsPersistentConnection(true);
   request->setUrl("http://localhost:8080/archives/aria2-1.0.0.tar.bz2");
-  request->setKeepAlive(false);
-  SegmentHandle segment = new PiecedSegment(1024*1024, new Piece(0, 1024*1024));
+  request->setPipeliningHint(false); // default: false
+  SharedHandle<Piece> p(new Piece(0, 1024*1024));
+  SharedHandle<Segment> segment(new PiecedSegment(1024*1024, p));
 
   HttpRequest httpRequest;
 
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
 
-  RangeHandle range = new Range(0, 0, 0);
+  SharedHandle<Range> range(new Range());
 
   CPPUNIT_ASSERT(httpRequest.isRangeSatisfied(range));
 
-  segment = new PiecedSegment(1024*1024, new Piece(1, 1024*1024));
+  p.reset(new Piece(1, 1024*1024));
+  segment.reset(new PiecedSegment(1024*1024, p));
   httpRequest.setSegment(segment);
 
   CPPUNIT_ASSERT(!httpRequest.isRangeSatisfied(range));
 
-  int64_t entityLength = segment->getSegmentLength()*10;
+  uint64_t entityLength = segment->getSegmentLength()*10;
 
-  range = new Range(segment->getPosition(), 0, entityLength);
+  range.reset(new Range(segment->getPosition(), 0, entityLength));
 
   CPPUNIT_ASSERT(httpRequest.isRangeSatisfied(range));
 
@@ -485,40 +573,38 @@ void HttpRequestTest::testIsRangeSatisfied()
 
   CPPUNIT_ASSERT(httpRequest.isRangeSatisfied(range));
 
-  request->setKeepAlive(true);
+  request->setPipeliningHint(true);
 
   CPPUNIT_ASSERT(!httpRequest.isRangeSatisfied(range));
 
-  range = new Range(segment->getPosition(),
-		    segment->getPosition()+segment->getLength()-1,
-		    entityLength);
+  range.reset(new Range(segment->getPosition(),
+			segment->getPosition()+segment->getLength()-1,
+			entityLength));
 
   CPPUNIT_ASSERT(httpRequest.isRangeSatisfied(range));
 
-  range = new Range(0, segment->getPosition()+segment->getLength()-1,
-		    entityLength);
+  range.reset(new Range(0, segment->getPosition()+segment->getLength()-1,
+			entityLength));
 
   CPPUNIT_ASSERT(!httpRequest.isRangeSatisfied(range));
 }
 
 void HttpRequestTest::testUserAgent()
 {
-  Option option;
-
-  AuthConfigFactoryHandle authConfigFactory = new AuthConfigFactory(&option);
-  AuthConfigFactorySingleton::instance(authConfigFactory);
-
-  RequestHandle request = new Request();
+  SharedHandle<Request> request(new Request());
   request->setUrl("http://localhost:8080/archives/aria2-1.0.0.tar.bz2");
 
-  SegmentHandle segment = new PiecedSegment(1024, new Piece(0, 1024));
+  SharedHandle<Piece> p(new Piece(0, 1024));
+  SharedHandle<Segment> segment(new PiecedSegment(1024, p));
 
   HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
   httpRequest.setUserAgent("aria2 (Linux)");
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
 
-  string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+  std::string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2 (Linux)\r\n"
     "Accept: */*\r\n"
     "Host: localhost:8080\r\n"
@@ -529,11 +615,102 @@ void HttpRequestTest::testUserAgent()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  string expectedTextForProxy = "CONNECT localhost:8080 HTTP/1.1\r\n"
+  SharedHandle<Request> proxyRequest(new Request());
+  CPPUNIT_ASSERT(proxyRequest->setUrl("http://localhost:9000"));
+  
+  httpRequest.setProxyRequest(proxyRequest);
+
+  std::string expectedTextForProxy = "CONNECT localhost:8080 HTTP/1.1\r\n"
     "User-Agent: aria2 (Linux)\r\n"
     "Host: localhost:8080\r\n"
-    "Proxy-Connection: close\r\n"
+    //"Proxy-Connection: close\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedTextForProxy, httpRequest.createProxyRequest());
 }
+
+void HttpRequestTest::testAddHeader()
+{
+  SharedHandle<Request> request(new Request());
+  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+
+  HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
+  httpRequest.setRequest(request);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
+  httpRequest.addHeader("X-ARIA2: v0.13\nX-ARIA2-DISTRIBUTE: enabled\n");
+
+  std::string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*\r\n"
+    "Host: localhost\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
+    "X-ARIA2: v0.13\r\n"
+    "X-ARIA2-DISTRIBUTE: enabled\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
+}
+
+void HttpRequestTest::testAddAcceptType()
+{
+  std::string acceptTypes[] = { "cream/custard",
+				"muffin/chocolate" };
+
+  SharedHandle<Request> request(new Request());
+  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+
+  HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
+  httpRequest.setRequest(request);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
+  httpRequest.addAcceptType(&acceptTypes[0],
+			    &acceptTypes[arrayLength(acceptTypes)]);
+
+  std::string expectedText =
+    "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*,cream/custard,muffin/chocolate\r\n"
+    "Host: localhost\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
+}
+
+void HttpRequestTest::testEnableAcceptEncoding()
+{
+  SharedHandle<Request> request(new Request());
+  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+
+  HttpRequest httpRequest;
+  httpRequest.setRequest(request);
+  httpRequest.setAuthConfigFactory(_authConfigFactory);
+
+  std::string acceptEncodings;
+#ifdef HAVE_LIBZ
+  acceptEncodings += "deflate, gzip";
+#endif // HAVE_LIBZ
+  
+  std::string expectedText =
+    "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*\r\n";
+  if(!acceptEncodings.empty()) {
+    expectedText += "Accept-Encoding: "+acceptEncodings+"\r\n";
+  }
+  expectedText +=
+    "Host: localhost\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
+}
+
+} // namespace aria2
