@@ -1,12 +1,18 @@
 #include "Util.h"
+
+#include <string>
+#include <iostream>
+
+#include <cppunit/extensions/HelperMacros.h>
+
 #include "FixedNumberRandomizer.h"
 #include "DlAbortEx.h"
 #include "BitfieldMan.h"
 #include "ByteArrayDiskWriter.h"
-#include <string>
-#include <cppunit/extensions/HelperMacros.h>
+#include "FileEntry.h"
+#include "File.h"
 
-using namespace std;
+namespace aria2 {
 
 class UtilTest:public CppUnit::TestFixture {
 
@@ -36,8 +42,15 @@ class UtilTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testParseIntRange);
   CPPUNIT_TEST(testParseIntRange_invalidRange);
   CPPUNIT_TEST(testParseInt);
+  CPPUNIT_TEST(testParseUInt);
   CPPUNIT_TEST(testParseLLInt);
+  CPPUNIT_TEST(testParseULLInt);
   CPPUNIT_TEST(testToString_binaryStream);
+  CPPUNIT_TEST(testItos);
+  CPPUNIT_TEST(testUitos);
+  CPPUNIT_TEST(testHttpGMT);
+  CPPUNIT_TEST(testNtoh64);
+  CPPUNIT_TEST(testUrlencode);
   CPPUNIT_TEST_SUITE_END();
 private:
 
@@ -70,76 +83,83 @@ public:
   void testParseIntRange();
   void testParseIntRange_invalidRange();
   void testParseInt();
+  void testParseUInt();
   void testParseLLInt();
+  void testParseULLInt();
   void testToString_binaryStream();
+  void testItos();
+  void testUitos();
+  void testHttpGMT();
+  void testNtoh64();
+  void testUrlencode();
 };
 
 
 CPPUNIT_TEST_SUITE_REGISTRATION( UtilTest );
 
 void UtilTest::testTrim() {
-  string str1 = "aria2";
+  std::string str1 = "aria2";
   CPPUNIT_ASSERT_EQUAL(str1, Util::trim("aria2"));
   CPPUNIT_ASSERT_EQUAL(str1, Util::trim(" aria2"));
   CPPUNIT_ASSERT_EQUAL(str1, Util::trim(" aria2 "));
   CPPUNIT_ASSERT_EQUAL(str1, Util::trim("  aria2  "));
-  string str2 = "aria2 debut";
+  std::string str2 = "aria2 debut";
   CPPUNIT_ASSERT_EQUAL(str2, Util::trim("aria2 debut"));
   CPPUNIT_ASSERT_EQUAL(str2, Util::trim(" aria2 debut "));
-  string str3 = "";
+  std::string str3 = "";
   CPPUNIT_ASSERT_EQUAL(str3, Util::trim(""));
   CPPUNIT_ASSERT_EQUAL(str3, Util::trim(" "));
   CPPUNIT_ASSERT_EQUAL(str3, Util::trim("  "));
-  string str4 = "A";
+  std::string str4 = "A";
   CPPUNIT_ASSERT_EQUAL(str4, Util::trim("A"));
   CPPUNIT_ASSERT_EQUAL(str4, Util::trim(" A "));
   CPPUNIT_ASSERT_EQUAL(str4, Util::trim("  A  "));
 }
 
 void UtilTest::testSplit() {
-  pair<string, string> p1;
+  std::pair<std::string, std::string> p1;
   Util::split(p1, "name=value", '=');
-  CPPUNIT_ASSERT_EQUAL(string("name"), p1.first);
-  CPPUNIT_ASSERT_EQUAL(string("value"), p1.second);
+  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
+  CPPUNIT_ASSERT_EQUAL(std::string("value"), p1.second);
   Util::split(p1, " name = value ", '=');
-  CPPUNIT_ASSERT_EQUAL(string("name"), p1.first);
-  CPPUNIT_ASSERT_EQUAL(string("value"), p1.second);
+  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
+  CPPUNIT_ASSERT_EQUAL(std::string("value"), p1.second);
   Util::split(p1, "=value", '=');
-  CPPUNIT_ASSERT_EQUAL(string(""), p1.first);
-  CPPUNIT_ASSERT_EQUAL(string("value"), p1.second);
+  CPPUNIT_ASSERT_EQUAL(std::string(""), p1.first);
+  CPPUNIT_ASSERT_EQUAL(std::string("value"), p1.second);
   Util::split(p1, "name=", '=');
-  CPPUNIT_ASSERT_EQUAL(string("name"), p1.first);
-  CPPUNIT_ASSERT_EQUAL(string(""), p1.second);
+  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
+  CPPUNIT_ASSERT_EQUAL(std::string(""), p1.second);
   Util::split(p1, "name", '=');
-  CPPUNIT_ASSERT_EQUAL(string("name"), p1.first);
-  CPPUNIT_ASSERT_EQUAL(string(""), p1.second);
+  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
+  CPPUNIT_ASSERT_EQUAL(std::string(""), p1.second);
 }
 
 void UtilTest::testSlice() {
-  Strings v1;
+  std::deque<std::string> v1;
   Util::slice(v1, "name1=value1; name2=value2; name3=value3;", ';', true);
   CPPUNIT_ASSERT_EQUAL(3, (int)v1.size());
   v1.clear();
   Util::slice(v1, "name1=value1; name2=value2; name3=value3", ';', true);
   CPPUNIT_ASSERT_EQUAL(3, (int)v1.size());
-  Strings::iterator itr = v1.begin();
-  CPPUNIT_ASSERT_EQUAL(string("name1=value1"), *itr++);
-  CPPUNIT_ASSERT_EQUAL(string("name2=value2"), *itr++);
-  CPPUNIT_ASSERT_EQUAL(string("name3=value3"), *itr++);
+  std::deque<std::string>::iterator itr = v1.begin();
+  CPPUNIT_ASSERT_EQUAL(std::string("name1=value1"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string("name2=value2"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string("name3=value3"), *itr++);
 
   v1.clear();
 
   Util::slice(v1, "name1=value1; name2=value2; name3=value3", ';', false);
   CPPUNIT_ASSERT_EQUAL(3, (int)v1.size());
   itr = v1.begin();
-  CPPUNIT_ASSERT_EQUAL(string("name1=value1"), *itr++);
-  CPPUNIT_ASSERT_EQUAL(string(" name2=value2"), *itr++);
-  CPPUNIT_ASSERT_EQUAL(string(" name3=value3"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string("name1=value1"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string(" name2=value2"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string(" name3=value3"), *itr++);
 }
 
 void UtilTest::testEndsWith() {
-  string target = "abcdefg";
-  string part = "fg";
+  std::string target = "abcdefg";
+  std::string part = "fg";
   CPPUNIT_ASSERT(Util::endsWith(target, part));
 
   target = "abdefg";
@@ -172,16 +192,16 @@ void UtilTest::testEndsWith() {
 }
 
 void UtilTest::testReplace() {
-  CPPUNIT_ASSERT_EQUAL(string("abc\n"), Util::replace("abc\r\n", "\r", ""));
-  CPPUNIT_ASSERT_EQUAL(string("abc"), Util::replace("abc\r\n", "\r\n", ""));
-  CPPUNIT_ASSERT_EQUAL(string(""), Util::replace("", "\r\n", ""));
-  CPPUNIT_ASSERT_EQUAL(string("abc"), Util::replace("abc", "", "a"));
-  CPPUNIT_ASSERT_EQUAL(string("xbc"), Util::replace("abc", "a", "x"));
+  CPPUNIT_ASSERT_EQUAL(std::string("abc\n"), Util::replace("abc\r\n", "\r", ""));
+  CPPUNIT_ASSERT_EQUAL(std::string("abc"), Util::replace("abc\r\n", "\r\n", ""));
+  CPPUNIT_ASSERT_EQUAL(std::string(""), Util::replace("", "\r\n", ""));
+  CPPUNIT_ASSERT_EQUAL(std::string("abc"), Util::replace("abc", "", "a"));
+  CPPUNIT_ASSERT_EQUAL(std::string("xbc"), Util::replace("abc", "a", "x"));
 }
 
 void UtilTest::testStartsWith() {
-  string target;
-  string part;
+  std::string target;
+  std::string part;
 
   target = "abcdefg";
   part = "abc";
@@ -214,38 +234,38 @@ void UtilTest::testStartsWith() {
 }
 
 void UtilTest::testGetContentDispositionFilename() {
-  string h1 = "attachment; filename=\"aria2.tar.bz2\"";
-  CPPUNIT_ASSERT_EQUAL(string("aria2.tar.bz2"), Util::getContentDispositionFilename(h1));
+  std::string h1 = "attachment; filename=\"aria2.tar.bz2\"";
+  CPPUNIT_ASSERT_EQUAL(std::string("aria2.tar.bz2"), Util::getContentDispositionFilename(h1));
 
-  string h2 = "attachment; filename=\"\"";
-  CPPUNIT_ASSERT_EQUAL(string(""), Util::getContentDispositionFilename(h2));
+  std::string h2 = "attachment; filename=\"\"";
+  CPPUNIT_ASSERT_EQUAL(std::string(""), Util::getContentDispositionFilename(h2));
 
-  string h3 = "attachment; filename=\"";
-  CPPUNIT_ASSERT_EQUAL(string(""), Util::getContentDispositionFilename(h3));
+  std::string h3 = "attachment; filename=\"";
+  CPPUNIT_ASSERT_EQUAL(std::string(""), Util::getContentDispositionFilename(h3));
 
-  string h4 = "attachment;";
-  CPPUNIT_ASSERT_EQUAL(string(""), Util::getContentDispositionFilename(h4));
+  std::string h4 = "attachment;";
+  CPPUNIT_ASSERT_EQUAL(std::string(""), Util::getContentDispositionFilename(h4));
 
-  string h5 = "attachment; filename=aria2.tar.bz2";
-  CPPUNIT_ASSERT_EQUAL(string("aria2.tar.bz2"), Util::getContentDispositionFilename(h5));
+  std::string h5 = "attachment; filename=aria2.tar.bz2";
+  CPPUNIT_ASSERT_EQUAL(std::string("aria2.tar.bz2"), Util::getContentDispositionFilename(h5));
 
-  string h6 = "attachment; filename='aria2.tar.bz2'";
-  CPPUNIT_ASSERT_EQUAL(string("aria2.tar.bz2"), Util::getContentDispositionFilename(h6));
+  std::string h6 = "attachment; filename='aria2.tar.bz2'";
+  CPPUNIT_ASSERT_EQUAL(std::string("aria2.tar.bz2"), Util::getContentDispositionFilename(h6));
 
-  string h7 = "attachment; filename='aria2.tar.bz2";
-  CPPUNIT_ASSERT_EQUAL(string("aria2.tar.bz2"), Util::getContentDispositionFilename(h7));
+  std::string h7 = "attachment; filename='aria2.tar.bz2";
+  CPPUNIT_ASSERT_EQUAL(std::string("aria2.tar.bz2"), Util::getContentDispositionFilename(h7));
 
-  string h8 = "attachment; filename=aria2.tar.bz2; creation-date=20 Jun 2007 00:00:00 GMT";
-  CPPUNIT_ASSERT_EQUAL(string("aria2.tar.bz2"), Util::getContentDispositionFilename(h8));
+  std::string h8 = "attachment; filename=aria2.tar.bz2; creation-date=20 Jun 2007 00:00:00 GMT";
+  CPPUNIT_ASSERT_EQUAL(std::string("aria2.tar.bz2"), Util::getContentDispositionFilename(h8));
 
-  string h9 = "attachment; filename=\"aria2.tar.bz2; creation-date=20 Jun 2007 00:00:00 GMT\"";
-  CPPUNIT_ASSERT_EQUAL(string("aria2.tar.bz2; creation-date=20 Jun 2007 00:00:00 GMT"), Util::getContentDispositionFilename(h9));
+  std::string h9 = "attachment; filename=\"aria2.tar.bz2; creation-date=20 Jun 2007 00:00:00 GMT\"";
+  CPPUNIT_ASSERT_EQUAL(std::string("aria2.tar.bz2; creation-date=20 Jun 2007 00:00:00 GMT"), Util::getContentDispositionFilename(h9));
 
-  string h10 = "attachment; filename=";
-  CPPUNIT_ASSERT_EQUAL(string(""), Util::getContentDispositionFilename(h10));
+  std::string h10 = "attachment; filename=";
+  CPPUNIT_ASSERT_EQUAL(std::string(""), Util::getContentDispositionFilename(h10));
 
-  string h11 = "attachment; filename=;";
-  CPPUNIT_ASSERT_EQUAL(string(""), Util::getContentDispositionFilename(h11));
+  std::string h11 = "attachment; filename=;";
+  CPPUNIT_ASSERT_EQUAL(std::string(""), Util::getContentDispositionFilename(h11));
 
 }
 
@@ -253,55 +273,54 @@ class Printer {
 public:
   template<class T>
   void operator()(T t) {
-    cerr << t << ", ";
+    std::cerr << t << ", ";
   }
 };
 
 void UtilTest::testRandomAlpha() {
-  string s = Util::randomAlpha(8, new FixedNumberRandomizer());
-  CPPUNIT_ASSERT_EQUAL(string("AAAAAAAA"), s);
+  SharedHandle<Randomizer> rand(new FixedNumberRandomizer());
+  std::string s = Util::randomAlpha(8, rand);
+  CPPUNIT_ASSERT_EQUAL(std::string("AAAAAAAA"), s);
 }
 
 void UtilTest::testToUpper() {
-  string src = "608cabc0f2fa18c260cafd974516865c772363d5";
-  string upp = "608CABC0F2FA18C260CAFD974516865C772363D5";
+  std::string src = "608cabc0f2fa18c260cafd974516865c772363d5";
+  std::string upp = "608CABC0F2FA18C260CAFD974516865C772363D5";
 
   CPPUNIT_ASSERT_EQUAL(upp, Util::toUpper(src));
 }
 
 void UtilTest::testToLower() {
-  string src = "608CABC0F2FA18C260CAFD974516865C772363D5";
-  string upp = "608cabc0f2fa18c260cafd974516865c772363d5";
+  std::string src = "608CABC0F2FA18C260CAFD974516865C772363D5";
+  std::string upp = "608cabc0f2fa18c260cafd974516865c772363d5";
 
   CPPUNIT_ASSERT_EQUAL(upp, Util::toLower(src));
 }
 
-#include "SharedHandle.h"
-
 void UtilTest::testUrldecode() {
-  string src = "http://aria2.sourceforge.net/aria2%200.7.0%20docs.html";
-  CPPUNIT_ASSERT_EQUAL(string("http://aria2.sourceforge.net/aria2 0.7.0 docs.html"),
+  std::string src = "http://aria2.sourceforge.net/aria2%200.7.0%20docs.html";
+  CPPUNIT_ASSERT_EQUAL(std::string("http://aria2.sourceforge.net/aria2 0.7.0 docs.html"),
 		       Util::urldecode(src));
 
-  string src2 = "aria2+aria2";
-  CPPUNIT_ASSERT_EQUAL(string("aria2+aria2"), Util::urldecode(src2));
+  std::string src2 = "aria2+aria2";
+  CPPUNIT_ASSERT_EQUAL(std::string("aria2+aria2"), Util::urldecode(src2));
 
-  string src3 = "%5t%20";
-  CPPUNIT_ASSERT_EQUAL(string("%5t "), Util::urldecode(src3));
+  std::string src3 = "%5t%20";
+  CPPUNIT_ASSERT_EQUAL(std::string("%5t "), Util::urldecode(src3));
 
-  string src4 = "%";
-  CPPUNIT_ASSERT_EQUAL(string("%"), Util::urldecode(src4));
+  std::string src4 = "%";
+  CPPUNIT_ASSERT_EQUAL(std::string("%"), Util::urldecode(src4));
   
-  string src5 = "%3";
-  CPPUNIT_ASSERT_EQUAL(string("%3"), Util::urldecode(src5));
+  std::string src5 = "%3";
+  CPPUNIT_ASSERT_EQUAL(std::string("%3"), Util::urldecode(src5));
 
-  string src6 = "%2f";
-  CPPUNIT_ASSERT_EQUAL(string("/"), Util::urldecode(src6));
+  std::string src6 = "%2f";
+  CPPUNIT_ASSERT_EQUAL(std::string("/"), Util::urldecode(src6));
 }
 
 void UtilTest::testCountBit() {
-  CPPUNIT_ASSERT_EQUAL((int32_t)32, Util::countBit(UINT32_MAX));
-  CPPUNIT_ASSERT_EQUAL((int32_t)8, Util::countBit(255));
+  CPPUNIT_ASSERT_EQUAL((unsigned int)32, Util::countBit(UINT32_MAX));
+  CPPUNIT_ASSERT_EQUAL((unsigned int)8, Util::countBit(255));
 }
 
 void UtilTest::testGetRealSize()
@@ -311,62 +330,57 @@ void UtilTest::testGetRealSize()
   try {
     Util::getRealSize("");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     Util::getRealSize("foo");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     Util::getRealSize("-1");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     Util::getRealSize("9223372036854775807K");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     Util::getRealSize("9223372036854775807M");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
 }
 
 void UtilTest::testAbbrevSize()
 {
-  CPPUNIT_ASSERT_EQUAL(string("4,096.0Mi"), Util::abbrevSize(4294967296LL));
-  CPPUNIT_ASSERT_EQUAL(string("1.0Ki"), Util::abbrevSize(1024));
-  CPPUNIT_ASSERT_EQUAL(string("1,023"), Util::abbrevSize(1023));
-  CPPUNIT_ASSERT_EQUAL(string("0"), Util::abbrevSize(0));
-  CPPUNIT_ASSERT_EQUAL(string("1.1Ki"), Util::abbrevSize(1127));
-  CPPUNIT_ASSERT_EQUAL(string("1.5Mi"), Util::abbrevSize(1572864));
+  CPPUNIT_ASSERT_EQUAL(std::string("4,096.0Mi"), Util::abbrevSize(4294967296LL));
+  CPPUNIT_ASSERT_EQUAL(std::string("1.0Ki"), Util::abbrevSize(1024));
+  CPPUNIT_ASSERT_EQUAL(std::string("1,023"), Util::abbrevSize(1023));
+  CPPUNIT_ASSERT_EQUAL(std::string("0"), Util::abbrevSize(0));
+  CPPUNIT_ASSERT_EQUAL(std::string("1.1Ki"), Util::abbrevSize(1127));
+  CPPUNIT_ASSERT_EQUAL(std::string("1.5Mi"), Util::abbrevSize(1572864));
 
 }
 
 void UtilTest::testToStream()
 {
-  ostringstream os;
-  FileEntryHandle f1 = new FileEntry("aria2.tar.bz2", 12300, 0);
-  FileEntryHandle f2 = new FileEntry("aria2.txt", 556, 0);
-  FileEntries entries;
+  std::ostringstream os;
+  SharedHandle<FileEntry> f1(new FileEntry("aria2.tar.bz2", 12300, 0));
+  SharedHandle<FileEntry> f2(new FileEntry("aria2.txt", 556, 0));
+  std::deque<SharedHandle<FileEntry> > entries;
   entries.push_back(f1);
   entries.push_back(f2);
   Util::toStream(os, entries);
   CPPUNIT_ASSERT_EQUAL(
-		       string("Files:\n"
+		       std::string("Files:\n"
 			      "idx|path/length\n"
 			      "===+===========================================================================\n"
 			      "  1|aria2.tar.bz2\n"
@@ -407,17 +421,19 @@ void UtilTest::testIsUppercase()
 
 void UtilTest::testAlphaToNum()
 {
-  CPPUNIT_ASSERT_EQUAL((int32_t)0, Util::alphaToNum("a"));
-  CPPUNIT_ASSERT_EQUAL((int32_t)0, Util::alphaToNum("aa"));
-  CPPUNIT_ASSERT_EQUAL((int32_t)1, Util::alphaToNum("b"));
-  CPPUNIT_ASSERT_EQUAL((int32_t)675, Util::alphaToNum("zz")); // 25*26+25
-  CPPUNIT_ASSERT_EQUAL((int32_t)675, Util::alphaToNum("ZZ")); // 25*26+25
-  CPPUNIT_ASSERT_EQUAL((int32_t)0, Util::alphaToNum(""));
+  CPPUNIT_ASSERT_EQUAL(0U, Util::alphaToNum("a"));
+  CPPUNIT_ASSERT_EQUAL(0U, Util::alphaToNum("aa"));
+  CPPUNIT_ASSERT_EQUAL(1U, Util::alphaToNum("b"));
+  CPPUNIT_ASSERT_EQUAL(675U, Util::alphaToNum("zz")); // 25*26+25
+  CPPUNIT_ASSERT_EQUAL(675U, Util::alphaToNum("ZZ")); // 25*26+25
+  CPPUNIT_ASSERT_EQUAL(0U, Util::alphaToNum(""));
+  CPPUNIT_ASSERT_EQUAL(4294967295U, Util::alphaToNum("NXMRLXV"));
+  CPPUNIT_ASSERT_EQUAL(0U, Util::alphaToNum("NXMRLXW")); // uint32_t overflow
 }
 
 void UtilTest::testMkdirs()
 {
-  string dir = "/tmp/aria2-UtilTest-testMkdirs";
+  std::string dir = "/tmp/aria2-UtilTest-testMkdirs";
   File d(dir);
   if(d.exists()) {
     CPPUNIT_ASSERT(d.remove());
@@ -426,15 +442,14 @@ void UtilTest::testMkdirs()
   Util::mkdirs(dir);
   CPPUNIT_ASSERT(d.isDir());
 
-  string file = "./UtilTest.cc";
+  std::string file = "./UtilTest.cc";
   File f(file);
   CPPUNIT_ASSERT(f.isFile());
   try {
     Util::mkdirs(file);
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(DlAbortEx* ex) {
-    cerr << ex->getMsg() << endl;
-    delete ex;
+  } catch(DlAbortEx& ex) {
+    std::cerr << ex.stackTrace() << std::endl;
   }
 }
 
@@ -447,7 +462,7 @@ void UtilTest::testConvertBitfield()
   // which corresponds to the index [1,2] in destBitfield
   Util::convertBitfield(&destBitfield, &srcBitfield);
   
-  CPPUNIT_ASSERT_EQUAL(string("9fffffffffffffffffffffffffffffff80"),
+  CPPUNIT_ASSERT_EQUAL(std::string("9fffffffffffffffffffffffffffffff80"),
 		       Util::toHex(destBitfield.getBitfield(),
 				   destBitfield.getBitfieldLength()));
 }
@@ -481,117 +496,230 @@ void UtilTest::testParseIntRange_invalidRange()
   try {
     IntSequence seq = Util::parseIntRange("-1");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     IntSequence seq = Util::parseIntRange("2147483648");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     IntSequence seq = Util::parseIntRange("2147483647-2147483648");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     IntSequence seq = Util::parseIntRange("1-2x");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     IntSequence seq = Util::parseIntRange("3x-4");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
 }
 
 void UtilTest::testParseInt()
 {
-  CPPUNIT_ASSERT_EQUAL((int32_t)-1, Util::parseInt(" -1 "));
-  CPPUNIT_ASSERT_EQUAL((int32_t)2147483647, Util::parseInt("2147483647"));
+  CPPUNIT_ASSERT_EQUAL(-1, Util::parseInt(" -1 "));
+  CPPUNIT_ASSERT_EQUAL(2147483647, Util::parseInt("2147483647"));
   try {
     Util::parseInt("2147483648");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     Util::parseInt("-2147483649");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     Util::parseInt("12x");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     Util::parseInt("");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
+  }
+}
+
+void UtilTest::testParseUInt()
+{
+  CPPUNIT_ASSERT_EQUAL(4294967295U, Util::parseUInt(" 4294967295 "));
+  try {
+    Util::parseUInt("-1");
+    CPPUNIT_FAIL("exception must be thrown.");
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
+  }
+  try {
+    Util::parseUInt("4294967296");
+    CPPUNIT_FAIL("exception must be thrown.");
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
 }
 
 void UtilTest::testParseLLInt()
 {
-  CPPUNIT_ASSERT_EQUAL((int64_t)-1, Util::parseLLInt(" -1 "));
+  CPPUNIT_ASSERT_EQUAL((int64_t)-1LL, Util::parseLLInt(" -1 "));
   CPPUNIT_ASSERT_EQUAL((int64_t)9223372036854775807LL,
 		       Util::parseLLInt("9223372036854775807"));
   try {
     Util::parseLLInt("9223372036854775808");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     Util::parseLLInt("-9223372036854775809");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     Util::parseLLInt("12x");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
   try {
     Util::parseLLInt("");
     CPPUNIT_FAIL("exception must be thrown.");
-  } catch(Exception* e) {
-    cerr << *e;
-    delete e;
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
+  }
+}
+
+void UtilTest::testParseULLInt()
+{
+  CPPUNIT_ASSERT_EQUAL((uint64_t)18446744073709551615ULL,
+		       Util::parseULLInt("18446744073709551615"));
+  try {
+    Util::parseUInt("-1");
+    CPPUNIT_FAIL("exception must be thrown.");
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
+  }
+  try {
+    Util::parseLLInt("18446744073709551616");
+    CPPUNIT_FAIL("exception must be thrown.");
+  } catch(Exception& e) {
+    std::cerr << e.stackTrace();
   }
 }
 
 void UtilTest::testToString_binaryStream()
 {
-  DiskWriterHandle dw = new ByteArrayDiskWriter();
-  string data = string(16*1024+256, 'a');
+  SharedHandle<DiskWriter> dw(new ByteArrayDiskWriter());
+  std::string data(16*1024+256, 'a');
   dw->initAndOpenFile("dummy");
   dw->writeData((const unsigned char*)data.c_str(), data.size(), 0);
 
-  string readData = Util::toString(dw);
+  std::string readData = Util::toString(dw);
 
   CPPUNIT_ASSERT_EQUAL(data, readData);
 }
+
+void UtilTest::testItos()
+{
+  {
+    int i = 0;
+    CPPUNIT_ASSERT_EQUAL(std::string("0"), Util::itos(i));
+  }
+  {
+    int i = 100;
+    CPPUNIT_ASSERT_EQUAL(std::string("100"), Util::itos(i, true));
+  }
+  {
+    int i = 100;
+    CPPUNIT_ASSERT_EQUAL(std::string("100"), Util::itos(i));
+  }
+  {
+    int i = 12345;
+    CPPUNIT_ASSERT_EQUAL(std::string("12,345"), Util::itos(i, true));
+  }
+  {
+    int i = 12345;
+    CPPUNIT_ASSERT_EQUAL(std::string("12345"), Util::itos(i));
+  }
+  {
+    int i = -12345;
+    CPPUNIT_ASSERT_EQUAL(std::string("-12,345"), Util::itos(i, true));
+  }
+  {
+    int64_t i = INT64_MAX;
+    CPPUNIT_ASSERT_EQUAL(std::string("9,223,372,036,854,775,807"), Util::itos(i, true));
+  }  
+}
+
+void UtilTest::testUitos()
+{
+  {
+    uint16_t i = 12345;
+    CPPUNIT_ASSERT_EQUAL(std::string("12345"), Util::uitos(i));
+  }
+  {
+    int16_t i = -12345;
+    CPPUNIT_ASSERT_EQUAL(std::string("/.-,+"), Util::uitos(i));
+  }
+}
+
+void UtilTest::testHttpGMT()
+{
+  CPPUNIT_ASSERT_EQUAL((time_t)0, Util::httpGMT("Thu, 1970-01-01 0:0:0 GMT"));
+  CPPUNIT_ASSERT_EQUAL((time_t)2147483647,
+		       Util::httpGMT("Tue, 2038-01-19 3:14:7 GMT"));
+  if(sizeof(time_t) == 4) {
+    CPPUNIT_ASSERT_EQUAL((time_t)2147483647,
+			 Util::httpGMT("Tue, 2038-01-19 3:14:8 GMT"));
+  } else if(sizeof(time_t) == 8) {
+    CPPUNIT_ASSERT_EQUAL((int64_t)2147483648LL,
+			 (int64_t)Util::httpGMT("Tue, 2038-01-19 3:14:8 GMT"));
+  }
+  CPPUNIT_ASSERT_EQUAL((time_t)-1,
+		       Util::httpGMT("Tue, 2008/10/10 23:33:33 UTC"));
+}
+
+void UtilTest::testNtoh64()
+{
+  uint64_t x = 0xff00ff00ee00ee00LL;
+#ifdef WORDS_BIGENDIAN
+  CPPUNIT_ASSERT_EQUAL(x, ntoh64(x));
+  CPPUNIT_ASSERT_EQUAL(x, hton64(x));
+#else // !WORDS_BIGENDIAN
+  uint64_t y = 0x00ee00ee00ff00ffLL;
+  CPPUNIT_ASSERT_EQUAL(y, ntoh64(x));
+  CPPUNIT_ASSERT_EQUAL(x, hton64(y));
+#endif // !WORDS_BIGENDIAN
+}
+
+void UtilTest::testUrlencode()
+{
+  CPPUNIT_ASSERT_EQUAL
+    (std::string("%3A%2F%3F%23%5B%5D%40%21%25%26%27%28%29%2A%2B%2C%3B%3D"),
+     Util::urlencode(":/?#[]@!%&'()*+,;="));
+
+  std::string unreserved =
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+    "abcdefghijklmnopqrstuvwxyz"
+    "0123456789"
+    "-._~";
+  CPPUNIT_ASSERT_EQUAL(unreserved, Util::urlencode(unreserved));
+
+  CPPUNIT_ASSERT_EQUAL(std::string("1%5EA%20"), Util::urlencode("1^A "));
+}
+
+} // namespace aria2
